@@ -18,16 +18,19 @@ mkdir Output/$TEST_UID 2> /dev/null
 cd Output/$TEST_UID
 rm -rf *
 
-# Copy all of the test input files into the current directory...
+# Copy all of the test input files into the current directory.
 cp -r $INPUT_DIR/* .
 
-# If there is an "all" input directory, take it as well...
+# If there is an "all" input directory, take it as well.
 cp -r $INPUT_DIR/../../all/input/* . > /dev/null 2>&1 
 
-# Run the program now...
+# If any of the input files is a .bz2 file, uncompress it now.
+bunzip2 *.bz2 > /dev/null 2>&1
+
+# Run the program now.
 echo Running: $*
 $*
 
-# This script is always successful...
+# This script is always successful.
 exit 0
 
