@@ -14,7 +14,7 @@ int main() { return 0; }
 
 #define I 1.0iF
 
-#if defined(__FreeBSD__)
+#if defined(__FreeBSD__) || defined(__OpenBSD__)
 #include <complex.h>
 #elif defined(__APPLE__)
 #include <math.h>
