@@ -1,10 +1,10 @@
 /* -*- mode: c -*-
- * $Id: fib2.c,v 1.2 2004/06/08 17:21:42 lattner Exp $
+ * $Id: fib2.c,v 1.4 2006/05/25 18:31:14 lattner Exp $
  * http://www.bagley.org/~doug/shootout/
  */
 
 int atoi(char *);
-void printf(char *, unsigned long);
+#include <stdio.h>
 
 unsigned long
 fib(unsigned long n) {
