@@ -2,7 +2,7 @@
 
 void print(float X) {
   printf("%f\n", X*2147483647);
-  printf("%x\n", X*2147483647);
+  printf("%x\n", (unsigned)(X*2147483647));
 }
 
 int main() {
