@@ -2,9 +2,10 @@
 
 int puts(const char *);
 
-void main(int argc, char **argv) {
+int main(int argc, char **argv) {
 	int i;
 	printf("#Args = %d. They are:\n", argc-1);
 	for (i = 1; i < argc; ++i)
 		puts(argv[i]);
+        return 0;
 }
