@@ -1,5 +1,5 @@
 /* C/ObjC/C++ command line option handling.
-   Copyright (C) 2002, 2003, 2004, 2005 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2003 Free Software Foundation, Inc.
    Contributed by Neil Booth.
 
 This file is part of GCC.
@@ -37,7 +37,6 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 #include "debug.h"		/* For debug_hooks.  */
 #include "opts.h"
 #include "options.h"
-#include "mkdeps.h"
 
 #ifndef DOLLARS_IN_IDENTIFIERS
 # define DOLLARS_IN_IDENTIFIERS true
@@ -47,9 +46,7 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 # define TARGET_SYSTEM_ROOT NULL
 #endif
 
-#ifndef TARGET_OPTF
-#define TARGET_OPTF(ARG)
-#endif
+static int saved_lineno;
 
 /* CPP's options.  */
 static cpp_options *cpp_opts;
@@ -91,15 +88,15 @@ static bool quote_chain_split;
 /* If -Wunused-macros.  */
 static bool warn_unused_macros;
 
-/* If -Wvariadic-macros.  */
-static bool warn_variadic_macros = true;
-
 /* Number of deferred options.  */
 static size_t deferred_count;
 
 /* Number of deferred options scanned for -include.  */
 static size_t include_cursor;
 
+/* Permit Fotran front-end options.  */
+static bool permit_fortran_options;
+
 static void set_Wimplicit (int);
 static void handle_OPT_d (const char *);
 static void set_std_cxx98 (int);
@@ -111,8 +108,7 @@ static void sanitize_cpp_opts (void);
 static void add_prefixed_path (const char *, size_t);
 static void push_command_line_include (void);
 static void cb_file_change (cpp_reader *, const struct line_map *);
-static void cb_dir_change (cpp_reader *, const char *);
-static void finish_options (void);
+static void finish_options (const char *);
 
 #ifndef STDC_0_IN_SYSTEM_HEADERS
 #define STDC_0_IN_SYSTEM_HEADERS 0
@@ -141,25 +137,23 @@ c_common_missing_argument (const char *opt, size_t code)
       return false;
 
     case OPT_fconstant_string_class_:
-      error ("no class name specified with %qs", opt);
+      error ("no class name specified with \"%s\"", opt);
       break;
 
     case OPT_A:
-      error ("assertion missing after %qs", opt);
+      error ("assertion missing after \"%s\"", opt);
       break;
 
     case OPT_D:
     case OPT_U:
-      error ("macro name missing after %qs", opt);
+      error ("macro name missing after \"%s\"", opt);
       break;
 
-    case OPT_F:
     case OPT_I:
     case OPT_idirafter:
     case OPT_isysroot:
     case OPT_isystem:
-    case OPT_iquote:
-      error ("missing path after %qs", opt);
+      error ("missing path after \"%s\"", opt);
       break;
 
     case OPT_MF:
@@ -168,12 +162,12 @@ c_common_missing_argument (const char *opt, size_t code)
     case OPT_include:
     case OPT_imacros:
     case OPT_o:
-      error ("missing filename after %qs", opt);
+      error ("missing filename after \"%s\"", opt);
       break;
 
     case OPT_MQ:
     case OPT_MT:
-      error ("missing makefile target after %qs", opt);
+      error ("missing makefile target after \"%s\"", opt);
       break;
     }
 
@@ -191,10 +185,10 @@ defer_opt (enum opt_code code, const char *arg)
 
 /* Common initialization before parsing options.  */
 unsigned int
-c_common_init_options (unsigned int argc, const char **argv)
+c_common_init_options (unsigned int argc, const char **argv ATTRIBUTE_UNUSED)
 {
   static const unsigned int lang_flags[] = {CL_C, CL_ObjC, CL_CXX, CL_ObjCXX};
-  unsigned int i, result;
+  unsigned int result;
 
   /* This is conditionalized only because that is the way the front
      ends used to do it.  Maybe this should be unconditional?  */
@@ -209,7 +203,7 @@ c_common_init_options (unsigned int argc, const char **argv)
     }
 
   parse_in = cpp_create_reader (c_dialect_cxx () ? CLK_GNUCXX: CLK_GNUC89,
-				ident_hash, &line_table);
+				ident_hash);
 
   cpp_opts = cpp_get_options (parse_in);
   cpp_opts->dollars_in_ident = DOLLARS_IN_IDENTIFIERS;
@@ -219,32 +213,24 @@ c_common_init_options (unsigned int argc, const char **argv)
      before passing on command-line options to cpplib.  */
   cpp_opts->warn_dollars = 0;
 
-  /* APPLE LOCAL begin -Wfour-char-constants  */
-#ifdef WARN_FOUR_CHAR_CONSTANTS
-  /* Warn about 4-char constants everywhere except on Macs.  */
-  cpp_opts->warn_four_char_constants = WARN_FOUR_CHAR_CONSTANTS;
-#endif
-  /* APPLE LOCAL end -Wfour-char-constants  */
-
   flag_const_strings = c_dialect_cxx ();
   flag_exceptions = c_dialect_cxx ();
   warn_pointer_arith = c_dialect_cxx ();
 
-  deferred_opts = XNEWVEC (struct deferred_opt, argc);
+  deferred_opts = xmalloc (argc * sizeof (struct deferred_opt));
 
   result = lang_flags[c_language];
 
-  if (c_language == clk_c)
+  /* If potentially preprocessing Fortran we have to accept its front
+     end options since the driver passes most of them through.  */
+#ifdef CL_F77
+  if (c_language == clk_c && argc > 2
+      && !strcmp (argv[2], "-traditional-cpp" ))
     {
-      /* If preprocessing assembly language, accept any of the C-family
-	 front end options since the driver may pass them through.  */
-      for (i = 1; i < argc; i++)
-	if (! strcmp (argv[i], "-lang-asm"))
-	  {
-	    result |= CL_C | CL_ObjC | CL_CXX | CL_ObjCXX;
-	    break;
-	  }
+      permit_fortran_options = true;
+      result |= CL_F77;
     }
+#endif
 
   return result;
 }
@@ -263,21 +249,11 @@ c_common_handle_option (size_t scode, const char *arg, int value)
   switch (code)
     {
     default:
-      if (cl_options[code].flags & (CL_C | CL_CXX | CL_ObjC | CL_ObjCXX))
-	break;
-      result = 0;
+      result = permit_fortran_options;
       break;
 
-      /* APPLE LOCAL begin ss2 */
-    case OPT_fsave_repository_:
-      flag_save_repository = 1;
-      break;
-      /* APPLE LOCAL end ss2 */
-
     case OPT__output_pch_:
       pch_file = arg;
-      /* APPLE LOCAL ss2 */
-      flag_pch_file = 1;
       break;
 
     case OPT_A:
@@ -305,20 +281,15 @@ c_common_handle_option (size_t scode, const char *arg, int value)
       cpp_opts->print_include_names = 1;
       break;
 
-    case OPT_F:
-      TARGET_OPTF (xstrdup (arg));
-      break;
-
     case OPT_I:
       if (strcmp (arg, "-"))
-	add_path (xstrdup (arg), BRACKET, 0, true);
+	add_path (xstrdup (arg), BRACKET, 0);
       else
 	{
 	  if (quote_chain_split)
 	    error ("-I- specified twice");
 	  quote_chain_split = true;
 	  split_quote_chain ();
-	  inform ("obsolete option -I- used, please use -iquote instead");
 	}
       break;
 
@@ -364,12 +335,6 @@ c_common_handle_option (size_t scode, const char *arg, int value)
       flag_no_line_commands = 1;
       break;
 
-      /* APPLE LOCAL begin -Wno-#warnings */
-    case OPT_W_warnings:
-      cpp_opts->no_pound_warnings = !value;
-      break;
-      /* APPLE LOCAL end -Wno-#warnings */
-
     case OPT_fworking_directory:
       flag_working_directory = value;
       break;
@@ -378,18 +343,17 @@ c_common_handle_option (size_t scode, const char *arg, int value)
       defer_opt (code, arg);
       break;
 
+    case OPT_Wabi:
+      warn_abi = value;
+      break;
+
     case OPT_Wall:
-      /* APPLE LOCAL -Wmost */
-    case OPT_Wmost:
       set_Wunused (value);
       set_Wformat (value);
       set_Wimplicit (value);
       warn_char_subscripts = value;
       warn_missing_braces = value;
-      /* APPLE LOCAL begin -Wmost --dpatel */
-      if (code != OPT_Wmost) 
-	warn_parentheses = value;
-      /* APPLE LOCAL end -Wmost --dpatel */
+      warn_parentheses = value;
       warn_return_type = value;
       warn_sequence_point = value;	/* Was C only.  */
       if (c_dialect_cxx ())
@@ -425,12 +389,37 @@ c_common_handle_option (size_t scode, const char *arg, int value)
       cpp_opts->warn_multichar = value;	/* Was C++ only.  */
       break;
 
+    case OPT_Wbad_function_cast:
+      warn_bad_function_cast = value;
+      break;
+
+    case OPT_Wcast_qual:
+      warn_cast_qual = value;
+      break;
+
+    case OPT_Wchar_subscripts:
+      warn_char_subscripts = value;
+      break;
+
     case OPT_Wcomment:
     case OPT_Wcomments:
       cpp_opts->warn_comments = value;
       break;
 
+    case OPT_Wconversion:
+      warn_conversion = value;
+      break;
+
+    case OPT_Wctor_dtor_privacy:
+      warn_ctor_dtor_privacy = value;
+      break;
+
+    case OPT_Wdeclaration_after_statement:
+      warn_declaration_after_statement = value;
+      break;
+
     case OPT_Wdeprecated:
+      warn_deprecated = value;
       cpp_opts->warn_deprecated = value;
       break;
 
@@ -438,25 +427,26 @@ c_common_handle_option (size_t scode, const char *arg, int value)
       warn_div_by_zero = value;
       break;
 
+    case OPT_Weffc__:
+      warn_ecpp = value;
+      break;
+
     case OPT_Wendif_labels:
       cpp_opts->warn_endif_labels = value;
       break;
 
     case OPT_Werror:
       cpp_opts->warnings_are_errors = value;
-      global_dc->warning_as_error_requested = value;
       break;
 
-      /* APPLE LOCAL begin -Wextra-tokens */
-    case OPT_Wextra_tokens:
-      cpp_opts->warn_extra_tokens = value;
-      break;
-      /* APPLE LOCAL end -Wextra-tokens */
-
     case OPT_Werror_implicit_function_declaration:
       mesg_implicit_function_declaration = 2;
       break;
 
+    case OPT_Wfloat_equal:
+      warn_float_equal = value;
+      break;
+
     case OPT_Wformat:
       set_Wformat (value);
       break;
@@ -465,24 +455,58 @@ c_common_handle_option (size_t scode, const char *arg, int value)
       set_Wformat (atoi (arg));
       break;
 
-      /* APPLE LOCAL begin -Wfour-char-constants */
-    case OPT_Wfour_char_constants:
-      cpp_opts->warn_four_char_constants = value;
+    case OPT_Wformat_extra_args:
+      warn_format_extra_args = value;
+      break;
+
+    case OPT_Wformat_nonliteral:
+      warn_format_nonliteral = value;
+      break;
+
+    case OPT_Wformat_security:
+      warn_format_security = value;
+      break;
+
+    case OPT_Wformat_y2k:
+      warn_format_y2k = value;
+      break;
+
+    case OPT_Wformat_zero_length:
+      warn_format_zero_length = value;
+      break;
+
+    case OPT_Winit_self:
+      warn_init_self = value;
       break;
-      /* APPLE LOCAL end -Wfour-char-constants */
 
     case OPT_Wimplicit:
       set_Wimplicit (value);
       break;
 
+    case OPT_Wimplicit_function_declaration:
+      mesg_implicit_function_declaration = value;
+      break;
+
+    case OPT_Wimplicit_int:
+      warn_implicit_int = value;
+      break;
+
     case OPT_Wimport:
       /* Silently ignore for now.  */
       break;
 
+    case OPT_Winvalid_offsetof:
+      warn_invalid_offsetof = value;
+      break;
+
     case OPT_Winvalid_pch:
       cpp_opts->warn_invalid_pch = value;
       break;
 
+    case OPT_Wlong_long:
+      warn_long_long = value;
+      break;
+
     case OPT_Wmain:
       if (value)
 	warn_main = 1;
@@ -490,41 +514,104 @@ c_common_handle_option (size_t scode, const char *arg, int value)
 	warn_main = -1;
       break;
 
-    case OPT_Wmissing_include_dirs:
-      cpp_opts->warn_missing_include_dirs = value;
+    case OPT_Wmissing_braces:
+      warn_missing_braces = value;
+      break;
+
+    case OPT_Wmissing_declarations:
+      warn_missing_declarations = value;
+      break;
+
+    case OPT_Wmissing_format_attribute:
+      warn_missing_format_attribute = value;
+      break;
+
+    case OPT_Wmissing_prototypes:
+      warn_missing_prototypes = value;
       break;
 
     case OPT_Wmultichar:
       cpp_opts->warn_multichar = value;
       break;
 
-      /* APPLE LOCAL begin -Wnewline-eof */
-    case OPT_Wnewline_eof:
-      cpp_opts->warn_newline_at_eof = value;
-      break;
-      /* APPLE LOCAL end -Wnewline-eof */
-
-/* APPLE LOCAL begin mainline UCNs 2005-04-17 3892809 */
-    case OPT_Wnormalized_:
-      if (!value || (arg && strcasecmp (arg, "none") == 0))
-	cpp_opts->warn_normalize = normalized_none;
-      else if (!arg || strcasecmp (arg, "nfkc") == 0)
-	cpp_opts->warn_normalize = normalized_KC;
-      else if (strcasecmp (arg, "id") == 0)
-	cpp_opts->warn_normalize = normalized_identifier_C;
-      else if (strcasecmp (arg, "nfc") == 0)
-	cpp_opts->warn_normalize = normalized_C;
-      else
-	error ("argument %qs to %<-Wnormalized%> not recognized", arg);
+    case OPT_Wnested_externs:
+      warn_nested_externs = value;
+      break;
+
+    case OPT_Wnon_template_friend:
+      warn_nontemplate_friend = value;
+      break;
+
+    case OPT_Wnon_virtual_dtor:
+      warn_nonvdtor = value;
+      break;
+
+    case OPT_Wnonnull:
+      warn_nonnull = value;
+      break;
+
+    case OPT_Wold_style_definition:
+      warn_old_style_definition = value;
+      break;
+
+    case OPT_Wold_style_cast:
+      warn_old_style_cast = value;
+      break;
+
+    case OPT_Woverloaded_virtual:
+      warn_overloaded_virtual = value;
+      break;
+
+    case OPT_Wparentheses:
+      warn_parentheses = value;
+      break;
+
+    case OPT_Wpmf_conversions:
+      warn_pmf2ptr = value;
+      break;
+
+    case OPT_Wpointer_arith:
+      warn_pointer_arith = value;
+      break;
+
+    case OPT_Wprotocol:
+      warn_protocol = value;
+      break;
+
+    case OPT_Wselector:
+      warn_selector = value;
+      break;
+
+    case OPT_Wredundant_decls:
+      warn_redundant_decls = value;
+      break;
+
+    case OPT_Wreorder:
+      warn_reorder = value;
       break;
 
-/* APPLE LOCAL end mainline UCNs 2005-04-17 3892809 */
     case OPT_Wreturn_type:
       warn_return_type = value;
       break;
 
-    case OPT_Wstrict_null_sentinel:
-      warn_strict_null_sentinel = value;
+    case OPT_Wsequence_point:
+      warn_sequence_point = value;
+      break;
+
+    case OPT_Wsign_compare:
+      warn_sign_compare = value;
+      break;
+
+    case OPT_Wsign_promo:
+      warn_sign_promo = value;
+      break;
+
+    case OPT_Wstrict_prototypes:
+      warn_strict_prototypes = value;
+      break;
+
+    case OPT_Wsynth:
+      warn_synth = value;
       break;
 
     case OPT_Wsystem_headers:
@@ -532,6 +619,7 @@ c_common_handle_option (size_t scode, const char *arg, int value)
       break;
 
     case OPT_Wtraditional:
+      warn_traditional = value;
       cpp_opts->warn_traditional = value;
       break;
 
@@ -539,6 +627,10 @@ c_common_handle_option (size_t scode, const char *arg, int value)
       cpp_opts->warn_trigraphs = value;
       break;
 
+    case OPT_Wundeclared_selector:
+      warn_undeclared_selector = value;
+      break;
+
     case OPT_Wundef:
       cpp_opts->warn_undef = value;
       break;
@@ -553,10 +645,6 @@ c_common_handle_option (size_t scode, const char *arg, int value)
       warn_unused_macros = value;
       break;
 
-    case OPT_Wvariadic_macros:
-      warn_variadic_macros = value;
-      break;
-
     case OPT_Wwrite_strings:
       if (!c_dialect_cxx ())
 	flag_const_strings = value;
@@ -571,11 +659,6 @@ c_common_handle_option (size_t scode, const char *arg, int value)
 	set_std_cxx98 (true);
       break;
 
-      /* APPLE LOCAL begin fat builds */
-    case OPT_arch:
-      break;
-      /* APPLE LOCAL end fat builds */
-
     case OPT_d:
       handle_OPT_d (arg);
       break;
@@ -589,9 +672,7 @@ c_common_handle_option (size_t scode, const char *arg, int value)
       /* Fall through.  */
 
     case OPT_fall_virtual:
-    case OPT_falt_external_templates:
     case OPT_fenum_int_equiv:
-    case OPT_fexternal_templates:
     case OPT_fguiding_decls:
     case OPT_fhonor_std:
     case OPT_fhuge_objects:
@@ -605,23 +686,30 @@ c_common_handle_option (size_t scode, const char *arg, int value)
     case OPT_fvtable_thunks:
     case OPT_fxref:
     case OPT_fvtable_gc:
-      warning ("switch %qs is no longer supported", option->opt_text);
+      warning ("switch \"%s\" is no longer supported", option->opt_text);
+      break;
+
+    case OPT_fabi_version_:
+      flag_abi_version = value;
       break;
 
     case OPT_faccess_control:
       flag_access_control = value;
       break;
 
+    case OPT_falt_external_templates:
+      flag_alt_external_templates = value;
+      if (value)
+	flag_external_templates = true;
+    cp_deprecated:
+      warning ("switch \"%s\" is deprecated, please see documentation "
+	       "for details", option->opt_text);
+      break;
+
     case OPT_fasm:
       flag_no_asm = !value;
       break;
 
-      /* APPLE LOCAL begin CW asm blocks */
-    case OPT_fasm_blocks:
-      flag_cw_asm_blocks = value;
-      break;
-      /* APPLE LOCAL end CW asm blocks */
-
     case OPT_fbuiltin:
       flag_no_builtin = !value;
       break;
@@ -633,16 +721,15 @@ c_common_handle_option (size_t scode, const char *arg, int value)
 	disable_builtin_function (arg);
       break;
 
-      /* APPLE LOCAL begin disable_typechecking_for_spec_flag */
-    case OPT_fdisable_typechecking_for_spec:
-      disable_typechecking_for_spec_flag = value;
-      break;
-      /* APPLE LOCAL end disable_typechecking_for_spec_flag */
-
     case OPT_fdollars_in_identifiers:
       cpp_opts->dollars_in_ident = value;
       break;
 
+    case OPT_fdump_:
+      if (!dump_switch_p (arg))
+	result = 0;
+      break;
+
     case OPT_ffreestanding:
       value = !value;
       /* Fall through....  */
@@ -668,6 +755,7 @@ c_common_handle_option (size_t scode, const char *arg, int value)
 
     case OPT_fsigned_bitfields:
       flag_signed_bitfields = value;
+      explicit_flag_signed_bitfields = 1;
       break;
 
     case OPT_fsigned_char:
@@ -676,18 +764,13 @@ c_common_handle_option (size_t scode, const char *arg, int value)
 
     case OPT_funsigned_bitfields:
       flag_signed_bitfields = !value;
+      explicit_flag_signed_bitfields = 1;
       break;
 
     case OPT_funsigned_char:
       flag_signed_char = !value;
       break;
 
-      /* APPLE LOCAL begin structor decloning */
-    case OPT_fclone_structors:
-      flag_clone_structors = value;
-      break;
-      /* APPLE LOCAL end structor decloning */
-
     case OPT_fcheck_new:
       flag_check_new = value;
       break;
@@ -716,6 +799,10 @@ c_common_handle_option (size_t scode, const char *arg, int value)
       flag_enforce_eh_specs = value;
       break;
 
+    case OPT_fexternal_templates:
+      flag_external_templates = value;
+      goto cp_deprecated;
+
     case OPT_ffixed_form:
     case OPT_ffixed_line_length_:
       /* Fortran front end options ignored when preprocessing only.  */
@@ -760,28 +847,10 @@ c_common_handle_option (size_t scode, const char *arg, int value)
       flag_next_runtime = value;
       break;
 
-    case OPT_fnil_receivers:
-      flag_nil_receivers = value;
-      break;
-
     case OPT_fnonansi_builtins:
       flag_no_nonansi_builtin = !value;
       break;
 
-    /* APPLE LOCAL begin mainline */
-    case OPT_fobjc_call_cxx_cdtors:
-      flag_objc_call_cxx_cdtors = value;
-      break;
-    /* APPLE LOCAL end mainline */
-
-    case OPT_fobjc_exceptions:
-      flag_objc_exceptions = value;
-      break;
-
-    case OPT_fobjc_sjlj_exceptions:
-      flag_objc_sjlj_exceptions = value;
-      break;
-
     case OPT_foperator_names:
       cpp_opts->operator_names = value;
       break;
@@ -794,24 +863,14 @@ c_common_handle_option (size_t scode, const char *arg, int value)
       cpp_opts->restore_pch_deps = value;
       break;
 
-    case OPT_fpch_preprocess:
-      flag_pch_preprocess = value;
-      break;
-
     case OPT_fpermissive:
       flag_permissive = value;
       break;
 
     case OPT_fpreprocessed:
       cpp_opts->preprocessed = value;
-      /* APPLE LOCAL private extern  Radar 2872481 --ilr */
-      flag_preprocessed = value;
       break;
 
-    case OPT_freplace_objc_classes:
-      flag_replace_objc_classes = value;
-      break;
-      
     case OPT_frepo:
       flag_use_repository = value;
       if (value)
@@ -844,10 +903,6 @@ c_common_handle_option (size_t scode, const char *arg, int value)
       cpp_opts->wide_charset = arg;
       break;
 
-    case OPT_finput_charset_:
-      cpp_opts->input_charset = arg;
-      break;
-
     case OPT_ftemplate_depth_:
       max_tinst_depth = value;
       break;
@@ -855,29 +910,17 @@ c_common_handle_option (size_t scode, const char *arg, int value)
     case OPT_fuse_cxa_atexit:
       flag_use_cxa_atexit = value;
       break;
-      
-    case OPT_fvisibility_inlines_hidden:
-      visibility_options.inlines_hidden = value;
-      break;
 
     case OPT_fweak:
       flag_weak = value;
       break;
 
-    case OPT_fthreadsafe_statics:
-      flag_threadsafe_statics = value;
-      break;
-
-    case OPT_fzero_link:
-      flag_zero_link = value;
-      break;
-
     case OPT_gen_decls:
       flag_gen_declaration = 1;
       break;
 
     case OPT_idirafter:
-      add_path (xstrdup (arg), AFTER, 0, true);
+      add_path (xstrdup (arg), AFTER, 0);
       break;
 
     case OPT_imacros:
@@ -889,16 +932,12 @@ c_common_handle_option (size_t scode, const char *arg, int value)
       iprefix = arg;
       break;
 
-    case OPT_iquote:
-      add_path (xstrdup (arg), QUOTE, 0, true);
-      break;
-
     case OPT_isysroot:
       sysroot = arg;
       break;
 
     case OPT_isystem:
-      add_path (xstrdup (arg), SYSTEM, 0, true);
+      add_path (xstrdup (arg), SYSTEM, 0);
       break;
 
     case OPT_iwithprefix:
@@ -948,13 +987,6 @@ c_common_handle_option (size_t scode, const char *arg, int value)
       print_struct_values = 1;
       break;
 
-/* APPLE LOCAL begin mainline 4.1 2005-06-17 3988498 */
-    case OPT_print_pch_checksum:
-      c_common_print_pch_checksum (stdout);
-      exit_after_options = true;
-      break;
-
-/* APPLE LOCAL end mainline 4.1 2005-06-17 3988498 */
     case OPT_remap:
       cpp_opts->remap = 1;
       break;
@@ -1005,13 +1037,6 @@ c_common_handle_option (size_t scode, const char *arg, int value)
     case OPT_v:
       verbose = true;
       break;
-
-    /* APPLE LOCAL begin -fast or -fastf or -fastcp */
-    case OPT_fast:
-    case OPT_fastcp:
-    case OPT_fastf:
-      break;
-    /* APPLE LOCAL end -fast or -fastf or -fastcp */
     }
 
   return result;
@@ -1021,20 +1046,12 @@ c_common_handle_option (size_t scode, const char *arg, int value)
 bool
 c_common_post_options (const char **pfilename)
 {
-  struct cpp_callbacks *cb;
-
   /* Canonicalize the input and output filenames.  */
   if (in_fnames == NULL)
     {
-      in_fnames = XNEWVEC (const char *, 1);
+      in_fnames = xmalloc (sizeof (in_fnames[0]));
       in_fnames[0] = "";
     }
-  /* APPLE LOCAL begin predictive compilation */
-  else if (predictive_compilation >= 0)
-      {
-	set_stdin_option(parse_in, predictive_compilation);
-      }
-  /* APPLE LOCAL end predictive compilation */
   else if (strcmp (in_fnames[0], "-") == 0)
     in_fnames[0] = "";
 
@@ -1053,29 +1070,23 @@ c_common_post_options (const char **pfilename)
 
   flag_inline_trees = 1;
 
-  /* Use tree inlining.  */
-  if (!flag_no_inline)
-    flag_no_inline = 1;
-  if (flag_inline_functions)
-    flag_inline_trees = 2;
+  /* Use tree inlining if possible.  Function instrumentation is only
+     done in the RTL level, so we disable tree inlining.  */
+  if (! flag_instrument_function_entry_exit)
+    {
+      if (!flag_no_inline)
+	flag_no_inline = 1;
+      if (flag_inline_functions)
+	{
+	  flag_inline_trees = 2;
+	  flag_inline_functions = 0;
+	}
+    }
 
-  /* If we are given more than one input file, we must use
-     unit-at-a-time mode.  */
-  if (num_in_fnames > 1)
-    flag_unit_at_a_time = 1;
-
-  /* Default to ObjC sjlj exception handling if NeXT runtime.  */
-  if (flag_objc_sjlj_exceptions < 0)
-    flag_objc_sjlj_exceptions = flag_next_runtime;
-  if (flag_objc_exceptions && !flag_objc_sjlj_exceptions)
-    flag_exceptions = 1;
-
-  /* -Wextra implies -Wsign-compare and -Wmissing-field-initializers,
-     but not if explicitly overridden.  */
+  /* -Wextra implies -Wsign-compare, but not if explicitly
+      overridden.  */
   if (warn_sign_compare == -1)
     warn_sign_compare = extra_warnings;
-  if (warn_missing_field_initializers == -1)
-    warn_missing_field_initializers = extra_warnings;
 
   /* Special format checking options don't work without -Wformat; warn if
      they are used.  */
@@ -1092,11 +1103,6 @@ c_common_post_options (const char **pfilename)
   if (warn_missing_format_attribute && !warn_format)
     warning ("-Wmissing-format-attribute ignored without -Wformat");
 
-  /* C99 requires special handling of complex multiplication and division;
-     -ffast-math and -fcx-limited-range are handled in process_options.  */
-  if (flag_isoc99)
-    flag_complex_method = 2;
-
   if (flag_preprocess_only)
     {
       /* Open the output now.  We must do so even if flag_no_output is
@@ -1124,33 +1130,22 @@ c_common_post_options (const char **pfilename)
       init_c_lex ();
 
       /* Yuk.  WTF is this?  I do know ObjC relies on it somewhere.  */
-      input_location = UNKNOWN_LOCATION;
+      input_line = 0;
     }
 
-  cb = cpp_get_callbacks (parse_in);
-  cb->file_change = cb_file_change;
-  cb->dir_change = cb_dir_change;
+  cpp_get_callbacks (parse_in)->file_change = cb_file_change;
   cpp_post_options (parse_in);
 
-  input_location = UNKNOWN_LOCATION;
+  /* NOTE: we use in_fname here, not the one supplied.  */
+  *pfilename = cpp_read_main_file (parse_in, in_fnames[0]);
+
+  saved_lineno = input_line;
+  input_line = 0;
 
   /* If an error has occurred in cpplib, note it so we fail
      immediately.  */
   errorcount += cpp_errors (parse_in);
 
-  *pfilename = this_input_filename
-    = cpp_read_main_file (parse_in, in_fnames[0]);
-  /* Don't do any compilation or preprocessing if there is no input file.  */
-  if (this_input_filename == NULL)
-    {
-      errorcount++;
-      return false;
-    }
-
-  if (flag_working_directory
-      && flag_preprocess_only && !flag_no_line_commands)
-    pp_dir_change (parse_in, get_src_pwd ());
-
   return flag_preprocess_only;
 }
 
@@ -1158,31 +1153,24 @@ c_common_post_options (const char **pfilename)
 bool
 c_common_init (void)
 {
+  input_line = saved_lineno;
+
   /* Set up preprocessor arithmetic.  Must be done after call to
      c_common_nodes_and_builtins for type nodes to be good.  */
   cpp_opts->precision = TYPE_PRECISION (intmax_type_node);
   cpp_opts->char_precision = TYPE_PRECISION (char_type_node);
   cpp_opts->int_precision = TYPE_PRECISION (integer_type_node);
   cpp_opts->wchar_precision = TYPE_PRECISION (wchar_type_node);
-  cpp_opts->unsigned_wchar = TYPE_UNSIGNED (wchar_type_node);
+  cpp_opts->unsigned_wchar = TREE_UNSIGNED (wchar_type_node);
   cpp_opts->bytes_big_endian = BYTES_BIG_ENDIAN;
-  /* APPLE LOCAL begin CW asm blocks */
-  if (flag_ms_asms)
-    cpp_opts->h_suffix = true;
-  /* APPLE LOCAL end CW asm blocks */
 
   /* This can't happen until after wchar_precision and bytes_big_endian
      are known.  */
   cpp_init_iconv (parse_in);
 
-/* APPLE LOCAL begin mainline 4.1 2005-06-17 3988498 */
-  if (version_flag)
-    c_common_print_pch_checksum (stderr);
-
-/* APPLE LOCAL end mainline 4.1 2005-06-17 3988498 */
   if (flag_preprocess_only)
     {
-      finish_options ();
+      finish_options (in_fnames[0]);
       preprocess_file (parse_in);
       return false;
     }
@@ -1196,44 +1184,40 @@ c_common_init (void)
 /* Initialize the integrated preprocessor after debug output has been
    initialized; loop over each input file.  */
 void
-c_common_parse_file (int set_yydebug)
+c_common_parse_file (int set_yydebug ATTRIBUTE_UNUSED)
 {
-  unsigned int i;
-
-  /* Enable parser debugging, if requested and we can.  If requested
-     and we can't, notify the user.  */
+  unsigned file_index;
+  
 #if YYDEBUG != 0
   yydebug = set_yydebug;
 #else
-  if (set_yydebug)
-    warning ("YYDEBUG was not defined at build time, -dy ignored");
+  warning ("YYDEBUG not defined");
 #endif
 
-  i = 0;
-  for (;;)
+  file_index = 0;
+  
+  do
     {
-      /* Start the main input file, if the debug writer wants it.  */
-      if (debug_hooks->start_end_main_source_file)
-        (*debug_hooks->start_source_file) (0, this_input_filename);
-      finish_options ();
-      pch_init ();
-      push_file_scope ();
+      if (file_index > 0)
+	{
+	  /* Reset the state of the parser.  */
+	  c_reset_state();
+
+	  /* Reset cpplib's macros and start a new file.  */
+	  cpp_undef_all (parse_in);
+	  cpp_read_main_file (parse_in, in_fnames[file_index]);
+	}
+
+      finish_options(in_fnames[file_index]);
+      if (file_index == 0)
+	pch_init();
       c_parse_file ();
-      finish_file ();
-      pop_file_scope ();
-      /* And end the main input file, if the debug writer wants it */
-      if (debug_hooks->start_end_main_source_file)
-        (*debug_hooks->end_source_file) (0);
-      if (++i >= num_in_fnames)
-	break;
-      cpp_undef_all (parse_in);
-      this_input_filename
-	= cpp_read_main_file (parse_in, in_fnames[i]);
-      /* If an input file is missing, abandon further compilation.
-         cpplib has issued a diagnostic.  */
-      if (!this_input_filename)
-	break;
-    }
+
+      file_index++;
+    } while (file_index < num_in_fnames);
+  
+  free_parser_stacks ();
+  finish_file ();
 }
 
 /* Common finish hook for the C, ObjC and C++ front ends.  */
@@ -1309,7 +1293,6 @@ check_deps_environment_vars (void)
 	deps_file = spec;
 
       deps_append = 1;
-      deps_seen = true;
     }
 }
 
@@ -1318,22 +1301,13 @@ static void
 handle_deferred_opts (void)
 {
   size_t i;
-  struct deps *deps;
-
-  /* Avoid allocating the deps buffer if we don't need it.
-     (This flag may be true without there having been -MT or -MQ
-     options, but we'll still need the deps buffer.)  */
-  if (!deps_seen)
-    return;
-
-  deps = cpp_get_deps (parse_in);
 
   for (i = 0; i < deferred_count; i++)
     {
       struct deferred_opt *opt = &deferred_opts[i];
 
       if (opt->code == OPT_MT || opt->code == OPT_MQ)
-	deps_add_target (deps, opt->arg, opt->code == OPT_MQ);
+	cpp_add_dependency_target (parse_in, opt->arg, opt->code == OPT_MQ);
     }
 }
 
@@ -1354,13 +1328,11 @@ sanitize_cpp_opts (void)
 
   /* Disable -dD, -dN and -dI if normal output is suppressed.  Allow
      -dM since at least glibc relies on -M -dM to work.  */
-  /* Also, flag_no_output implies flag_no_line_commands, always. */
   if (flag_no_output)
     {
       if (flag_dump_macros != 'M')
 	flag_dump_macros = 0;
       flag_dump_includes = 0;
-      flag_no_line_commands = 1;
     }
 
   cpp_opts->unsigned_char = !flag_signed_char;
@@ -1371,17 +1343,14 @@ sanitize_cpp_opts (void)
   cpp_opts->warn_long_long
     = warn_long_long && ((!flag_isoc99 && pedantic) || warn_traditional);
 
-  /* Similarly with -Wno-variadic-macros.  No check for c99 here, since
-     this also turns off warnings about GCCs extension.  */
-  cpp_opts->warn_variadic_macros
-    = warn_variadic_macros && (pedantic || warn_traditional);
-
   /* If we're generating preprocessor output, emit current directory
      if explicitly requested or if debugging information is enabled.
      ??? Maybe we should only do it for debugging formats that
      actually output the current directory?  */
   if (flag_working_directory == -1)
     flag_working_directory = (debug_info_level != DINFO_LEVEL_NONE);
+  cpp_opts->working_directory
+    = flag_preprocess_only && flag_working_directory;
 }
 
 /* Add include path with a prefix at the front of its name.  */
@@ -1396,17 +1365,19 @@ add_prefixed_path (const char *suffix, size_t chain)
   prefix     = iprefix ? iprefix : cpp_GCC_INCLUDE_DIR;
   prefix_len = iprefix ? strlen (iprefix) : cpp_GCC_INCLUDE_DIR_len;
 
-  path = (char *) xmalloc (prefix_len + suffix_len + 1);
+  path = xmalloc (prefix_len + suffix_len + 1);
   memcpy (path, prefix, prefix_len);
   memcpy (path + prefix_len, suffix, suffix_len);
   path[prefix_len + suffix_len] = '\0';
 
-  add_path (path, chain, 0, false);
+  add_path (path, chain, 0);
 }
 
-/* Handle -D, -U, -A, -imacros, and the first -include.  */
+/* Handle -D, -U, -A, -imacros, and the first -include.  
+   TIF is the input file to which we will return after processing all
+   the includes.  */
 static void
-finish_options (void)
+finish_options (const char *tif)
 {
   if (!cpp_opts->preprocessed)
     {
@@ -1452,15 +1423,12 @@ finish_options (void)
 
 	  if (opt->code == OPT_imacros
 	      && cpp_push_include (parse_in, opt->arg))
-	    {
-	      /* Disable push_command_line_include callback for now.  */
-	      include_cursor = deferred_count + 1;
-	      cpp_scan_nooutput (parse_in);
-	    }
+	    cpp_scan_nooutput (parse_in);
 	}
     }
 
   include_cursor = 0;
+  this_input_filename = tif;
   push_command_line_include ();
 }
 
@@ -1468,33 +1436,30 @@ finish_options (void)
 static void
 push_command_line_include (void)
 {
+  if (cpp_opts->preprocessed)
+    return;
+
   while (include_cursor < deferred_count)
     {
       struct deferred_opt *opt = &deferred_opts[include_cursor++];
 
-      if (!cpp_opts->preprocessed && opt->code == OPT_include
-	  && cpp_push_include (parse_in, opt->arg))
+      if (opt->code == OPT_include && cpp_push_include (parse_in, opt->arg))
 	return;
     }
 
   if (include_cursor == deferred_count)
     {
-      include_cursor++;
+      /* Restore the line map from <command line>.  */
+      cpp_change_file (parse_in, LC_RENAME, this_input_filename);
       /* -Wunused-macros should only warn about macros defined hereafter.  */
       cpp_opts->warn_unused_macros = warn_unused_macros;
-      /* Restore the line map from <command line>.  */
-      if (!cpp_opts->preprocessed)
-	cpp_change_file (parse_in, LC_RENAME, this_input_filename);
-
-      /* Set this here so the client can change the option if it wishes,
-	 and after stacking the main file so we don't trace the main file.  */
-      line_table.trace_includes = cpp_opts->print_include_names;
+      include_cursor++;
     }
 }
 
 /* File change callback.  Has to handle -include files.  */
 static void
-cb_file_change (cpp_reader * ARG_UNUSED (pfile),
+cb_file_change (cpp_reader *pfile ATTRIBUTE_UNUSED,
 		const struct line_map *new_map)
 {
   if (flag_preprocess_only)
@@ -1502,34 +1467,22 @@ cb_file_change (cpp_reader * ARG_UNUSED (pfile),
   else
     fe_file_change (new_map);
 
-  if (new_map == 0 || (new_map->reason == LC_LEAVE && MAIN_FILE_P (new_map)))
+  if (new_map->reason == LC_LEAVE && MAIN_FILE_P (new_map))
     push_command_line_include ();
 }
 
-void
-cb_dir_change (cpp_reader * ARG_UNUSED (pfile), const char *dir)
-{
-  if (!set_src_pwd (dir))
-    warning ("too late for # directive to set debug directory");
-}
-
 /* Set the C 89 standard (with 1994 amendments if C94, without GNU
    extensions if ISO).  There is no concept of gnu94.  */
 static void
 set_std_c89 (int c94, int iso)
 {
-  /* APPLE LOCAL begin preprocess .s files (radar #3191171) */
-  /* Do not override CLK_ASM if set */
-  if (cpp_opts->lang != CLK_ASM)
-     cpp_set_lang (parse_in, c94 ? CLK_STDC94: iso ? CLK_STDC89: CLK_GNUC89);
-  /* APPLE LOCAL end preprocess .s files (radar #3191171) */
+  cpp_set_lang (parse_in, c94 ? CLK_STDC94: iso ? CLK_STDC89: CLK_GNUC89);
   flag_iso = iso;
   flag_no_asm = iso;
   flag_no_gnu_keywords = iso;
   flag_no_nonansi_builtin = iso;
   flag_isoc94 = c94;
   flag_isoc99 = 0;
-  /* APPLE LOCAL fwritable strings  */
   flag_writable_strings = 0;
 }
 
@@ -1543,7 +1496,6 @@ set_std_c99 (int iso)
   flag_iso = iso;
   flag_isoc99 = 1;
   flag_isoc94 = 1;
-  /* APPLE LOCAL fwritable strings  */
   flag_writable_strings = 0;
 }
 
