@@ -1,5 +1,5 @@
 /* Basic block reordering routines for the GNU compiler.
-   Copyright (C) 2000, 2001, 2003, 2004 Free Software Foundation, Inc.
+   Copyright (C) 2000, 2001, 2003 Free Software Foundation, Inc.
 
 This file is part of GCC.
 
@@ -25,17 +25,22 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 #include "tree.h"
 #include "rtl.h"
 #include "hard-reg-set.h"
-#include "obstack.h"
 #include "basic-block.h"
 #include "insn-config.h"
 #include "output.h"
 #include "function.h"
+#include "obstack.h"
 #include "cfglayout.h"
 #include "cfgloop.h"
 #include "target.h"
 #include "ggc.h"
 #include "alloc-pool.h"
-#include "flags.h"
+
+/* The contents of the current function definition are allocated
+   in this obstack, and all are freed at the end of the function.  */
+extern struct obstack flow_obstack;
+
+alloc_pool cfg_layout_pool;
 
 /* Holds the interesting trailing notes for the function.  */
 rtx cfg_layout_function_footer, cfg_layout_function_header;
@@ -50,8 +55,9 @@ static void change_scope (rtx, tree, tree);
 
 void verify_insn_chain (void);
 static void fixup_fallthru_exit_predecessor (void);
+static rtx duplicate_insn_chain (rtx, rtx);
+static void break_superblocks (void);
 static tree insn_scope (rtx);
-static void update_unlikely_executed_notes (basic_block);
 
 rtx
 unlink_insn_chain (rtx first, rtx last)
@@ -83,9 +89,9 @@ skip_insns_after_block (basic_block bb)
 
   next_head = NULL_RTX;
   if (bb->next_bb != EXIT_BLOCK_PTR)
-    next_head = BB_HEAD (bb->next_bb);
+    next_head = bb->next_bb->head;
 
-  for (last_insn = insn = BB_END (bb); (insn = NEXT_INSN (insn)) != 0; )
+  for (last_insn = insn = bb->end; (insn = NEXT_INSN (insn)) != 0; )
     {
       if (insn == next_head)
 	break;
@@ -115,7 +121,7 @@ skip_insns_after_block (basic_block bb)
 
 	case CODE_LABEL:
 	  if (NEXT_INSN (insn)
-	      && JUMP_P (NEXT_INSN (insn))
+	      && GET_CODE (NEXT_INSN (insn)) == JUMP_INSN
 	      && (GET_CODE (PATTERN (NEXT_INSN (insn))) == ADDR_VEC
 	          || GET_CODE (PATTERN (NEXT_INSN (insn))) == ADDR_DIFF_VEC))
 	    {
@@ -142,10 +148,10 @@ skip_insns_after_block (basic_block bb)
      created by removing the basic block originally following
      NOTE_INSN_LOOP_BEG.  In such case reorder the notes.  */
 
-  for (insn = last_insn; insn != BB_END (bb); insn = prev)
+  for (insn = last_insn; insn != bb->end; insn = prev)
     {
       prev = PREV_INSN (insn);
-      if (NOTE_P (insn))
+      if (GET_CODE (insn) == NOTE)
 	switch (NOTE_LINE_NUMBER (insn))
 	  {
 	  case NOTE_INSN_LOOP_END:
@@ -166,12 +172,12 @@ skip_insns_after_block (basic_block bb)
 static rtx
 label_for_bb (basic_block bb)
 {
-  rtx label = BB_HEAD (bb);
+  rtx label = bb->head;
 
-  if (!LABEL_P (label))
+  if (GET_CODE (label) != CODE_LABEL)
     {
-      if (dump_file)
-	fprintf (dump_file, "Emitting label for block %d\n", bb->index);
+      if (rtl_dump_file)
+	fprintf (rtl_dump_file, "Emitting label for block %d\n", bb->index);
 
       label = block_label (bb);
     }
@@ -191,13 +197,12 @@ record_effective_endpoints (void)
 
   for (insn = get_insns ();
        insn
-       && NOTE_P (insn)
+       && GET_CODE (insn) == NOTE
        && NOTE_LINE_NUMBER (insn) != NOTE_INSN_BASIC_BLOCK;
        insn = NEXT_INSN (insn))
     continue;
-  /* No basic blocks at all?  */
-  gcc_assert (insn);
-  
+  if (!insn)
+    abort ();  /* No basic blocks at all?  */
   if (PREV_INSN (insn))
     cfg_layout_function_header =
 	    unlink_insn_chain (get_insns (), PREV_INSN (insn));
@@ -209,13 +214,13 @@ record_effective_endpoints (void)
     {
       rtx end;
 
-      if (PREV_INSN (BB_HEAD (bb)) && next_insn != BB_HEAD (bb))
+      if (PREV_INSN (bb->head) && next_insn != bb->head)
 	bb->rbi->header = unlink_insn_chain (next_insn,
-					      PREV_INSN (BB_HEAD (bb)));
+					      PREV_INSN (bb->head));
       end = skip_insns_after_block (bb);
-      if (NEXT_INSN (BB_END (bb)) && BB_END (bb) != end)
-	bb->rbi->footer = unlink_insn_chain (NEXT_INSN (BB_END (bb)), end);
-      next_insn = NEXT_INSN (BB_END (bb));
+      if (NEXT_INSN (bb->end) && bb->end != end)
+	bb->rbi->footer = unlink_insn_chain (NEXT_INSN (bb->end), end);
+      next_insn = NEXT_INSN (bb->end);
     }
 
   cfg_layout_function_footer = next_insn;
@@ -251,7 +256,7 @@ insn_locators_initialize (void)
   rtx insn, next;
   int loc = 0;
   int line_number = 0, last_line_number = 0;
-  const char *file_name = NULL, *last_file_name = NULL;
+  char *file_name = NULL, *last_file_name = NULL;
 
   prologue_locator = epilogue_locator = 0;
 
@@ -264,31 +269,12 @@ insn_locators_initialize (void)
 
   for (insn = get_insns (); insn; insn = next)
     {
-      int active = 0;
-      
       next = NEXT_INSN (insn);
 
-      if (NOTE_P (insn))
-	{
-	  gcc_assert (NOTE_LINE_NUMBER (insn) != NOTE_INSN_BLOCK_BEG
-		      && NOTE_LINE_NUMBER (insn) != NOTE_INSN_BLOCK_END);
-	  if (NOTE_LINE_NUMBER (insn) > 0)
-	    {
-	      expanded_location xloc;
-	      NOTE_EXPANDED_LOCATION (xloc, insn);
-	      line_number = xloc.line;
-	      file_name = xloc.file;
-	    }
-	}
-      else
-	active = (active_insn_p (insn)
-		  && GET_CODE (PATTERN (insn)) != ADDR_VEC
-		  && GET_CODE (PATTERN (insn)) != ADDR_DIFF_VEC);
-      
-      check_block_change (insn, &block);
-
-      if (active
-	  || !next
+      if ((active_insn_p (insn)
+	   && GET_CODE (PATTERN (insn)) != ADDR_VEC
+	   && GET_CODE (PATTERN (insn)) != ADDR_DIFF_VEC)
+	  || !NEXT_INSN (insn)
 	  || (!prologue_locator && file_name))
 	{
 	  if (last_block != block)
@@ -309,23 +295,44 @@ insn_locators_initialize (void)
 	    {
 	      loc++;
 	      VARRAY_PUSH_INT (file_locators_locs, loc);
-	      VARRAY_PUSH_CHAR_PTR (file_locators_files, (char *) file_name);
+	      VARRAY_PUSH_CHAR_PTR (file_locators_files, file_name);
 	      last_file_name = file_name;
 	    }
-	  if (!prologue_locator && file_name)
-	    prologue_locator = loc;
-	  if (!next)
-	    epilogue_locator = loc;
-	  if (active)
-	    INSN_LOCATOR (insn) = loc;
+	}
+      if (!prologue_locator && file_name)
+	prologue_locator = loc;
+      if (!NEXT_INSN (insn))
+	epilogue_locator = loc;
+      if (active_insn_p (insn))
+        INSN_LOCATOR (insn) = loc;
+      else if (GET_CODE (insn) == NOTE)
+	{
+	  switch (NOTE_LINE_NUMBER (insn))
+	    {
+	    case NOTE_INSN_BLOCK_BEG:
+	      block = NOTE_BLOCK (insn);
+	      delete_insn (insn);
+	      break;
+	    case NOTE_INSN_BLOCK_END:
+	      block = BLOCK_SUPERCONTEXT (block);
+	      if (block && TREE_CODE (block) == FUNCTION_DECL)
+		block = 0;
+	      delete_insn (insn);
+	      break;
+	    default:
+	      if (NOTE_LINE_NUMBER (insn) > 0)
+		{
+		  line_number = NOTE_LINE_NUMBER (insn);
+		  file_name = (char *)NOTE_SOURCE_FILE (insn);
+		}
+	      break;
+	    }
 	}
     }
 
   /* Tag the blocks with a depth number so that change_scope can find
      the common parent easily.  */
   set_block_levels (DECL_INITIAL (cfun->decl), 0);
-
-  free_block_changes ();
 }
 
 /* For each lexical block, set BLOCK_NUMBER to the depth at which it is
@@ -343,7 +350,7 @@ set_block_levels (tree block, int level)
 }
 
 /* Return sope resulting from combination of S1 and S2.  */
-static tree
+tree
 choose_inner_scope (tree s1, tree s2)
 {
    if (!s1)
@@ -367,7 +374,8 @@ change_scope (rtx orig_insn, tree s1, tree s2)
 
   while (ts1 != ts2)
     {
-      gcc_assert (ts1 && ts2);
+      if (ts1 == NULL || ts2 == NULL)
+	abort ();
       if (BLOCK_NUMBER (ts1) > BLOCK_NUMBER (ts2))
 	ts1 = BLOCK_SUPERCONTEXT (ts1);
       else if (BLOCK_NUMBER (ts1) < BLOCK_NUMBER (ts2))
@@ -407,18 +415,6 @@ insn_scope (rtx insn)
   int min = 0;
   int loc = INSN_LOCATOR (insn);
 
-  /* When block_locators_locs was initialized, the pro- and epilogue
-     insns didn't exist yet and can therefore not be found this way.
-     But we know that they belong to the outer most block of the
-     current function.
-     Without this test, the prologue would be put inside the block of
-     the first valid instruction in the function and when that first
-     insn is part of an inlined function then the low_pc of that
-     inlined function is messed up.  Likewise for the epilogue and
-     the last valid instruction.  */
-  if (loc == prologue_locator || loc == epilogue_locator)
-    return DECL_INITIAL (cfun->decl);
-
   if (!max || !loc)
     return NULL;
   while (1)
@@ -523,12 +519,6 @@ reemit_insn_block_notes (void)
     {
       tree this_block;
 
-      /* Avoid putting scope notes between jump table and its label.  */
-      if (JUMP_P (insn)
-	  && (GET_CODE (PATTERN (insn)) == ADDR_VEC
-	      || GET_CODE (PATTERN (insn)) == ADDR_DIFF_VEC))
-	continue;
-
       this_block = insn_scope (insn);
       /* For sequences compute scope resulting from merging all scopes
          of instructions nested inside.  */
@@ -596,11 +586,11 @@ fixup_reorder_chain (void)
 	    insn = NEXT_INSN (insn);
 	}
       if (insn)
-	NEXT_INSN (insn) = BB_HEAD (bb);
+	NEXT_INSN (insn) = bb->head;
       else
-	set_first_insn (BB_HEAD (bb));
-      PREV_INSN (BB_HEAD (bb)) = insn;
-      insn = BB_END (bb);
+	set_first_insn (bb->head);
+      PREV_INSN (bb->head) = insn;
+      insn = bb->end;
       if (bb->rbi->footer)
 	{
 	  NEXT_INSN (insn) = bb->rbi->footer;
@@ -610,7 +600,8 @@ fixup_reorder_chain (void)
 	}
     }
 
-  gcc_assert (index == n_basic_blocks);
+  if (index != n_basic_blocks)
+    abort ();
 
   NEXT_INSN (insn) = cfg_layout_function_footer;
   if (cfg_layout_function_footer)
@@ -633,30 +624,28 @@ fixup_reorder_chain (void)
       edge e_fall, e_taken, e;
       rtx bb_end_insn;
       basic_block nb;
-      basic_block old_bb;
-      edge_iterator ei;
 
-      if (EDGE_COUNT (bb->succs) == 0)
+      if (bb->succ == NULL)
 	continue;
 
       /* Find the old fallthru edge, and another non-EH edge for
 	 a taken jump.  */
       e_taken = e_fall = NULL;
-
-      FOR_EACH_EDGE (e, ei, bb->succs)
+      for (e = bb->succ; e ; e = e->succ_next)
 	if (e->flags & EDGE_FALLTHRU)
 	  e_fall = e;
 	else if (! (e->flags & EDGE_EH))
 	  e_taken = e;
 
-      bb_end_insn = BB_END (bb);
-      if (JUMP_P (bb_end_insn))
+      bb_end_insn = bb->end;
+      if (GET_CODE (bb_end_insn) == JUMP_INSN)
 	{
 	  if (any_condjump_p (bb_end_insn))
 	    {
 	      /* If the old fallthru is still next, nothing to do.  */
 	      if (bb->rbi->next == e_fall->dest
-	          || e_fall->dest == EXIT_BLOCK_PTR)
+	          || (!bb->rbi->next
+		      && e_fall->dest == EXIT_BLOCK_PTR))
 		continue;
 
 	      /* The degenerated case of conditional jump jumping to the next
@@ -671,15 +660,12 @@ fixup_reorder_chain (void)
 		{
 		  rtx note;
 		  edge e_fake;
-		  bool redirected;
 
 		  e_fake = unchecked_make_edge (bb, e_fall->dest, 0);
 
-		  redirected = redirect_jump (BB_END (bb),
-					      block_label (bb), 0);
-		  gcc_assert (redirected);
-		  
-		  note = find_reg_note (BB_END (bb), REG_BR_PROB, NULL_RTX);
+		  if (!redirect_jump (bb->end, block_label (bb), 0))
+		    abort ();
+		  note = find_reg_note (bb->end, REG_BR_PROB, NULL_RTX);
 		  if (note)
 		    {
 		      int prob = INTVAL (XEXP (note, 0));
@@ -705,51 +691,44 @@ fixup_reorder_chain (void)
 		  if (note
 		      && INTVAL (XEXP (note, 0)) < REG_BR_PROB_BASE / 2
 		      && invert_jump (bb_end_insn,
-				      (e_fall->dest == EXIT_BLOCK_PTR
-				       ? NULL_RTX
-				       : label_for_bb (e_fall->dest)), 0))
+				      label_for_bb (e_fall->dest), 0))
 		    {
 		      e_fall->flags &= ~EDGE_FALLTHRU;
-#ifdef ENABLE_CHECKING
-		      gcc_assert (could_fall_through
-				  (e_taken->src, e_taken->dest));
-#endif
 		      e_taken->flags |= EDGE_FALLTHRU;
 		      update_br_prob_note (bb);
 		      e = e_fall, e_fall = e_taken, e_taken = e;
 		    }
 		}
 
-	      /* If the "jumping" edge is a crossing edge, and the fall
-		 through edge is non-crossing, leave things as they are.  */
-	      else if ((e_taken->flags & EDGE_CROSSING)
-		       && !(e_fall->flags & EDGE_CROSSING))
-		continue;
-
 	      /* Otherwise we can try to invert the jump.  This will
 		 basically never fail, however, keep up the pretense.  */
 	      else if (invert_jump (bb_end_insn,
-				    (e_fall->dest == EXIT_BLOCK_PTR
-				     ? NULL_RTX
-				     : label_for_bb (e_fall->dest)), 0))
+				    label_for_bb (e_fall->dest), 0))
 		{
 		  e_fall->flags &= ~EDGE_FALLTHRU;
-#ifdef ENABLE_CHECKING
-		  gcc_assert (could_fall_through
-			      (e_taken->src, e_taken->dest));
-#endif
 		  e_taken->flags |= EDGE_FALLTHRU;
 		  update_br_prob_note (bb);
 		  continue;
 		}
 	    }
+	  else if (returnjump_p (bb_end_insn))
+	    continue;
 	  else
 	    {
-	      /* Otherwise we have some return, switch or computed
-		 jump.  In the 99% case, there should not have been a
-		 fallthru edge.  */
-	      gcc_assert (returnjump_p (bb_end_insn) || !e_fall);
-	      continue;
+	      /* Otherwise we have some switch or computed jump.  In the
+		 99% case, there should not have been a fallthru edge.  */
+	      if (! e_fall)
+		continue;
+
+#ifdef CASE_DROPS_THROUGH
+	      /* Except for VAX.  Since we didn't have predication for the
+		 tablejump, the fallthru block should not have moved.  */
+	      if (bb->rbi->next == e_fall->dest)
+		continue;
+	      bb_end_insn = skip_insns_after_block (bb);
+#else
+	      abort ();
+#endif
 	    }
 	}
       else
@@ -765,7 +744,7 @@ fixup_reorder_chain (void)
 	    continue;
 
 	  /* A fallthru to exit block.  */
-	  if (e_fall->dest == EXIT_BLOCK_PTR)
+	  if (!bb->rbi->next && e_fall->dest == EXIT_BLOCK_PTR)
 	    continue;
 	}
 
@@ -773,63 +752,31 @@ fixup_reorder_chain (void)
       nb = force_nonfallthru (e_fall);
       if (nb)
 	{
-	  initialize_bb_rbi (nb);
+	  cfg_layout_initialize_rbi (nb);
 	  nb->rbi->visited = 1;
 	  nb->rbi->next = bb->rbi->next;
 	  bb->rbi->next = nb;
 	  /* Don't process this new block.  */
-	  old_bb = bb;
 	  bb = nb;
-	  
-	  /* Make sure new bb is tagged for correct section (same as
-	     fall-thru source, since you cannot fall-throu across
-	     section boundaries).  */
-	  BB_COPY_PARTITION (e_fall->src, EDGE_PRED (bb, 0)->src);
-	  if (flag_reorder_blocks_and_partition
-	      && targetm.have_named_sections)
-	    {
-	      if (BB_PARTITION (EDGE_PRED (bb, 0)->src) == BB_COLD_PARTITION)
-		{
-		  rtx new_note;
-		  rtx note = BB_HEAD (e_fall->src);
-		  
-		  while (!INSN_P (note)
-			 && note != BB_END (e_fall->src))
-		    note = NEXT_INSN (note);
-		  
-		  new_note = emit_note_before 
-                                          (NOTE_INSN_UNLIKELY_EXECUTED_CODE, 
-					   note);
-		  NOTE_BASIC_BLOCK (new_note) = bb;
-		}
-	      if (JUMP_P (BB_END (bb))
-		  && !any_condjump_p (BB_END (bb))
-  		  && (EDGE_SUCC (bb, 0)->flags & EDGE_CROSSING))
-		REG_NOTES (BB_END (bb)) = gen_rtx_EXPR_LIST 
-		  (REG_CROSSING_JUMP, NULL_RTX, REG_NOTES (BB_END (bb)));
-	    }
 	}
     }
 
   /* Put basic_block_info in the new order.  */
 
-  if (dump_file)
+  if (rtl_dump_file)
     {
-      fprintf (dump_file, "Reordered sequence:\n");
-      for (bb = ENTRY_BLOCK_PTR->next_bb, index = 0;
-	   bb;
-	   bb = bb->rbi->next, index++)
+      fprintf (rtl_dump_file, "Reordered sequence:\n");
+      for (bb = ENTRY_BLOCK_PTR->next_bb, index = 0; bb; bb = bb->rbi->next, index ++)
 	{
-	  fprintf (dump_file, " %i ", index);
+	  fprintf (rtl_dump_file, " %i ", index);
 	  if (bb->rbi->original)
-	    fprintf (dump_file, "duplicate of %i ",
+	    fprintf (rtl_dump_file, "duplicate of %i ",
 		     bb->rbi->original->index);
-	  else if (forwarder_block_p (bb)
-		   && !LABEL_P (BB_HEAD (bb)))
-	    fprintf (dump_file, "compensation ");
+	  else if (forwarder_block_p (bb) && GET_CODE (bb->head) != CODE_LABEL)
+	    fprintf (rtl_dump_file, "compensation ");
 	  else
-	    fprintf (dump_file, "bb %i ", bb->index);
-	  fprintf (dump_file, " [%i]\n", bb->frequency);
+	    fprintf (rtl_dump_file, "bb %i ", bb->index);
+	  fprintf (rtl_dump_file, " [%i]\n", bb->frequency);
 	}
     }
 
@@ -842,8 +789,6 @@ fixup_reorder_chain (void)
       bb->index = index;
       BASIC_BLOCK (index) = bb;
 
-      update_unlikely_executed_notes (bb);
-
       bb->prev_bb = prev_bb;
       prev_bb->next_bb = bb;
     }
@@ -854,32 +799,13 @@ fixup_reorder_chain (void)
   FOR_EACH_BB (bb)
     {
       edge e;
-      edge_iterator ei;
-
-      FOR_EACH_EDGE (e, ei, bb->succs)
-	if (e->flags & EDGE_FALLTHRU)
-	  break;
-
+      for (e = bb->succ; e && !(e->flags & EDGE_FALLTHRU); e = e->succ_next)
+	continue;
       if (e && !can_fallthru (e->src, e->dest))
 	force_nonfallthru (e);
     }
 }
 
-/* Update the basic block number information in any 
-   NOTE_INSN_UNLIKELY_EXECUTED_CODE notes within the basic block.  */
-
-static void
-update_unlikely_executed_notes (basic_block bb)
-{
-  rtx cur_insn;
-
-  for (cur_insn = BB_HEAD (bb); cur_insn != BB_END (bb); 
-       cur_insn = NEXT_INSN (cur_insn)) 
-    if (NOTE_P (cur_insn)
-	&& NOTE_LINE_NUMBER (cur_insn) == NOTE_INSN_UNLIKELY_EXECUTED_CODE)
-      NOTE_BASIC_BLOCK (cur_insn) = bb;
-}
-
 /* Perform sanity checks on the insn chain.
    1. Check that next/prev pointers are consistent in both the forward and
       reverse direction.
@@ -895,34 +821,31 @@ verify_insn_chain (void)
   for (prevx = NULL, insn_cnt1 = 1, x = get_insns ();
        x != 0;
        prevx = x, insn_cnt1++, x = NEXT_INSN (x))
-    gcc_assert (PREV_INSN (x) == prevx);
+    if (PREV_INSN (x) != prevx)
+      abort ();
 
-  gcc_assert (prevx == get_last_insn ());
+  if (prevx != get_last_insn ())
+    abort ();
 
   for (nextx = NULL, insn_cnt2 = 1, x = get_last_insn ();
        x != 0;
        nextx = x, insn_cnt2++, x = PREV_INSN (x))
-    gcc_assert (NEXT_INSN (x) == nextx);
+    if (NEXT_INSN (x) != nextx)
+      abort ();
 
-  gcc_assert (insn_cnt1 == insn_cnt2);
+  if (insn_cnt1 != insn_cnt2)
+    abort ();
 }
 
-/* If we have assembler epilogues, the block falling through to exit must
-   be the last one in the reordered chain when we reach final.  Ensure
-   that this condition is met.  */
+/* The block falling through to exit must be the last one in the
+   reordered chain.  Ensure that this condition is met.  */
 static void
 fixup_fallthru_exit_predecessor (void)
 {
   edge e;
-  edge_iterator ei;
   basic_block bb = NULL;
 
-  /* This transformation is not valid before reload, because we might
-     separate a call from the instruction that copies the return
-     value.  */
-  gcc_assert (reload_completed);
-
-  FOR_EACH_EDGE (e, ei, EXIT_BLOCK_PTR->preds)
+  for (e = EXIT_BLOCK_PTR->pred; e; e = e->pred_next)
     if (e->flags & EDGE_FALLTHRU)
       bb = e->src;
 
@@ -930,18 +853,6 @@ fixup_fallthru_exit_predecessor (void)
     {
       basic_block c = ENTRY_BLOCK_PTR->next_bb;
 
-      /* If the very first block is the one with the fall-through exit
-	 edge, we have to split that block.  */
-      if (c == bb)
-	{
-	  bb = split_block (bb, NULL)->dest;
-	  initialize_bb_rbi (bb);
-	  bb->rbi->next = c->rbi->next;
-	  c->rbi->next = bb;
-	  bb->rbi->footer = c->rbi->footer;
-	  c->rbi->footer = NULL;
-	}
-
       while (c->rbi->next != bb)
 	c = c->rbi->next;
 
@@ -956,30 +867,35 @@ fixup_fallthru_exit_predecessor (void)
 
 /* Return true in case it is possible to duplicate the basic block BB.  */
 
-/* We do not want to declare the function in a header file, since it should
-   only be used through the cfghooks interface, and we do not want to move
-   it to cfgrtl.c since it would require also moving quite a lot of related
-   code.  */
-extern bool cfg_layout_can_duplicate_bb_p (basic_block);
-
 bool
 cfg_layout_can_duplicate_bb_p (basic_block bb)
 {
+  edge s;
+
+  if (bb == EXIT_BLOCK_PTR || bb == ENTRY_BLOCK_PTR)
+    return false;
+
+  /* Duplicating fallthru block to exit would require adding a jump
+     and splitting the real last BB.  */
+  for (s = bb->succ; s; s = s->succ_next)
+    if (s->dest == EXIT_BLOCK_PTR && s->flags & EDGE_FALLTHRU)
+       return false;
+
   /* Do not attempt to duplicate tablejumps, as we need to unshare
      the dispatch table.  This is difficult to do, as the instructions
      computing jump destination may be hoisted outside the basic block.  */
-  if (tablejump_p (BB_END (bb), NULL, NULL))
+  if (tablejump_p (bb->end, NULL, NULL))
     return false;
 
   /* Do not duplicate blocks containing insns that can't be copied.  */
   if (targetm.cannot_copy_insn_p)
     {
-      rtx insn = BB_HEAD (bb);
+      rtx insn = bb->head;
       while (1)
 	{
-	  if (INSN_P (insn) && targetm.cannot_copy_insn_p (insn))
+	  if (INSN_P (insn) && (*targetm.cannot_copy_insn_p) (insn))
 	    return false;
-	  if (insn == BB_END (bb))
+	  if (insn == bb->end)
 	    break;
 	  insn = NEXT_INSN (insn);
 	}
@@ -988,7 +904,7 @@ cfg_layout_can_duplicate_bb_p (basic_block bb)
   return true;
 }
 
-rtx
+static rtx
 duplicate_insn_chain (rtx from, rtx to)
 {
   rtx insn, last;
@@ -1029,6 +945,8 @@ duplicate_insn_chain (rtx from, rtx to)
 	         in first BB, we may want to copy the block.  */
 	    case NOTE_INSN_PROLOGUE_END:
 
+	    case NOTE_INSN_LOOP_VTOP:
+	    case NOTE_INSN_LOOP_CONT:
 	    case NOTE_INSN_LOOP_BEG:
 	    case NOTE_INSN_LOOP_END:
 	      /* Strip down the loop notes - we don't really want to keep
@@ -1047,51 +965,62 @@ duplicate_insn_chain (rtx from, rtx to)
 	    case NOTE_INSN_BASIC_BLOCK:
 	      break;
 
+	      /* There is no purpose to duplicate prologue.  */
+	    case NOTE_INSN_BLOCK_BEG:
+	    case NOTE_INSN_BLOCK_END:
+	      /* The BLOCK_BEG/BLOCK_END notes should be eliminated when BB
+	         reordering is in the progress.  */
+	    case NOTE_INSN_EH_REGION_BEG:
+	    case NOTE_INSN_EH_REGION_END:
+	      /* Should never exist at BB duplication time.  */
+	      abort ();
+	      break;
 	    case NOTE_INSN_REPEATED_LINE_NUMBER:
-	    case NOTE_INSN_UNLIKELY_EXECUTED_CODE:
 	      emit_note_copy (insn);
 	      break;
 
 	    default:
-	      /* All other notes should have already been eliminated.
-	       */
-	      gcc_assert (NOTE_LINE_NUMBER (insn) >= 0);
-	      
+	      if (NOTE_LINE_NUMBER (insn) < 0)
+		abort ();
 	      /* It is possible that no_line_number is set and the note
 	         won't be emitted.  */
 	      emit_note_copy (insn);
 	    }
 	  break;
 	default:
-	  gcc_unreachable ();
+	  abort ();
 	}
     }
   insn = NEXT_INSN (last);
   delete_insn (last);
   return insn;
 }
-/* Create a duplicate of the basic block BB.  */
+/* Create a duplicate of the basic block BB and redirect edge E into it.
+   If E is not specified, BB is just copied, but updating the frequencies
+   etc. is left to the caller.  */
 
-/* We do not want to declare the function in a header file, since it should
-   only be used through the cfghooks interface, and we do not want to move
-   it to cfgrtl.c since it would require also moving quite a lot of related
-   code.  */
-extern basic_block cfg_layout_duplicate_bb (basic_block);
-
-/* APPLE LOCAL lno */
-extern basic_block cfg_layout_duplicate_bb (basic_block);
 basic_block
-cfg_layout_duplicate_bb (basic_block bb)
+cfg_layout_duplicate_bb (basic_block bb, edge e)
 {
   rtx insn;
+  edge s, n;
   basic_block new_bb;
+  gcov_type new_count = e ? e->count : 0;
 
-  insn = duplicate_insn_chain (BB_HEAD (bb), BB_END (bb));
+  if (bb->count < new_count)
+    new_count = bb->count;
+  if (!bb->pred)
+    abort ();
+#ifdef ENABLE_CHECKING
+  if (!cfg_layout_can_duplicate_bb_p (bb))
+    abort ();
+#endif
+
+  insn = duplicate_insn_chain (bb->head, bb->end);
   new_bb = create_basic_block (insn,
 			       insn ? get_last_insn () : NULL,
 			       EXIT_BLOCK_PTR->prev_bb);
 
-  BB_COPY_PARTITION (new_bb, bb);
   if (bb->rbi->header)
     {
       insn = bb->rbi->header;
@@ -1114,58 +1043,109 @@ cfg_layout_duplicate_bb (basic_block bb)
 
   if (bb->global_live_at_start)
     {
-      new_bb->global_live_at_start = ALLOC_REG_SET (&reg_obstack);
-      new_bb->global_live_at_end = ALLOC_REG_SET (&reg_obstack);
+      new_bb->global_live_at_start = OBSTACK_ALLOC_REG_SET (&flow_obstack);
+      new_bb->global_live_at_end = OBSTACK_ALLOC_REG_SET (&flow_obstack);
       COPY_REG_SET (new_bb->global_live_at_start, bb->global_live_at_start);
       COPY_REG_SET (new_bb->global_live_at_end, bb->global_live_at_end);
     }
 
+  new_bb->loop_depth = bb->loop_depth;
+  new_bb->flags = bb->flags;
+  for (s = bb->succ; s; s = s->succ_next)
+    {
+      /* Since we are creating edges from a new block to successors
+	 of another block (which therefore are known to be disjoint), there
+	 is no need to actually check for duplicated edges.  */
+      n = unchecked_make_edge (new_bb, s->dest, s->flags);
+      n->probability = s->probability;
+      if (e && bb->count)
+	{
+	  /* Take care for overflows!  */
+	  n->count = s->count * (new_count * 10000 / bb->count) / 10000;
+	  s->count -= n->count;
+	}
+      else
+	n->count = s->count;
+      n->aux = s->aux;
+    }
+
+  if (e)
+    {
+      new_bb->count = new_count;
+      bb->count -= new_count;
+
+      new_bb->frequency = EDGE_FREQUENCY (e);
+      bb->frequency -= EDGE_FREQUENCY (e);
+
+      redirect_edge_and_branch_force (e, new_bb);
+
+      if (bb->count < 0)
+	bb->count = 0;
+      if (bb->frequency < 0)
+	bb->frequency = 0;
+    }
+  else
+    {
+      new_bb->count = bb->count;
+      new_bb->frequency = bb->frequency;
+    }
+
+  new_bb->rbi->original = bb;
+  bb->rbi->copy = new_bb;
+
   return new_bb;
 }
 
+void
+cfg_layout_initialize_rbi (basic_block bb)
+{
+  if (bb->rbi)
+    abort ();
+  bb->rbi = pool_alloc (cfg_layout_pool);
+  memset (bb->rbi, 0, sizeof (struct reorder_block_def));
+}
+
 /* Main entry point to this module - initialize the datastructures for
-   CFG layout changes.  It keeps LOOPS up-to-date if not null.
-
-   FLAGS is a set of additional flags to pass to cleanup_cfg().  It should
-   include CLEANUP_UPDATE_LIFE if liveness information must be kept up
-   to date.  */
+   CFG layout changes.  It keeps LOOPS up-to-date if not null.  */
 
 void
-cfg_layout_initialize (unsigned int flags)
+cfg_layout_initialize (void)
 {
   basic_block bb;
 
-  /* Our algorithm depends on fact that there are no dead jumptables
+  /* Our algorithm depends on fact that there are now dead jumptables
      around the code.  */
-  alloc_rbi_pool ();
-
+  cfg_layout_pool =
+    create_alloc_pool ("cfg layout pool", sizeof (struct reorder_block_def),
+		       n_basic_blocks + 2);
   FOR_BB_BETWEEN (bb, ENTRY_BLOCK_PTR, NULL, next_bb)
-    initialize_bb_rbi (bb);
+    cfg_layout_initialize_rbi (bb);
 
   cfg_layout_rtl_register_cfg_hooks ();
 
   record_effective_endpoints ();
 
-  cleanup_cfg (CLEANUP_CFGLAYOUT | flags);
+  cleanup_cfg (CLEANUP_CFGLAYOUT);
 }
 
 /* Splits superblocks.  */
-void
+static void
 break_superblocks (void)
 {
   sbitmap superblocks;
-  bool need = false;
-  basic_block bb;
+  int i, need;
 
-  superblocks = sbitmap_alloc (last_basic_block);
+  superblocks = sbitmap_alloc (n_basic_blocks);
   sbitmap_zero (superblocks);
 
-  FOR_EACH_BB (bb)
-    if (bb->flags & BB_SUPERBLOCK)
+  need = 0;
+
+  for (i = 0; i < n_basic_blocks; i++)
+    if (BASIC_BLOCK(i)->flags & BB_SUPERBLOCK)
       {
-	bb->flags &= ~BB_SUPERBLOCK;
-	SET_BIT (superblocks, bb->index);
-	need = true;
+	BASIC_BLOCK(i)->flags &= ~BB_SUPERBLOCK;
+	SET_BIT (superblocks, i);
+	need = 1;
       }
 
   if (need)
@@ -1189,19 +1169,14 @@ cfg_layout_finalize (void)
   verify_flow_info ();
 #endif
   rtl_register_cfg_hooks ();
-  if (reload_completed
-#ifdef HAVE_epilogue
-      && !HAVE_epilogue
-#endif
-      )
-    fixup_fallthru_exit_predecessor ();
+  fixup_fallthru_exit_predecessor ();
   fixup_reorder_chain ();
 
 #ifdef ENABLE_CHECKING
   verify_insn_chain ();
 #endif
-  
-  free_rbi_pool ();
+
+  free_alloc_pool (cfg_layout_pool);
   FOR_BB_BETWEEN (bb, ENTRY_BLOCK_PTR, NULL, next_bb)
     bb->rbi = NULL;
 
@@ -1226,8 +1201,7 @@ can_copy_bbs_p (basic_block *bbs, unsigned n)
   for (i = 0; i < n; i++)
     {
       /* In case we should redirect abnormal edge during duplication, fail.  */
-      edge_iterator ei;
-      FOR_EACH_EDGE (e, ei, bbs[i]->succs)
+      for (e = bbs[i]->succ; e; e = e->succ_next)
 	if ((e->flags & EDGE_ABNORMAL)
 	    && e->dest->rbi->duplicated)
 	  {
@@ -1235,7 +1209,7 @@ can_copy_bbs_p (basic_block *bbs, unsigned n)
 	    goto end;
 	  }
 
-      if (!can_duplicate_block_p (bbs[i]))
+      if (!cfg_layout_can_duplicate_bb_p (bbs[i]))
 	{
 	  ret = false;
 	  break;
@@ -1267,7 +1241,7 @@ end:
 void
 copy_bbs (basic_block *bbs, unsigned n, basic_block *new_bbs,
 	  edge *edges, unsigned n_edges, edge *new_edges,
-	  struct loop *base)
+	  struct loop *base, struct loops *loops)
 {
   unsigned i, j;
   basic_block bb, new_bb, dom_bb;
@@ -1278,10 +1252,11 @@ copy_bbs (basic_block *bbs, unsigned n, basic_block *new_bbs,
     {
       /* Duplicate.  */
       bb = bbs[i];
-      new_bb = new_bbs[i] = duplicate_block (bb, NULL);
+      new_bb = new_bbs[i] = cfg_layout_duplicate_bb (bb, NULL);
       bb->rbi->duplicated = 1;
       /* Add to loop.  */
       add_bb_to_loop (new_bb, bb->loop_father->copy);
+      add_to_dominance_info (loops->cfg.dom, new_bb);
       /* Possibly set header.  */
       if (bb->loop_father->header == bb && bb->loop_father != base)
 	new_bb->loop_father->header = new_bb;
@@ -1296,11 +1271,11 @@ copy_bbs (basic_block *bbs, unsigned n, basic_block *new_bbs,
       bb = bbs[i];
       new_bb = new_bbs[i];
 
-      dom_bb = get_immediate_dominator (CDI_DOMINATORS, bb);
+      dom_bb = get_immediate_dominator (loops->cfg.dom, bb);
       if (dom_bb->rbi->duplicated)
 	{
 	  dom_bb = dom_bb->rbi->copy;
-	  set_immediate_dominator (CDI_DOMINATORS, new_bb, dom_bb);
+	  set_immediate_dominator (loops->cfg.dom, new_bb, dom_bb);
 	}
     }
 
@@ -1309,11 +1284,10 @@ copy_bbs (basic_block *bbs, unsigned n, basic_block *new_bbs,
     new_edges[j] = NULL;
   for (i = 0; i < n; i++)
     {
-      edge_iterator ei;
       new_bb = new_bbs[i];
       bb = bbs[i];
 
-      FOR_EACH_EDGE (e, ei, new_bb->succs)
+      for (e = new_bb->succ; e; e = e->succ_next)
 	{
 	  for (j = 0; j < n_edges; j++)
 	    if (edges[j] && edges[j]->src == bb && edges[j]->dest == e->dest)
