@@ -1,6 +1,5 @@
 /* Definitions of target machine GNU compiler.  IA-64 version.
-   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005
-   Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2003 Free Software Foundation, Inc.
    Contributed by James E. Wilson <wilson@cygnus.com> and
    		  David Mosberger <davidm@hpl.hp.com>.
 
@@ -44,13 +43,8 @@ do {						\
 	  builtin_define("__BIG_ENDIAN__");	\
 } while (0)
 
-#ifndef SUBTARGET_EXTRA_SPECS
-#define SUBTARGET_EXTRA_SPECS
-#endif
-
 #define EXTRA_SPECS \
-  { "asm_extra", ASM_EXTRA_SPEC }, \
-  SUBTARGET_EXTRA_SPECS
+  { "asm_extra", ASM_EXTRA_SPEC },
 
 #define CC1_SPEC "%(cc1_cpu) "
 
@@ -75,6 +69,8 @@ extern int target_flags;
 
 #define MASK_ILP32      0x00000020      /* Generate ILP32 code.  */
 
+#define MASK_B_STEP	0x00000040	/* Emit code for Itanium B step.  */
+
 #define MASK_REG_NAMES	0x00000080	/* Use in/loc/out register names.  */
 
 #define MASK_NO_SDATA   0x00000100	/* Disable sdata/scommon/sbss.  */
@@ -87,17 +83,13 @@ extern int target_flags;
 
 #define MASK_INLINE_FLOAT_DIV_THR 0x00001000 /* inline div, max throughput.  */
 
-#define MASK_INLINE_INT_DIV_LAT   0x00002000 /* inline div, min latency.  */
+#define MASK_INLINE_INT_DIV_LAT   0x00000800 /* inline div, min latency.  */
 
-#define MASK_INLINE_INT_DIV_THR   0x00004000 /* inline div, max throughput.  */
+#define MASK_INLINE_INT_DIV_THR   0x00001000 /* inline div, max throughput.  */
 
-#define MASK_INLINE_SQRT_LAT      0x00008000 /* inline sqrt, min latency.  */
+#define MASK_DWARF2_ASM 0x40000000	/* test dwarf2 line info via gas.  */
 
-#define MASK_INLINE_SQRT_THR      0x00010000 /* inline sqrt, max throughput.  */
-
-#define MASK_DWARF2_ASM 0x00020000	/* test dwarf2 line info via gas.  */
-
-#define MASK_EARLY_STOP_BITS 0x00040000 /* tune stop bits for the model.  */
+#define MASK_EARLY_STOP_BITS 0x00002000 /* tune stop bits for the model.  */
 
 #define TARGET_BIG_ENDIAN	(target_flags & MASK_BIG_ENDIAN)
 
@@ -111,6 +103,8 @@ extern int target_flags;
 
 #define TARGET_ILP32            (target_flags & MASK_ILP32)
 
+#define TARGET_B_STEP		(target_flags & MASK_B_STEP)
+
 #define TARGET_REG_NAMES	(target_flags & MASK_REG_NAMES)
 
 #define TARGET_NO_SDATA		(target_flags & MASK_NO_SDATA)
@@ -133,35 +127,14 @@ extern int target_flags;
 #define TARGET_INLINE_INT_DIV \
   (target_flags & (MASK_INLINE_INT_DIV_LAT | MASK_INLINE_INT_DIV_THR))
 
-#define TARGET_INLINE_SQRT_LAT (target_flags & MASK_INLINE_SQRT_LAT)
-
-#define TARGET_INLINE_SQRT_THR (target_flags & MASK_INLINE_SQRT_THR)
-
-#define TARGET_INLINE_SQRT \
-  (target_flags & (MASK_INLINE_SQRT_LAT | MASK_INLINE_SQRT_THR))
-
 #define TARGET_DWARF2_ASM	(target_flags & MASK_DWARF2_ASM)
 
-/* Variables which are this size or smaller are put in the sdata/sbss
-   sections.  */
-extern unsigned int ia64_section_threshold;
-
-/* If the assembler supports thread-local storage, assume that the
-   system does as well.  If a particular target system has an
-   assembler that supports TLS -- but the rest of the system does not
-   support TLS -- that system should explicit define TARGET_HAVE_TLS
-   to false in its own configuration file.  */
-#if !defined(TARGET_HAVE_TLS) && defined(HAVE_AS_TLS)
-#define TARGET_HAVE_TLS true
-#endif
-
 extern int ia64_tls_size;
 #define TARGET_TLS14		(ia64_tls_size == 14)
 #define TARGET_TLS22		(ia64_tls_size == 22)
 #define TARGET_TLS64		(ia64_tls_size == 64)
 #define TARGET_EARLY_STOP_BITS	(target_flags & MASK_EARLY_STOP_BITS)
 
-#define TARGET_HPUX		0
 #define TARGET_HPUX_LD		0
 
 #ifndef HAVE_AS_LTOFFX_LDXMOV_RELOCS
@@ -192,6 +165,8 @@ extern int ia64_tls_size;
       N_("Emit stop bits before and after volatile extended asms") },	\
   { "no-volatile-asm-stop", -MASK_VOL_ASM_STOP,				\
       N_("Don't emit stop bits before and after volatile extended asms") }, \
+  { "b-step",		MASK_B_STEP,					\
+      N_("Emit code for Itanium (TM) processor B step")},		\
   { "register-names",	MASK_REG_NAMES,					\
       N_("Use in/loc/out register names")},				\
   { "no-sdata",		MASK_NO_SDATA,					\
@@ -206,21 +181,10 @@ extern int ia64_tls_size;
       N_("Generate inline floating point division, optimize for latency") },\
   { "inline-float-divide-max-throughput", MASK_INLINE_FLOAT_DIV_THR,	\
       N_("Generate inline floating point division, optimize for throughput") },\
-  { "no-inline-float-divide", 						\
-      -(MASK_INLINE_FLOAT_DIV_LAT|MASK_INLINE_FLOAT_DIV_THR),		\
-      N_("Do not inline floating point division") },			\
   { "inline-int-divide-min-latency", MASK_INLINE_INT_DIV_LAT,		\
       N_("Generate inline integer division, optimize for latency") },	\
   { "inline-int-divide-max-throughput", MASK_INLINE_INT_DIV_THR,	\
       N_("Generate inline integer division, optimize for throughput") },\
-  { "no-inline-int-divide", -(MASK_INLINE_INT_DIV_LAT|MASK_INLINE_INT_DIV_THR),	\
-      N_("Do not inline integer division") },				\
-  { "inline-sqrt-min-latency", MASK_INLINE_SQRT_LAT,			\
-      N_("Generate inline square root, optimize for latency") },	\
-  { "inline-sqrt-max-throughput", MASK_INLINE_SQRT_THR,			\
-      N_("Generate inline square root, optimize for throughput") },     \
-  { "no-inline-sqrt", -(MASK_INLINE_SQRT_LAT|MASK_INLINE_SQRT_THR),	\
-      N_("Do not inline square root") },				\
   { "dwarf2-asm", 	MASK_DWARF2_ASM,				\
       N_("Enable Dwarf 2 line debug info via GNU as")},			\
   { "no-dwarf2-asm", 	-MASK_DWARF2_ASM,				\
@@ -237,7 +201,7 @@ extern int ia64_tls_size;
 /* Default target_flags if no switches are specified  */
 
 #ifndef TARGET_DEFAULT
-#define TARGET_DEFAULT (MASK_DWARF2_ASM | MASK_INLINE_FLOAT_DIV_THR)
+#define TARGET_DEFAULT MASK_DWARF2_ASM
 #endif
 
 #ifndef TARGET_CPU_DEFAULT
@@ -261,7 +225,7 @@ extern const char *ia64_tls_size_string;
 
 enum processor_type
 {
-  PROCESSOR_ITANIUM,			/* Original Itanium.  */
+  PROCESSOR_ITANIUM,			/* Original Itanium. */
   PROCESSOR_ITANIUM2,
   PROCESSOR_max
 };
@@ -297,16 +261,16 @@ extern const char *ia64_tune_string;
 
 /* Driver configuration */
 
-/* A C string constant that tells the GCC driver program options to pass to
-   `cc1'.  It can also specify how to translate options you give to GCC into
-   options for GCC to pass to the `cc1'.  */
+/* A C string constant that tells the GNU CC driver program options to pass to
+   `cc1'.  It can also specify how to translate options you give to GNU CC into
+   options for GNU CC to pass to the `cc1'.  */
 
 #undef CC1_SPEC
 #define CC1_SPEC "%{G*}"
 
-/* A C string constant that tells the GCC driver program options to pass to
-   `cc1plus'.  It can also specify how to translate options you give to GCC
-   into options for GCC to pass to the `cc1plus'.  */
+/* A C string constant that tells the GNU CC driver program options to pass to
+   `cc1plus'.  It can also specify how to translate options you give to GNU CC
+   into options for GNU CC to pass to the `cc1plus'.  */
 
 /* #define CC1PLUS_SPEC "" */
 
@@ -332,8 +296,6 @@ extern const char *ia64_tune_string;
 
 #define UNITS_PER_WORD 8
 
-#define UNITS_PER_SIMD_WORD UNITS_PER_WORD
-
 #define POINTER_SIZE (TARGET_ILP32 ? 32 : 64)
 
 /* A C expression whose value is zero if pointers that need to be extended
@@ -357,6 +319,12 @@ do									\
   }									\
 while (0)
 
+/* ??? ABI doesn't allow us to define this.  */
+/* #define PROMOTE_FUNCTION_ARGS */
+
+/* ??? ABI doesn't allow us to define this.  */
+/* #define PROMOTE_FUNCTION_RETURN */
+
 #define PARM_BOUNDARY 64
 
 /* Define this macro if you wish to preserve a certain alignment for the stack
@@ -422,6 +390,7 @@ while (0)
    For reasons known only to HP, the vtable entries (as opposed to
    normal function descriptors) are 16 bytes wide in 32-bit mode as
    well, even though the 3rd and 4th words are unused.  */
+#if 0 /* FIXME: DISABLED BY LLVM */
 #define TARGET_VTABLE_USES_DESCRIPTORS (TARGET_ILP32 ? 4 : 2)
 
 /* Due to silliness in the HPUX linker, vtable entries must be
@@ -432,6 +401,7 @@ while (0)
 /* Due to the above, we need extra padding for the data entries below 0
    to retain the alignment of the descriptors.  */
 #define TARGET_VTABLE_DATA_ENTRY_DISTANCE (TARGET_ILP32 ? 2 : 1)
+#endif  /* end of fixme */
 
 /* Layout of Source Language Data Types */
 
@@ -441,17 +411,19 @@ while (0)
 
 #define LONG_TYPE_SIZE (TARGET_ILP32 ? 32 : 64)
 
+#define MAX_LONG_TYPE_SIZE 64
+
 #define LONG_LONG_TYPE_SIZE 64
 
 #define FLOAT_TYPE_SIZE 32
 
 #define DOUBLE_TYPE_SIZE 64
 
-/* long double is XFmode normally, TFmode for HPUX.  */
-#define LONG_DOUBLE_TYPE_SIZE (TARGET_HPUX ? 128 : 80)
+#define LONG_DOUBLE_TYPE_SIZE 128
 
-/* We always want the XFmode operations from libgcc2.c.  */
-#define LIBGCC2_LONG_DOUBLE_TYPE_SIZE 80
+/* By default we use the 80-bit Intel extended float format packaged
+   in a 128-bit entity.  */
+#define INTEL_EXTENDED_IEEE_FORMAT 1
 
 #define DEFAULT_SIGNED_CHAR 1
 
@@ -620,7 +592,7 @@ while (0)
 
 #define CALL_REALLY_USED_REGISTERS \
 { /* General registers.  */				\
-  0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 1, 1,	\
+  1, 1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 1, 1,	\
   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,	\
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,	\
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,	\
@@ -629,7 +601,7 @@ while (0)
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,	\
   0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1,	\
   /* Floating-point registers.  */			\
-  0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,	\
+  1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,	\
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,	\
   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,	\
   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,	\
@@ -638,7 +610,7 @@ while (0)
   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,	\
   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,	\
   /* Predicate registers.  */				\
-  0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,	\
+  1, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,	\
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,	\
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,	\
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,	\
@@ -681,7 +653,7 @@ while (0)
 /* Order of allocation of registers */
 
 /* If defined, an initializer for a vector of integers, containing the numbers
-   of hard registers in the order in which GCC should prefer to use them
+   of hard registers in the order in which GNU CC should prefer to use them
    (from most preferred to least).
 
    If this macro is not defined, registers are used lowest numbered first (all
@@ -809,8 +781,7 @@ while (0)
   ((REGNO) == PR_REG (0) && (MODE) == DImode ? 64			\
    : PR_REGNO_P (REGNO) && (MODE) == BImode ? 2				\
    : PR_REGNO_P (REGNO) && (MODE) == CCImode ? 1			\
-   : FR_REGNO_P (REGNO) && (MODE) == XFmode ? 1				\
-   : FR_REGNO_P (REGNO) && (MODE) == XCmode ? 2				\
+   : FR_REGNO_P (REGNO) && (MODE) == TFmode && INTEL_EXTENDED_IEEE_FORMAT ? 1 \
    : (GET_MODE_SIZE (MODE) + UNITS_PER_WORD - 1) / UNITS_PER_WORD)
 
 /* A C expression that is nonzero if it is permissible to store a value of mode
@@ -822,11 +793,10 @@ while (0)
      GET_MODE_CLASS (MODE) != MODE_CC &&			\
      (MODE) != TImode &&					\
      (MODE) != BImode &&					\
-     (MODE) != TFmode 						\
+     ((MODE) != TFmode || INTEL_EXTENDED_IEEE_FORMAT) 		\
    : PR_REGNO_P (REGNO) ?					\
      (MODE) == BImode || GET_MODE_CLASS (MODE) == MODE_CC	\
-   : GR_REGNO_P (REGNO) ?					\
-     (MODE) != CCImode && (MODE) != XFmode && (MODE) != XCmode	\
+   : GR_REGNO_P (REGNO) ? (MODE) != CCImode && (MODE) != TFmode	\
    : AR_REGNO_P (REGNO) ? (MODE) == DImode			\
    : BR_REGNO_P (REGNO) ? (MODE) == DImode			\
    : 0)
@@ -839,20 +809,12 @@ while (0)
    ever different for any R, then `MODES_TIEABLE_P (MODE1, MODE2)' must be
    zero.  */
 /* Don't tie integer and FP modes, as that causes us to get integer registers
-   allocated for FP instructions.  XFmode only supported in FP registers so
+   allocated for FP instructions.  TFmode only supported in FP registers so
    we can't tie it with any other modes.  */
 #define MODES_TIEABLE_P(MODE1, MODE2)			\
   (GET_MODE_CLASS (MODE1) == GET_MODE_CLASS (MODE2)	\
-   && ((((MODE1) == XFmode) || ((MODE1) == XCmode))	\
-       == (((MODE2) == XFmode) || ((MODE2) == XCmode)))	\
+   && (((MODE1) == TFmode) == ((MODE2) == TFmode))	\
    && (((MODE1) == BImode) == ((MODE2) == BImode)))
-
-/* Specify the modes required to caller save a given hard regno.
-   We need to ensure floating pt regs are not saved as DImode.  */
-
-#define HARD_REGNO_CALLER_SAVE_MODE(REGNO, NREGS, MODE) \
-  ((FR_REGNO_P (REGNO) && (NREGS) == 1) ? XFmode        \
-   : choose_hard_reg_mode ((REGNO), (NREGS), false))
 
 /* Handling Leaf Functions */
 
@@ -1020,8 +982,18 @@ enum reg_class
    The value is a register class; perhaps CLASS, or perhaps another, smaller
    class.  */
 
+/* Don't allow volatile mem reloads into floating point registers.  This
+   is defined to force reload to choose the r/m case instead of the f/f case
+   when reloading (set (reg fX) (mem/v)).
+
+   Do not reload expressions into AR regs.  */
+
 #define PREFERRED_RELOAD_CLASS(X, CLASS) \
-  ia64_preferred_reload_class (X, CLASS)
+  (CLASS == FR_REGS && GET_CODE (X) == MEM && MEM_VOLATILE_P (X) ? NO_REGS   \
+   : CLASS == FR_REGS && GET_CODE (X) == CONST_DOUBLE ? NO_REGS		     \
+   : GET_RTX_CLASS (GET_CODE (X)) != 'o'				     \
+     && (CLASS == AR_M_REGS || CLASS == AR_I_REGS) ? NO_REGS		     \
+   : CLASS)
 
 /* You should define this macro to indicate to the reload phase that it may
    need to allocate at least one register for a reload in addition to the
@@ -1041,14 +1013,13 @@ enum reg_class
    into a register of CLASS2.  */
 
 #if 0
-/* ??? May need this, but since we've disallowed XFmode in GR_REGS,
+/* ??? May need this, but since we've disallowed TFmode in GR_REGS,
    I'm not quite sure how it could be invoked.  The normal problems
    with unions should be solved with the addressof fiddling done by
-   movxf and friends.  */
+   movtf and friends.  */
 #define SECONDARY_MEMORY_NEEDED(CLASS1, CLASS2, MODE)			\
-  (((MODE) == XFmode || (MODE) == XCmode)				\
-   && (((CLASS1) == GR_REGS && (CLASS2) == FR_REGS)			\
-       || ((CLASS1) == FR_REGS && (CLASS2) == GR_REGS)))
+  ((MODE) == TFmode && (((CLASS1) == GR_REGS && (CLASS2) == FR_REGS)	\
+			|| ((CLASS1) == FR_REGS && (CLASS2) == GR_REGS)))
 #endif
 
 /* A C expression for the maximum number of consecutive registers of
@@ -1057,8 +1028,7 @@ enum reg_class
 
 #define CLASS_MAX_NREGS(CLASS, MODE) \
   ((MODE) == BImode && (CLASS) == PR_REGS ? 2			\
-   : ((CLASS) == FR_REGS && (MODE) == XFmode) ? 1		\
-   : ((CLASS) == FR_REGS && (MODE) == XCmode) ? 2		\
+   : ((CLASS) == FR_REGS && (MODE) == TFmode) ? 1		\
    : (GET_MODE_SIZE (MODE) + UNITS_PER_WORD - 1) / UNITS_PER_WORD)
 
 /* In FP regs, we can't change FP values to integer values and vice
@@ -1092,7 +1062,15 @@ enum reg_class
 #define CONST_OK_FOR_P(VALUE) ((VALUE) == 0 || (VALUE) == -1)
 
 #define CONST_OK_FOR_LETTER_P(VALUE, C) \
-  ia64_const_ok_for_letter_p (VALUE, C)
+((C) == 'I' ? CONST_OK_FOR_I (VALUE)		\
+ : (C) == 'J' ? CONST_OK_FOR_J (VALUE)		\
+ : (C) == 'K' ? CONST_OK_FOR_K (VALUE)		\
+ : (C) == 'L' ? CONST_OK_FOR_L (VALUE)		\
+ : (C) == 'M' ? CONST_OK_FOR_M (VALUE)		\
+ : (C) == 'N' ? CONST_OK_FOR_N (VALUE)		\
+ : (C) == 'O' ? CONST_OK_FOR_O (VALUE)		\
+ : (C) == 'P' ? CONST_OK_FOR_P (VALUE)		\
+ : 0)
 
 /* A C expression that defines the machine-dependent operand constraint letters
    (`G', `H') that specify particular ranges of `const_double' values.  */
@@ -1103,14 +1081,33 @@ enum reg_class
    || (VALUE) == CONST1_RTX (GET_MODE (VALUE)))
 
 #define CONST_DOUBLE_OK_FOR_LETTER_P(VALUE, C) \
-  ia64_const_double_ok_for_letter_p (VALUE, C)
+  ((C) == 'G' ? CONST_DOUBLE_OK_FOR_G (VALUE) : 0)
 
 /* A C expression that defines the optional machine-dependent constraint
    letters (`Q', `R', `S', `T', `U') that can be used to segregate specific
    types of operands, usually memory references, for the target machine.  */
 
+/* Non-volatile memory for FP_REG loads/stores.  */
+#define CONSTRAINT_OK_FOR_Q(VALUE) \
+  (memory_operand((VALUE), VOIDmode) && ! MEM_VOLATILE_P (VALUE))
+/* 1..4 for shladd arguments.  */
+#define CONSTRAINT_OK_FOR_R(VALUE) \
+  (GET_CODE (VALUE) == CONST_INT && INTVAL (VALUE) >= 1 && INTVAL (VALUE) <= 4)
+/* Non-post-inc memory for asms and other unsavory creatures.  */
+#define CONSTRAINT_OK_FOR_S(VALUE)				\
+  (GET_CODE (VALUE) == MEM					\
+   && GET_RTX_CLASS (GET_CODE (XEXP ((VALUE), 0))) != 'a'	\
+   && (reload_in_progress || memory_operand ((VALUE), VOIDmode)))
+/* Symbol ref to small-address-area: */
+#define CONSTRAINT_OK_FOR_T(VALUE)						\
+	(GET_CODE (VALUE) == SYMBOL_REF && SYMBOL_REF_SMALL_ADDR_P (VALUE))
+
 #define EXTRA_CONSTRAINT(VALUE, C) \
-  ia64_extra_constraint (VALUE, C)
+  ((C) == 'Q' ? CONSTRAINT_OK_FOR_Q (VALUE)	\
+   : (C) == 'R' ? CONSTRAINT_OK_FOR_R (VALUE)	\
+   : (C) == 'S' ? CONSTRAINT_OK_FOR_S (VALUE)	\
+   : (C) == 'T' ? CONSTRAINT_OK_FOR_T (VALUE)	\
+   : 0)
 
 /* Basic Stack Layout */
 
@@ -1247,6 +1244,13 @@ enum reg_class
 
 /* Passing Function Arguments on the Stack */
 
+/* Define this macro if an argument declared in a prototype as an integral type
+   smaller than `int' should actually be passed as an `int'.  In addition to
+   avoiding errors in certain cases of mismatch, it also makes for better code
+   on certain machines.  */
+/* ??? Investigate.  */
+/* #define PROMOTE_PROTOTYPES */
+
 /* If defined, the maximum amount of space required for outgoing arguments will
    be computed and placed into the variable
    `current_function_outgoing_args_size'.  */
@@ -1285,6 +1289,22 @@ enum reg_class
 #define FUNCTION_INCOMING_ARG(CUM, MODE, TYPE, NAMED) \
   ia64_function_arg (&CUM, MODE, TYPE, NAMED, 1)
 
+/* A C expression for the number of words, at the beginning of an argument,
+   must be put in registers.  The value must be zero for arguments that are
+   passed entirely in registers or that are entirely pushed on the stack.  */
+
+#define FUNCTION_ARG_PARTIAL_NREGS(CUM, MODE, TYPE, NAMED) \
+ ia64_function_arg_partial_nregs (&CUM, MODE, TYPE, NAMED)
+
+/* A C expression that indicates when an argument must be passed by reference.
+   If nonzero for an argument, a copy of that argument is made in memory and a
+   pointer to the argument is passed instead of the argument itself.  The
+   pointer is passed in whatever way is appropriate for passing a pointer to
+   that type.  */
+
+#define FUNCTION_ARG_PASS_BY_REFERENCE(CUM, MODE, TYPE, NAMED) \
+  ia64_function_arg_pass_by_reference (&CUM, MODE, TYPE, NAMED)
+
 /* A C type for declaring a variable that is used as the first argument of
    `FUNCTION_ARG' and other related values.  For some target machines, the type
    `int' suffices and can hold the number of bytes of argument so far.  */
@@ -1300,7 +1320,7 @@ typedef struct ia64_args
 /* A C statement (sans semicolon) for initializing the variable CUM for the
    state at the beginning of the argument list.  */
 
-#define INIT_CUMULATIVE_ARGS(CUM, FNTYPE, LIBNAME, INDIRECT, N_NAMED_ARGS) \
+#define INIT_CUMULATIVE_ARGS(CUM, FNTYPE, LIBNAME, INDIRECT) \
 do {									\
   (CUM).words = 0;							\
   (CUM).int_regs = 0;							\
@@ -1333,11 +1353,15 @@ do {									\
 /* If defined, a C expression that gives the alignment boundary, in bits, of an
    argument with the specified mode and type.  */
 
-/* Return the alignment boundary in bits for an argument with a specified
-   mode and type.  */
+/* Arguments with alignment larger than 8 bytes start at the next even
+   boundary.  See ia64_function_arg.  */
 
 #define FUNCTION_ARG_BOUNDARY(MODE, TYPE) \
-  ia64_function_arg_boundary (MODE, TYPE)
+  (((TYPE) ? (TYPE_ALIGN (TYPE) > 8 * BITS_PER_UNIT)		\
+    : (((((MODE) == BLKmode					\
+	  ? int_size_in_bytes (TYPE) : GET_MODE_SIZE (MODE))	\
+	 + UNITS_PER_WORD - 1) / UNITS_PER_WORD) > 1))		\
+    ? 128 : PARM_BOUNDARY)
 
 /* A C expression that is nonzero if REGNO is the number of a hard register in
    which function arguments are sometimes passed.  This does *not* include
@@ -1345,9 +1369,13 @@ do {									\
    On many machines, no registers can be used for this purpose since all
    function arguments are pushed on the stack.  */
 #define FUNCTION_ARG_REGNO_P(REGNO) \
-(((REGNO) >= AR_ARG_FIRST && (REGNO) < (AR_ARG_FIRST + MAX_ARGUMENT_SLOTS)) \
+(((REGNO) >= GR_ARG_FIRST && (REGNO) < (GR_ARG_FIRST + MAX_ARGUMENT_SLOTS)) \
  || ((REGNO) >= FR_ARG_FIRST && (REGNO) < (FR_ARG_FIRST + MAX_ARGUMENT_SLOTS)))
 
+/* Implement `va_arg'.  */
+#define EXPAND_BUILTIN_VA_ARG(valist, type) \
+  ia64_va_arg (valist, type)
+
 /* How Scalar Function Values are Returned */
 
 /* A C expression to create an RTX representing the place where a function
@@ -1363,7 +1391,7 @@ do {									\
   gen_rtx_REG (MODE,							\
 	       (((GET_MODE_CLASS (MODE) == MODE_FLOAT			\
 		 || GET_MODE_CLASS (MODE) == MODE_COMPLEX_FLOAT) &&	\
-		      (MODE) != TFmode)	\
+		      ((MODE) != TFmode || INTEL_EXTENDED_IEEE_FORMAT))	\
 		? FR_RET_FIRST : GR_RET_FIRST))
 
 /* A C expression that is nonzero if REGNO is the number of a hard register in
@@ -1376,8 +1404,22 @@ do {									\
 
 /* How Large Values are Returned */
 
+/* A nonzero value says to return the function value in memory, just as large
+   structures are always returned.  */
+
+#define RETURN_IN_MEMORY(TYPE) \
+  ia64_return_in_memory (TYPE)
+
+/* If you define this macro to be 0, then the conventions used for structure
+   and union return values are decided by the `RETURN_IN_MEMORY' macro.  */
+
 #define DEFAULT_PCC_STRUCT_RETURN 0
 
+/* If the structure value address is passed in a register, then
+   `STRUCT_VALUE_REGNUM' should be the number of that register.  */
+
+#define STRUCT_VALUE_REGNUM GR_REG (8)
+
 
 /* Caller-Saves Register Allocation */
 
@@ -1420,7 +1462,6 @@ do {									\
         fputs ("\tdata8.ua @iplt(", FILE);				\
       else								\
         fputs ("\tdata16.ua @iplt(", FILE);				\
-      mark_decl_referenced (DECL);					\
       assemble_name (FILE, XSTR (XEXP (DECL_RTL (DECL), 0), 0));	\
       fputs (")\n", FILE);						\
       if (TARGET_ILP32)							\
@@ -1452,6 +1493,21 @@ do {									\
   fputs ("\tmov out2 = b0\n", FILE);					\
   fputs ("\tbr.call.sptk.many b0 = _mcount;;\n", FILE);			\
 } while (0)
+
+/* Implementing the Varargs Macros.  */
+
+/* Define this macro to store the anonymous register arguments into the stack
+   so that all the arguments appear to have been passed consecutively on the
+   stack.  */
+
+#define SETUP_INCOMING_VARARGS(ARGS_SO_FAR, MODE, TYPE, PRETEND_ARGS_SIZE, SECOND_TIME) \
+    ia64_setup_incoming_varargs (ARGS_SO_FAR, MODE, TYPE, & PRETEND_ARGS_SIZE, SECOND_TIME)
+
+/* Define this macro if the location where a function argument is passed
+   depends on whether or not it is a named argument.  */
+
+#define STRICT_ARGUMENT_NAMING  1
+
 
 /* Trampolines for Nested Functions.  */
 
@@ -1498,6 +1554,15 @@ do {									\
 
 #define INITIALIZE_TRAMPOLINE(ADDR, FNADDR, STATIC_CHAIN) \
   ia64_initialize_trampoline((ADDR), (FNADDR), (STATIC_CHAIN))
+
+/* Implicit Calls to Library Routines */
+
+/* Define this macro if GNU CC should generate calls to the System V (and ANSI
+   C) library functions `memcpy' and `memset' rather than the BSD functions
+   `bcopy' and `bzero'.  */
+
+#define TARGET_MEM_FUNCTIONS
+
 
 /* Addressing Modes */
 
@@ -1564,6 +1629,13 @@ do {									\
 
 #define REG_OK_FOR_INDEX_P(X) REG_OK_FOR_BASE_P (X)
 
+/* A C compound statement that attempts to replace X with a valid memory
+   address for an operand of mode MODE.
+
+   This must be present, but there is nothing useful to be done here.  */
+
+#define LEGITIMIZE_ADDRESS(X, OLDX, MODE, WIN)
+
 /* A C statement or compound statement with a conditional `goto LABEL;'
    executed if memory address X (an RTX) can have different meanings depending
    on the machine mode of the memory reference it is used for or if the address
@@ -1576,7 +1648,10 @@ do {									\
 /* A C expression that is nonzero if X is a legitimate constant for an
    immediate operand on the target machine.  */
 
-#define LEGITIMATE_CONSTANT_P(X) ia64_legitimate_constant_p (X)
+#define LEGITIMATE_CONSTANT_P(X) \
+  (GET_CODE (X) != CONST_DOUBLE || GET_MODE (X) == VOIDmode	\
+   || GET_MODE (X) == DImode || CONST_DOUBLE_OK_FOR_G (X))	\
+
 
 /* Condition Code Status */
 
@@ -1672,12 +1747,17 @@ do {									\
 /* A C string constant for text to be output before each `asm' statement or
    group of consecutive ones.  */
 
-#define ASM_APP_ON (TARGET_GNU_AS ? "#APP\n" : "//APP\n")
+/* ??? This won't work with the Intel assembler, because it does not accept
+   # as a comment start character.  However, //APP does not work in gas, so we
+   can't use that either.  Same problem for ASM_APP_OFF below.  */
+
+#define ASM_APP_ON "#APP\n"
 
 /* A C string constant for text to be output after each `asm' statement or
    group of consecutive ones.  */
 
-#define ASM_APP_OFF (TARGET_GNU_AS ? "#NO_APP\n" : "//NO_APP\n")
+#define ASM_APP_OFF "#NO_APP\n"
+
 
 /* Output of Uninitialized Variables.  */
 
@@ -2042,7 +2122,7 @@ do {									\
 
 /* Macros for SDB and Dwarf Output.  */
 
-/* Define this macro if GCC should produce dwarf version 2 format debugging
+/* Define this macro if GNU CC should produce dwarf version 2 format debugging
    output in response to the `-g' option.  */
 
 #define DWARF2_DEBUGGING_INFO 1
@@ -2055,20 +2135,18 @@ do {									\
    add brackets around the label.  */
 
 #define ASM_OUTPUT_DEBUG_LABEL(FILE, PREFIX, NUM) \
-  fprintf (FILE, TARGET_GNU_AS ? "[.%s%d:]\n" : ".%s%d:\n", PREFIX, NUM)
+  fprintf (FILE, "[.%s%d:]\n", PREFIX, NUM)
 
 /* Use section-relative relocations for debugging offsets.  Unlike other
    targets that fake this by putting the section VMA at 0, IA-64 has
    proper relocations for them.  */
-/* APPLE LOCAL begin dwarf 4383509 */
-#define ASM_OUTPUT_DWARF_OFFSET(FILE, SIZE, LABEL, SECTION)	\
-  do {								\
-    fputs (integer_asm_op (SIZE, FALSE), FILE);			\
-    fputs ("@secrel(", FILE);					\
-    assemble_name (FILE, LABEL);				\
-    fputc (')', FILE);						\
+#define ASM_OUTPUT_DWARF_OFFSET(FILE, SIZE, LABEL)	\
+  do {							\
+    fputs (integer_asm_op (SIZE, FALSE), FILE);		\
+    fputs ("@secrel(", FILE);				\
+    assemble_name (FILE, LABEL);			\
+    fputc (')', FILE);					\
   } while (0)
-/* APPLE LOCAL end dwarf 4383509 */
 
 /* Emit a PC-relative relocation.  */
 #define ASM_OUTPUT_DWARF_PCREL(FILE, SIZE, LABEL)	\
@@ -2096,6 +2174,58 @@ do {									\
 #define SYMBOL_REF_SMALL_ADDR_P(X)	\
 	((SYMBOL_REF_FLAGS (X) & SYMBOL_FLAG_SMALL_ADDR) != 0)
 
+/* Define this if you have defined special-purpose predicates in the file
+   `MACHINE.c'.  For each predicate, list all rtl codes that can be in
+   expressions matched by the predicate.  */
+
+#define PREDICATE_CODES \
+{ "call_operand", {SUBREG, REG, SYMBOL_REF}},				\
+{ "got_symbolic_operand", {SYMBOL_REF, CONST, LABEL_REF}},		\
+{ "sdata_symbolic_operand", {SYMBOL_REF, CONST}},			\
+{ "small_addr_symbolic_operand", {SYMBOL_REF}},				\
+{ "symbolic_operand", {SYMBOL_REF, CONST, LABEL_REF}},			\
+{ "function_operand", {SYMBOL_REF}},					\
+{ "setjmp_operand", {SYMBOL_REF}},					\
+{ "destination_operand", {SUBREG, REG, MEM}},				\
+{ "not_postinc_memory_operand", {MEM}},					\
+{ "move_operand", {SUBREG, REG, MEM, CONST_INT, CONST_DOUBLE,		\
+		     CONSTANT_P_RTX, SYMBOL_REF, CONST, LABEL_REF}},	\
+{ "gr_register_operand", {SUBREG, REG}},				\
+{ "fr_register_operand", {SUBREG, REG}},				\
+{ "grfr_register_operand", {SUBREG, REG}},				\
+{ "gr_nonimmediate_operand", {SUBREG, REG, MEM}},			\
+{ "fr_nonimmediate_operand", {SUBREG, REG, MEM}},			\
+{ "grfr_nonimmediate_operand", {SUBREG, REG, MEM}},			\
+{ "gr_reg_or_0_operand", {SUBREG, REG, CONST_INT}},			\
+{ "gr_reg_or_5bit_operand", {SUBREG, REG, CONST_INT, CONSTANT_P_RTX}},	\
+{ "gr_reg_or_6bit_operand", {SUBREG, REG, CONST_INT, CONSTANT_P_RTX}},	\
+{ "gr_reg_or_8bit_operand", {SUBREG, REG, CONST_INT, CONSTANT_P_RTX}},	\
+{ "grfr_reg_or_8bit_operand", {SUBREG, REG, CONST_INT, CONSTANT_P_RTX}}, \
+{ "gr_reg_or_8bit_adjusted_operand", {SUBREG, REG, CONST_INT,		\
+				     CONSTANT_P_RTX}},			\
+{ "gr_reg_or_8bit_and_adjusted_operand", {SUBREG, REG, CONST_INT,	\
+					 CONSTANT_P_RTX}},		\
+{ "gr_reg_or_14bit_operand", {SUBREG, REG, CONST_INT, CONSTANT_P_RTX}}, \
+{ "gr_reg_or_22bit_operand", {SUBREG, REG, CONST_INT, CONSTANT_P_RTX}}, \
+{ "shift_count_operand", {SUBREG, REG, CONST_INT, CONSTANT_P_RTX}},	\
+{ "shift_32bit_count_operand", {SUBREG, REG, CONST_INT,			\
+				  CONSTANT_P_RTX}},			\
+{ "shladd_operand", {CONST_INT}},					\
+{ "fetchadd_operand", {CONST_INT}},					\
+{ "fr_reg_or_fp01_operand", {SUBREG, REG, CONST_DOUBLE}},		\
+{ "normal_comparison_operator", {EQ, NE, GT, LE, GTU, LEU}},		\
+{ "adjusted_comparison_operator", {LT, GE, LTU, GEU}},			\
+{ "signed_inequality_operator", {GE, GT, LE, LT}},			\
+{ "predicate_operator", {NE, EQ}},					\
+{ "condop_operator", {PLUS, MINUS, IOR, XOR, AND}},			\
+{ "ar_lc_reg_operand", {REG}},						\
+{ "ar_ccv_reg_operand", {REG}},						\
+{ "ar_pfs_reg_operand", {REG}},						\
+{ "general_tfmode_operand", {SUBREG, REG, CONST_DOUBLE, MEM}},		\
+{ "destination_tfmode_operand", {SUBREG, REG, MEM}},			\
+{ "tfreg_or_fp01_operand", {REG, CONST_DOUBLE}},			\
+{ "basereg_operand", {SUBREG, REG}},
+
 /* An alias for a machine mode name.  This is the machine mode that elements of
    a jump-table should have.  */
 
@@ -2174,7 +2304,8 @@ do {									\
 
 extern int ia64_final_schedule;
 
-#define TARGET_UNWIND_INFO	1
+#define IA64_UNWIND_INFO	1
+#define IA64_UNWIND_EMIT(f,i)	process_for_unwind_directive (f,i)
 
 #define EH_RETURN_DATA_REGNO(N) ((N) < 4 ? (N) + 15 : INVALID_REGNUM)
 
@@ -2192,9 +2323,6 @@ struct machine_function GTY(())
 
   /* The number of varargs registers to save.  */
   int n_varargs;
-
-  /* The number of the next unwind state to copy.  */
-  int state_num;
 };
 
 
@@ -2264,10 +2392,6 @@ enum fetchop_code {
 #undef  PROFILE_BEFORE_PROLOGUE
 #define PROFILE_BEFORE_PROLOGUE 1
 
-/* Initialize library function table. */
-#undef TARGET_INIT_LIBFUNCS
-#define TARGET_INIT_LIBFUNCS ia64_init_libfuncs
-
 
 
 /* Switch on code for querying unit reservations.  */
