@@ -1,5 +1,5 @@
 /* Process source files and output type information.
-   Copyright (C) 2002, 2003, 2004, 2005 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2003 Free Software Foundation, Inc.
 
 This file is part of GCC.
 
@@ -24,7 +24,10 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 #include "tm.h"
 #include "gengtype.h"
 #include "gtyp-gen.h"
-#include "errors.h"
+
+#define NO_GENRTL_H
+#include "rtl.h"
+#undef abort
 
 /* Nonzero iff an error has occurred.  */
 static int hit_error = 0;
@@ -114,7 +117,7 @@ do_typedef (const char *s, type_p t, struct fileloc *pos)
 	return;
       }
 
-  p = XNEW (struct pair);
+  p = xmalloc (sizeof (struct pair));
   p->next = typedefs;
   p->name = s;
   p->type = t;
@@ -170,7 +173,7 @@ new_structure (const char *name, int isunion, struct fileloc *pos,
 	else if (si->u.s.line.file != NULL && si->u.s.bitmap != bitmap)
 	  {
 	    ls = si;
-	    si = XCNEW (struct type);
+	    si = xcalloc (1, sizeof (struct type));
 	    memcpy (si, ls, sizeof (struct type));
 	    ls->kind = TYPE_LANG_STRUCT;
 	    ls->u.s.lang_struct = si;
@@ -184,7 +187,7 @@ new_structure (const char *name, int isunion, struct fileloc *pos,
 
 	if (ls != NULL && s == NULL)
 	  {
-	    s = XCNEW (struct type);
+	    s = xcalloc (1, sizeof (struct type));
 	    s->next = ls->u.s.lang_struct;
 	    ls->u.s.lang_struct = s;
 	    s->u.s.lang_struct = ls;
@@ -194,7 +197,7 @@ new_structure (const char *name, int isunion, struct fileloc *pos,
 
   if (s == NULL)
     {
-      s = XCNEW (struct type);
+      s = xcalloc (1, sizeof (struct type));
       s->next = structures;
       structures = s;
     }
@@ -230,7 +233,7 @@ find_structure (const char *name, int isunion)
 	&& UNION_P (s) == isunion)
       return s;
 
-  s = XCNEW (struct type);
+  s = xcalloc (1, sizeof (struct type));
   s->next = structures;
   structures = s;
   s->kind = isunion ? TYPE_UNION : TYPE_STRUCT;
@@ -255,7 +258,7 @@ find_param_structure (type_p t, type_p param[NUM_PARAM])
       break;
   if (res == NULL)
     {
-      res = XCNEW (struct type);
+      res = xcalloc (1, sizeof (*res));
       res->kind = TYPE_PARAM_STRUCT;
       res->next = param_structs;
       param_structs = res;
@@ -270,9 +273,9 @@ find_param_structure (type_p t, type_p param[NUM_PARAM])
 type_p
 create_scalar_type (const char *name, size_t name_len)
 {
-  type_p r = XCNEW (struct type);
+  type_p r = xcalloc (1, sizeof (struct type));
   r->kind = TYPE_SCALAR;
-  r->u.sc = (char *) xmemdup (name, name_len, name_len + 1);
+  r->u.sc = xmemdup (name, name_len, name_len + 1);
   return r;
 }
 
@@ -283,7 +286,7 @@ create_pointer (type_p t)
 {
   if (! t->pointer_to)
     {
-      type_p r = XCNEW (struct type);
+      type_p r = xcalloc (1, sizeof (struct type));
       r->kind = TYPE_POINTER;
       r->u.p = t;
       t->pointer_to = r;
@@ -298,23 +301,13 @@ create_array (type_p t, const char *len)
 {
   type_p v;
 
-  v = XCNEW (struct type);
+  v = xcalloc (1, sizeof (*v));
   v->kind = TYPE_ARRAY;
   v->u.a.p = t;
   v->u.a.len = len;
   return v;
 }
 
-/* Return an options structure with name NAME and info INFO.  */
-options_p
-create_option (const char *name, void *info)
-{
-  options_p o = XNEW (struct options);
-  o->name = name;
-  o->info = (const char*) info;
-  return o;
-}
-
 /* Add a variable named S of type T with options O defined at POS,
    to `variables'.  */
 
@@ -322,7 +315,7 @@ void
 note_variable (const char *s, type_p t, options_p o, struct fileloc *pos)
 {
   pair_p n;
-  n = XNEW (struct pair);
+  n = xmalloc (sizeof (*n));
   n->name = s;
   n->type = t;
   n->line = *pos;
@@ -331,25 +324,9 @@ note_variable (const char *s, type_p t, options_p o, struct fileloc *pos)
   variables = n;
 }
 
-/* We don't care how long a CONST_DOUBLE is.  */
+/* We really don't care how long a CONST_DOUBLE is.  */
 #define CONST_DOUBLE_FORMAT "ww"
-/* We don't want to see codes that are only for generator files.  */
-#undef GENERATOR_FILE
-
-enum rtx_code {
-#define DEF_RTL_EXPR(ENUM, NAME, FORMAT, CLASS) ENUM ,
-#include "rtl.def"
-#undef DEF_RTL_EXPR
-  NUM_RTX_CODE
-};
-
-static const char * const rtx_name[NUM_RTX_CODE] = {
-#define DEF_RTL_EXPR(ENUM, NAME, FORMAT, CLASS)   NAME ,
-#include "rtl.def"
-#undef DEF_RTL_EXPR
-};
-
-static const char * const rtx_format[NUM_RTX_CODE] = {
+const char * const rtx_format[NUM_RTX_CODE] = {
 #define DEF_RTL_EXPR(ENUM, NAME, FORMAT, CLASS)   FORMAT ,
 #include "rtl.def"
 #undef DEF_RTL_EXPR
@@ -357,27 +334,6 @@ static const char * const rtx_format[NUM_RTX_CODE] = {
 
 static int rtx_next_new[NUM_RTX_CODE];
 
-/* We also need codes and names for insn notes (not register notes).
-   Note that we do *not* bias the note values here.  */
-enum insn_note {
-#define DEF_INSN_NOTE(NAME) NAME,
-#include "insn-notes.def"
-#undef DEF_INSN_NOTE
-
-  NOTE_INSN_MAX
-};
-
-/* We must allocate one more entry here, as we use NOTE_INSN_MAX as the
-   default field for line number notes.  */
-static const char *const note_insn_name[NOTE_INSN_MAX+1] = {
-#define DEF_INSN_NOTE(NAME) #NAME,
-#include "insn-notes.def"
-#undef DEF_INSN_NOTE
-};
-
-#undef CONST_DOUBLE_FORMAT
-#define GENERATOR_FILE
-
 /* Generate the contents of the rtx_next array.  This really doesn't belong
    in gengtype at all, but it's needed for adjust_field_rtx_def.  */
 
@@ -415,7 +371,7 @@ write_rtx_next (void)
       oprintf (f, "  0,\n");
     else
       oprintf (f,
-	       "  RTX_HDR_SIZE + %d * sizeof (rtunion),\n",
+	       "  offsetof (struct rtx_def, fld) + %d * sizeof (rtunion),\n",
 	       rtx_next_new[i]);
   oprintf (f, "};\n");
 }
@@ -425,7 +381,7 @@ write_rtx_next (void)
    are based in a complex way on the type of RTL.  */
 
 static type_p
-adjust_field_rtx_def (type_p t, options_p ARG_UNUSED (opt))
+adjust_field_rtx_def (type_p t, options_p opt ATTRIBUTE_UNUSED)
 {
   pair_p flds = NULL;
   options_p nodot;
@@ -433,14 +389,20 @@ adjust_field_rtx_def (type_p t, options_p ARG_UNUSED (opt))
   type_p rtx_tp, rtvec_tp, tree_tp, mem_attrs_tp, note_union_tp, scalar_tp;
   type_p bitmap_tp, basic_block_tp, reg_attrs_tp;
 
-  if (t->kind != TYPE_UNION)
+  static const char * const rtx_name[NUM_RTX_CODE] = {
+#define DEF_RTL_EXPR(ENUM, NAME, FORMAT, CLASS)   NAME ,
+#include "rtl.def"
+#undef DEF_RTL_EXPR
+  };
+
+  if (t->kind != TYPE_ARRAY)
     {
       error_at_line (&lexer_line,
-		     "special `rtx_def' must be applied to a union");
+		     "special `rtx_def' must be applied to an array");
       return &string_type;
     }
 
-  nodot = XNEW (struct options);
+  nodot = xmalloc (sizeof (*nodot));
   nodot->next = NULL;
   nodot->name = "dot";
   nodot->info = "";
@@ -458,17 +420,17 @@ adjust_field_rtx_def (type_p t, options_p ARG_UNUSED (opt))
     pair_p note_flds = NULL;
     int c;
 
-    for (c = 0; c <= NOTE_INSN_MAX; c++)
+    for (c = NOTE_INSN_BIAS; c <= NOTE_INSN_MAX; c++)
       {
 	pair_p old_note_flds = note_flds;
 
-	note_flds = XNEW (struct pair);
+	note_flds = xmalloc (sizeof (*note_flds));
 	note_flds->line.file = __FILE__;
 	note_flds->line.line = __LINE__;
-	note_flds->opt = XNEW (struct options);
+	note_flds->opt = xmalloc (sizeof (*note_flds->opt));
 	note_flds->opt->next = nodot;
 	note_flds->opt->name = "tag";
-	note_flds->opt->info = note_insn_name[c];
+	note_flds->opt->info = xasprintf ("%d", c);
 	note_flds->next = old_note_flds;
 
 	switch (c)
@@ -477,24 +439,23 @@ adjust_field_rtx_def (type_p t, options_p ARG_UNUSED (opt))
 	       number notes.  */
 	  case NOTE_INSN_MAX:
 	    note_flds->opt->name = "default";
-	    note_flds->name = "rt_str";
+	    note_flds->name = "rtstr";
 	    note_flds->type = &string_type;
 	    break;
 
 	  case NOTE_INSN_BLOCK_BEG:
 	  case NOTE_INSN_BLOCK_END:
-	    note_flds->name = "rt_tree";
+	    note_flds->name = "rttree";
 	    note_flds->type = tree_tp;
 	    break;
 
 	  case NOTE_INSN_EXPECTED_VALUE:
-	  case NOTE_INSN_VAR_LOCATION:
-	    note_flds->name = "rt_rtx";
+	    note_flds->name = "rtx";
 	    note_flds->type = rtx_tp;
 	    break;
 
 	  default:
-	    note_flds->name = "rt_int";
+	    note_flds->name = "rtint";
 	    note_flds->type = scalar_tp;
 	    break;
 	  }
@@ -525,48 +486,48 @@ adjust_field_rtx_def (type_p t, options_p ARG_UNUSED (opt))
 	    case 'n':
 	    case 'w':
 	      t = scalar_tp;
-	      subname = "rt_int";
+	      subname = "rtint";
 	      break;
 
 	    case '0':
 	      if (i == MEM && aindex == 1)
-		t = mem_attrs_tp, subname = "rt_mem";
+		t = mem_attrs_tp, subname = "rtmem";
 	      else if (i == JUMP_INSN && aindex == 9)
-		t = rtx_tp, subname = "rt_rtx";
+		t = rtx_tp, subname = "rtx";
 	      else if (i == CODE_LABEL && aindex == 4)
-		t = scalar_tp, subname = "rt_int";
+		t = scalar_tp, subname = "rtint";
 	      else if (i == CODE_LABEL && aindex == 5)
-		t = rtx_tp, subname = "rt_rtx";
+		t = rtx_tp, subname = "rtx";
 	      else if (i == LABEL_REF
 		       && (aindex == 1 || aindex == 2))
-		t = rtx_tp, subname = "rt_rtx";
+		t = rtx_tp, subname = "rtx";
 	      else if (i == NOTE && aindex == 4)
 		t = note_union_tp, subname = "";
 	      else if (i == NOTE && aindex >= 7)
-		t = scalar_tp, subname = "rt_int";
+		t = scalar_tp, subname = "rtint";
 	      else if (i == ADDR_DIFF_VEC && aindex == 4)
-		t = scalar_tp, subname = "rt_int";
+		t = scalar_tp, subname = "rtint";
 	      else if (i == VALUE && aindex == 0)
-		t = scalar_tp, subname = "rt_int";
+		t = scalar_tp, subname = "rtint";
 	      else if (i == REG && aindex == 1)
-		t = scalar_tp, subname = "rt_int";
+		t = scalar_tp, subname = "rtint";
 	      else if (i == REG && aindex == 2)
-		t = reg_attrs_tp, subname = "rt_reg";
+		t = reg_attrs_tp, subname = "rtreg";
 	      else if (i == SCRATCH && aindex == 0)
-		t = scalar_tp, subname = "rt_int";
+		t = scalar_tp, subname = "rtint";
 	      else if (i == SYMBOL_REF && aindex == 1)
-		t = scalar_tp, subname = "rt_int";
+		t = scalar_tp, subname = "rtint";
 	      else if (i == SYMBOL_REF && aindex == 2)
-		t = tree_tp, subname = "rt_tree";
+		t = tree_tp, subname = "rttree";
 	      else if (i == BARRIER && aindex >= 3)
-		t = scalar_tp, subname = "rt_int";
+		t = scalar_tp, subname = "rtint";
 	      else
 		{
 		  error_at_line (&lexer_line,
 			"rtx type `%s' has `0' in position %lu, can't handle",
 				 rtx_name[i], (unsigned long) aindex);
 		  t = &string_type;
-		  subname = "rt_int";
+		  subname = "rtint";
 		}
 	      break;
 
@@ -574,34 +535,34 @@ adjust_field_rtx_def (type_p t, options_p ARG_UNUSED (opt))
 	    case 'S':
 	    case 'T':
 	      t = &string_type;
-	      subname = "rt_str";
+	      subname = "rtstr";
 	      break;
 
 	    case 'e':
 	    case 'u':
 	      t = rtx_tp;
-	      subname = "rt_rtx";
+	      subname = "rtx";
 	      break;
 
 	    case 'E':
 	    case 'V':
 	      t = rtvec_tp;
-	      subname = "rt_rtvec";
+	      subname = "rtvec";
 	      break;
 
 	    case 't':
 	      t = tree_tp;
-	      subname = "rt_tree";
+	      subname = "rttree";
 	      break;
 
 	    case 'b':
 	      t = bitmap_tp;
-	      subname = "rt_bit";
+	      subname = "rtbit";
 	      break;
 
 	    case 'B':
 	      t = basic_block_tp;
-	      subname = "rt_bb";
+	      subname = "bb";
 	      break;
 
 	    default:
@@ -610,20 +571,20 @@ adjust_field_rtx_def (type_p t, options_p ARG_UNUSED (opt))
 			     rtx_name[i], rtx_format[i][aindex],
 			     (unsigned long)aindex);
 	      t = &string_type;
-	      subname = "rt_int";
+	      subname = "rtint";
 	      break;
 	    }
 
-	  subfields = XNEW (struct pair);
+	  subfields = xmalloc (sizeof (*subfields));
 	  subfields->next = old_subf;
 	  subfields->type = t;
-	  subfields->name = xasprintf (".fld[%lu].%s", (unsigned long)aindex,
+	  subfields->name = xasprintf ("[%lu].%s", (unsigned long)aindex,
 				       subname);
 	  subfields->line.file = __FILE__;
 	  subfields->line.line = __LINE__;
 	  if (t == note_union_tp)
 	    {
-	      subfields->opt = XNEW (struct options);
+	      subfields->opt = xmalloc (sizeof (*subfields->opt));
 	      subfields->opt->next = nodot;
 	      subfields->opt->name = "desc";
 	      subfields->opt->info = "NOTE_LINE_NUMBER (&%0)";
@@ -631,7 +592,7 @@ adjust_field_rtx_def (type_p t, options_p ARG_UNUSED (opt))
 	  else if (t == basic_block_tp)
 	    {
 	      /* We don't presently GC basic block structures...  */
-	      subfields->opt = XNEW (struct options);
+	      subfields->opt = xmalloc (sizeof (*subfields->opt));
 	      subfields->opt->next = nodot;
 	      subfields->opt->name = "skip";
 	      subfields->opt->info = NULL;
@@ -640,7 +601,7 @@ adjust_field_rtx_def (type_p t, options_p ARG_UNUSED (opt))
 	    subfields->opt = nodot;
 	}
 
-      flds = XNEW (struct pair);
+      flds = xmalloc (sizeof (*flds));
       flds->next = old_flds;
       flds->name = "";
       sname = xasprintf ("rtx_def_%s", rtx_name[i]);
@@ -648,7 +609,7 @@ adjust_field_rtx_def (type_p t, options_p ARG_UNUSED (opt))
       flds->type = find_structure (sname, 0);
       flds->line.file = __FILE__;
       flds->line.line = __LINE__;
-      flds->opt = XNEW (struct options);
+      flds->opt = xmalloc (sizeof (*flds->opt));
       flds->opt->next = nodot;
       flds->opt->name = "tag";
       ftag = xstrdup (rtx_name[i]);
@@ -672,6 +633,17 @@ adjust_field_tree_exp (type_p t, options_p opt ATTRIBUTE_UNUSED)
 {
   pair_p flds;
   options_p nodot;
+  size_t i;
+  static const struct {
+    const char *name;
+    int first_rtl;
+    int num_rtl;
+  } data[] = {
+    { "SAVE_EXPR", 2, 1 },
+    { "GOTO_SUBROUTINE_EXPR", 0, 2 },
+    { "RTL_EXPR", 0, 2 },
+    { "WITH_CLEANUP_EXPR", 2, 1 },
+  };
 
   if (t->kind != TYPE_ARRAY)
     {
@@ -680,29 +652,67 @@ adjust_field_tree_exp (type_p t, options_p opt ATTRIBUTE_UNUSED)
       return &string_type;
     }
 
-  nodot = XNEW (struct options);
+  nodot = xmalloc (sizeof (*nodot));
   nodot->next = NULL;
   nodot->name = "dot";
   nodot->info = "";
 
-  flds = XNEW (struct pair);
+  flds = xmalloc (sizeof (*flds));
   flds->next = NULL;
   flds->name = "";
   flds->type = t;
   flds->line.file = __FILE__;
   flds->line.line = __LINE__;
-  flds->opt = XNEW (struct options);
+  flds->opt = xmalloc (sizeof (*flds->opt));
   flds->opt->next = nodot;
   flds->opt->name = "length";
   flds->opt->info = "TREE_CODE_LENGTH (TREE_CODE ((tree) &%0))";
   {
     options_p oldopt = flds->opt;
-    flds->opt = XNEW (struct options);
+    flds->opt = xmalloc (sizeof (*flds->opt));
     flds->opt->next = oldopt;
     flds->opt->name = "default";
     flds->opt->info = "";
   }
 
+  for (i = 0; i < ARRAY_SIZE (data); i++)
+    {
+      pair_p old_flds = flds;
+      pair_p subfields = NULL;
+      int r_index;
+      const char *sname;
+
+      for (r_index = 0;
+	   r_index < data[i].first_rtl + data[i].num_rtl;
+	   r_index++)
+	{
+	  pair_p old_subf = subfields;
+	  subfields = xmalloc (sizeof (*subfields));
+	  subfields->next = old_subf;
+	  subfields->name = xasprintf ("[%d]", r_index);
+	  if (r_index < data[i].first_rtl)
+	    subfields->type = t->u.a.p;
+	  else
+	    subfields->type = create_pointer (find_structure ("rtx_def", 0));
+	  subfields->line.file = __FILE__;
+	  subfields->line.line = __LINE__;
+	  subfields->opt = nodot;
+	}
+
+      flds = xmalloc (sizeof (*flds));
+      flds->next = old_flds;
+      flds->name = "";
+      sname = xasprintf ("tree_exp_%s", data[i].name);
+      new_structure (sname, 0, &lexer_line, subfields, NULL);
+      flds->type = find_structure (sname, 0);
+      flds->line.file = __FILE__;
+      flds->line.line = __LINE__;
+      flds->opt = xmalloc (sizeof (*flds->opt));
+      flds->opt->next = nodot;
+      flds->opt->name = "tag";
+      flds->opt->info = data[i].name;
+    }
+
   new_structure ("tree_exp_subunion", 1, &lexer_line, flds, nodot);
   return find_structure ("tree_exp_subunion", 1);
 }
@@ -758,7 +768,7 @@ adjust_field_type (type_p t, options_p opt)
       }
     else if (strcmp (opt->name, "special") == 0)
       {
-	const char *special_name = opt->info;
+	const char *special_name = (const char *)opt->info;
 	if (strcmp (special_name, "tree_exp") == 0)
 	  t = adjust_field_tree_exp (t, opt);
 	else if (strcmp (special_name, "rtx_def") == 0)
@@ -854,7 +864,7 @@ note_yacc_type (options_p o, pair_p fields, pair_p typeinfo,
 }
 
 static void process_gc_options (options_p, enum gc_used_enum,
-				int *, int *, int *, type_p *);
+				int *, int *, int *);
 static void set_gc_used_type (type_p, enum gc_used_enum, type_p *);
 static void set_gc_used (pair_p);
 
@@ -862,7 +872,7 @@ static void set_gc_used (pair_p);
 
 static void
 process_gc_options (options_p opt, enum gc_used_enum level, int *maybe_undef,
-		    int *pass_param, int *length, type_p *nested_ptr)
+		    int *pass_param, int *length)
 {
   options_p o;
   for (o = opt; o; o = o->next)
@@ -874,8 +884,6 @@ process_gc_options (options_p opt, enum gc_used_enum level, int *maybe_undef,
       *pass_param = 1;
     else if (strcmp (o->name, "length") == 0)
       *length = 1;
-    else if (strcmp (o->name, "nested_ptr") == 0)
-      *nested_ptr = ((const struct nested_ptr_data *) o->info)->type;
 }
 
 /* Set the gc_used field of T to LEVEL, and handle the types it references.  */
@@ -895,24 +903,18 @@ set_gc_used_type (type_p t, enum gc_used_enum level, type_p param[NUM_PARAM])
       {
 	pair_p f;
 	int dummy;
-	type_p dummy2;
 
-	process_gc_options (t->u.s.opt, level, &dummy, &dummy, &dummy,
-			    &dummy2);
+	process_gc_options (t->u.s.opt, level, &dummy, &dummy, &dummy);
 
 	for (f = t->u.s.fields; f; f = f->next)
 	  {
 	    int maybe_undef = 0;
 	    int pass_param = 0;
 	    int length = 0;
-	    type_p nested_ptr = NULL;
 	    process_gc_options (f->opt, level, &maybe_undef, &pass_param,
-				&length, &nested_ptr);
+				&length);
 
-	    if (nested_ptr && f->type->kind == TYPE_POINTER)
-	      set_gc_used_type (nested_ptr, GC_POINTED_TO, 
-				pass_param ? param : NULL);
-	    else if (length && f->type->kind == TYPE_POINTER)
+	    if (length && f->type->kind == TYPE_POINTER)
 	      set_gc_used_type (f->type->u.p, GC_USED, NULL);
 	    else if (maybe_undef && f->type->kind == TYPE_POINTER)
 	      set_gc_used_type (f->type->u.p, GC_MAYBE_POINTED_TO, NULL);
@@ -978,7 +980,7 @@ static outf_p output_files;
 
 /* The output header file that is included into pretty much every
    source file.  */
-static outf_p header_file;
+outf_p header_file;
 
 /* Number of files specified in gtfiles.  */
 #define NUM_GT_FILES (ARRAY_SIZE (all_files) - 1)
@@ -1002,7 +1004,7 @@ static outf_p
 create_file (const char *name, const char *oname)
 {
   static const char *const hdr[] = {
-    "   Copyright (C) 2004 Free Software Foundation, Inc.\n",
+    "   Copyright (C) 2003 Free Software Foundation, Inc.\n",
     "\n",
     "This file is part of GCC.\n",
     "\n",
@@ -1026,7 +1028,7 @@ create_file (const char *name, const char *oname)
   outf_p f;
   size_t i;
 
-  f = XCNEW (struct outf);
+  f = xcalloc (sizeof (*f), 1);
   f->next = output_files;
   f->name = oname;
   output_files = f;
@@ -1056,7 +1058,7 @@ oprintf (outf_p o, const char *format, ...)
       do {
 	new_len *= 2;
       } while (o->bufused + slength >= new_len);
-      o->buf = XRESIZEVEC (char, o->buf, new_len);
+      o->buf = xrealloc (o->buf, new_len);
       o->buflength = new_len;
     }
   memcpy (o->buf + o->bufused, s, slength);
@@ -1082,12 +1084,11 @@ open_base_files (void)
   {
     /* The order of files here matters very much.  */
     static const char *const ifiles [] = {
-      "config.h", "system.h", "coretypes.h", "tm.h", "varray.h", 
-      "hashtab.h", "splay-tree.h",  "obstack.h", "bitmap.h", "input.h",
-      "tree.h", "rtl.h", "function.h", "insn-config.h", "expr.h",
-      "hard-reg-set.h", "basic-block.h", "cselib.h", "insn-addr.h",
-      "optabs.h", "libfuncs.h", "debug.h", "ggc.h", "cgraph.h",
-      "tree-flow.h", "reload.h", "cpp-id-data.h", "tree-chrec.h",
+      "config.h", "system.h", "coretypes.h", "tm.h", "varray.h",
+      "hashtab.h", "splay-tree.h", "bitmap.h", "tree.h", "rtl.h",
+      "function.h", "insn-config.h", "expr.h", "hard-reg-set.h",
+      "basic-block.h", "cselib.h", "insn-addr.h", "ssa.h", "optabs.h",
+      "libfuncs.h", "debug.h", "ggc.h", "cgraph.h",
       NULL
     };
     const char *const *ifp;
@@ -1124,11 +1125,11 @@ get_file_basename (const char *f)
       s2 = lang_dir_names [i];
       l1 = strlen (s1);
       l2 = strlen (s2);
-      if (l1 >= l2 && IS_DIR_SEPARATOR (s1[-1]) && !memcmp (s1, s2, l2))
+      if (l1 >= l2 && !memcmp (s1, s2, l2))
         {
           basename -= l2 + 1;
           if ((basename - f - 1) != srcdir_len)
-	    fatal ("filename `%s' should be preceded by $srcdir", f);
+            abort (); /* Match is wrong - should be preceded by $srcdir.  */
           break;
         }
     }
@@ -1153,10 +1154,6 @@ get_base_file_bitmap (const char *input_file)
   unsigned k;
   unsigned bitmap;
 
-  /* If the file resides in a language subdirectory (e.g., 'cp'), assume that
-     it belongs to the corresponding language.  The file may belong to other
-     languages as well (which is checked for below).  */
-
   if (slashpos)
     {
       size_t i;
@@ -1166,7 +1163,10 @@ get_base_file_bitmap (const char *input_file)
           {
             /* It's in a language directory, set that language.  */
             bitmap = 1 << i;
+            return bitmap;
           }
+
+      abort (); /* Should have found the language.  */
     }
 
   /* If it's in any config-lang.in, then set for the languages
@@ -1229,30 +1229,11 @@ get_output_file_with_visibility (const char *input_file)
       memcpy (s, ".h", sizeof (".h"));
       for_name = basename;
     }
-  /* Some headers get used by more than one front-end; hence, it
-     would be inappropriate to spew them out to a single gtype-<lang>.h
-     (and gengtype doesn't know how to direct spewage into multiple
-     gtype-<lang>.h headers at this time).  Instead, we pair up these
-     headers with source files (and their special purpose gt-*.h headers).  */
   else if (strcmp (basename, "c-common.h") == 0)
     output_name = "gt-c-common.h", for_name = "c-common.c";
   else if (strcmp (basename, "c-tree.h") == 0)
     output_name = "gt-c-decl.h", for_name = "c-decl.c";
-  /* APPLE LOCAL begin mainline */
-  else if (strncmp (basename, "cp", 2) == 0 && IS_DIR_SEPARATOR (basename[2])
-	   && strcmp (basename + 3, "cp-tree.h") == 0)
-    output_name = "gt-cp-tree.h", for_name = "cp/tree.c";
-  else if (strncmp (basename, "cp", 2) == 0 && IS_DIR_SEPARATOR (basename[2])
-	   && strcmp (basename + 3, "decl.h") == 0)
-    output_name = "gt-cp-decl.h", for_name = "cp/decl.c";
-  else if (strncmp (basename, "cp", 2) == 0 && IS_DIR_SEPARATOR (basename[2])
-	   && strcmp (basename + 3, "name-lookup.h") == 0)
-    output_name = "gt-cp-name-lookup.h", for_name = "cp/name-lookup.c";
-  /* APPLE LOCAL end mainline */
-  else if (strncmp (basename, "objc", 4) == 0 && IS_DIR_SEPARATOR (basename[4])
-	   && strcmp (basename + 5, "objc-act.h") == 0)
-    output_name = "gt-objc-objc-act.h", for_name = "objc/objc-act.c";
-  else 
+  else
     {
       size_t i;
 
@@ -1415,8 +1396,7 @@ struct walk_type_data
   int used_length;
   type_p orig_s;
   const char *reorder_fn;
-  bool needs_cast_p;
-  bool fn_wants_lvalue;
+  int needs_cast_p;
 };
 
 /* Print a mangled name representing T to OF.  */
@@ -1453,7 +1433,7 @@ output_mangled_typename (outf_p of, type_p t)
       }
       break;
     case TYPE_ARRAY:
-      gcc_unreachable ();
+      abort ();
     }
 }
 
@@ -1517,12 +1497,11 @@ walk_type (type_p t, struct walk_type_data *d)
   int use_param_num = -1;
   int use_params_p = 0;
   options_p oo;
-  const struct nested_ptr_data *nested_ptr_d = NULL;
 
-  d->needs_cast_p = false;
+  d->needs_cast_p = 0;
   for (oo = d->opt; oo; oo = oo->next)
     if (strcmp (oo->name, "length") == 0)
-      length = oo->info;
+      length = (const char *)oo->info;
     else if (strcmp (oo->name, "maybe_undef") == 0)
       maybe_undef_p = 1;
     else if (strncmp (oo->name, "use_param", 9) == 0
@@ -1531,9 +1510,7 @@ walk_type (type_p t, struct walk_type_data *d)
     else if (strcmp (oo->name, "use_params") == 0)
       use_params_p = 1;
     else if (strcmp (oo->name, "desc") == 0)
-      desc = oo->info;
-    else if (strcmp (oo->name, "nested_ptr") == 0)
-      nested_ptr_d = (const struct nested_ptr_data *) oo->info;
+      desc = (const char *)oo->info;
     else if (strcmp (oo->name, "dot") == 0)
       ;
     else if (strcmp (oo->name, "tag") == 0)
@@ -1619,7 +1596,7 @@ walk_type (type_p t, struct walk_type_data *d)
 	if (maybe_undef_p
 	    && t->u.p->u.s.line.file == NULL)
 	  {
-	    oprintf (d->of, "%*sgcc_assert (!%s);\n", d->indent, "", d->val);
+	    oprintf (d->of, "%*sif (%s) abort();\n", d->indent, "", d->val);
 	    break;
 	  }
 
@@ -1634,53 +1611,7 @@ walk_type (type_p t, struct walk_type_data *d)
 		break;
 	      }
 
-	    if (nested_ptr_d)
-	      {
-		const char *oldprevval2 = d->prev_val[2];
-
-		if (! UNION_OR_STRUCT_P (nested_ptr_d->type))
-		  {
-		    error_at_line (d->line,
-				   "field `%s' has invalid "
-				   "option `nested_ptr'\n",
-				   d->val);
-		    return;
-		  }
-
-		d->prev_val[2] = d->val;
-		oprintf (d->of, "%*s{\n", d->indent, "");
-		d->indent += 2;
-		d->val = xasprintf ("x%d", d->counter++);
-		oprintf (d->of, "%*s%s %s * %s%s =\n", d->indent, "",
-			 (nested_ptr_d->type->kind == TYPE_UNION 
-			  ? "union" : "struct"), 
-			 nested_ptr_d->type->u.s.tag, 
-			 d->fn_wants_lvalue ? "" : "const ",
-			 d->val);
-		oprintf (d->of, "%*s", d->indent + 2, "");
-		output_escaped_param (d, nested_ptr_d->convert_from,
-				      "nested_ptr");
-		oprintf (d->of, ";\n");
-
-		d->process_field (nested_ptr_d->type, d);
-
-		if (d->fn_wants_lvalue)
-		  {
-		    oprintf (d->of, "%*s%s = ", d->indent, "",
-			     d->prev_val[2]);
-		    d->prev_val[2] = d->val;
-		    output_escaped_param (d, nested_ptr_d->convert_to,
-					  "nested_ptr");
-		    oprintf (d->of, ";\n");
-		  }
-
-		d->indent -= 2;
-		oprintf (d->of, "%*s}\n", d->indent, "");
-		d->val = d->prev_val[2];
-		d->prev_val[2] = oldprevval2;
-	      }
-	    else
-	      d->process_field (t->u.p, d);
+	    d->process_field (t->u.p, d);
 	  }
 	else
 	  {
@@ -1692,7 +1623,7 @@ walk_type (type_p t, struct walk_type_data *d)
 	    oprintf (d->of, "%*sif (%s != NULL) {\n", d->indent, "", d->val);
 	    d->indent += 2;
 	    oprintf (d->of, "%*ssize_t i%d;\n", d->indent, "", loopcounter);
-	    oprintf (d->of, "%*sfor (i%d = 0; i%d != (size_t)(", d->indent, "",
+	    oprintf (d->of, "%*sfor (i%d = 0; i%d < (size_t)(", d->indent, "",
 		     loopcounter, loopcounter);
 	    output_escaped_param (d, length, "length");
 	    oprintf (d->of, "); i%d++) {\n", loopcounter);
@@ -1728,7 +1659,7 @@ walk_type (type_p t, struct walk_type_data *d)
 	oprintf (d->of, "%*s{\n", d->indent, "");
 	d->indent += 2;
 	oprintf (d->of, "%*ssize_t i%d;\n", d->indent, "", loopcounter);
-	oprintf (d->of, "%*sfor (i%d = 0; i%d != (size_t)(", d->indent, "",
+	oprintf (d->of, "%*sfor (i%d = 0; i%d < (size_t)(", d->indent, "",
 		 loopcounter, loopcounter);
 	if (length)
 	  output_escaped_param (d, length, "length");
@@ -1774,7 +1705,7 @@ walk_type (type_p t, struct walk_type_data *d)
 	/* Some things may also be defined in the structure's options.  */
 	for (o = t->u.s.opt; o; o = o->next)
 	  if (! desc && strcmp (o->name, "desc") == 0)
-	    desc = o->info;
+	    desc = (const char *)o->info;
 
 	d->prev_val[2] = oldval;
 	d->prev_val[1] = oldprevval2;
@@ -1805,15 +1736,15 @@ walk_type (type_p t, struct walk_type_data *d)
 	    d->reorder_fn = NULL;
 	    for (oo = f->opt; oo; oo = oo->next)
 	      if (strcmp (oo->name, "dot") == 0)
-		dot = oo->info;
+		dot = (const char *)oo->info;
 	      else if (strcmp (oo->name, "tag") == 0)
-		tagid = oo->info;
+		tagid = (const char *)oo->info;
 	      else if (strcmp (oo->name, "skip") == 0)
 		skip_p = 1;
 	      else if (strcmp (oo->name, "default") == 0)
 		default_p = 1;
 	      else if (strcmp (oo->name, "reorder") == 0)
-		d->reorder_fn = oo->info;
+		d->reorder_fn = (const char *)oo->info;
 	      else if (strncmp (oo->name, "use_param", 9) == 0
 		       && (oo->name[9] == '\0' || ISDIGIT (oo->name[9])))
 		use_param_p = 1;
@@ -1852,10 +1783,9 @@ walk_type (type_p t, struct walk_type_data *d)
 	    d->line = &f->line;
 	    d->val = newval = xasprintf ("%s%s%s", oldval, dot, f->name);
 	    d->opt = f->opt;
-	    d->used_length = false;
 
 	    if (union_p && use_param_p && d->param == NULL)
-	      oprintf (d->of, "%*sgcc_unreachable ();\n", d->indent, "");
+	      oprintf (d->of, "%*sabort();\n", d->indent, "");
 	    else
 	      walk_type (f->type, d);
 
@@ -1911,7 +1841,7 @@ walk_type (type_p t, struct walk_type_data *d)
       break;
 
     default:
-      gcc_unreachable ();
+      abort ();
     }
 }
 
@@ -1968,7 +1898,7 @@ write_types_process_field (type_p f, const struct walk_type_data *d)
       break;
 
     default:
-      gcc_unreachable ();
+      abort ();
     }
 }
 
@@ -1981,8 +1911,8 @@ write_types_process_field (type_p f, const struct walk_type_data *d)
 */
 
 static void
-write_func_for_structure (type_p orig_s, type_p s, type_p *param,
-			  const struct write_types_data *wtd)
+write_func_for_structure  (type_p orig_s, type_p s, type_p *param,
+			   const struct write_types_data *wtd)
 {
   const char *fn = s->u.s.line.file;
   int i;
@@ -2002,9 +1932,9 @@ write_func_for_structure (type_p orig_s, type_p s, type_p *param,
 
   for (opt = s->u.s.opt; opt; opt = opt->next)
     if (strcmp (opt->name, "chain_next") == 0)
-      chain_next = opt->info;
+      chain_next = (const char *) opt->info;
     else if (strcmp (opt->name, "chain_prev") == 0)
-      chain_prev = opt->info;
+      chain_prev = (const char *) opt->info;
 
   if (chain_prev != NULL && chain_next == NULL)
     error_at_line (&s->u.s.line, "chain_prev without chain_next");
@@ -2017,7 +1947,7 @@ write_func_for_structure (type_p orig_s, type_p s, type_p *param,
   d.bitmap = s->u.s.bitmap;
   d.param = param;
   d.prev_val[0] = "*x";
-  d.prev_val[1] = "not valid postage";  /* Guarantee an error.  */
+  d.prev_val[1] = "not valid postage";  /* guarantee an error */
   d.prev_val[3] = "x";
   d.val = "(*x)";
 
@@ -2238,14 +2168,14 @@ write_types_local_process_field (type_p f, const struct walk_type_data *d)
       break;
 
     default:
-      gcc_unreachable ();
+      abort ();
     }
 }
 
 /* For S, a structure that's part of ORIG_S, and using parameters
    PARAM, write out a routine that:
    - Is of type gt_note_pointers
-   - Calls PROCESS_FIELD on each field of S or its substructures.
+   - If calls PROCESS_FIELD on each field of S or its substructures.
 */
 
 static void
@@ -2270,19 +2200,15 @@ write_local_func_for_structure (type_p orig_s, type_p s, type_p *param)
   d.bitmap = s->u.s.bitmap;
   d.param = param;
   d.prev_val[0] = d.prev_val[2] = "*x";
-  d.prev_val[1] = "not valid postage";  /* Guarantee an error.  */
+  d.prev_val[1] = "not valid postage";  /* guarantee an error */
   d.prev_val[3] = "x";
   d.val = "(*x)";
-  d.fn_wants_lvalue = true;
 
   oprintf (d.of, "\n");
   oprintf (d.of, "void\n");
   oprintf (d.of, "gt_pch_p_");
   output_mangled_typename (d.of, orig_s);
-  oprintf (d.of, " (ATTRIBUTE_UNUSED void *this_obj,\n"
-	   "\tvoid *x_p,\n"
-	   "\tATTRIBUTE_UNUSED gt_pointer_operator op,\n"
-	   "\tATTRIBUTE_UNUSED void *cookie)\n");
+  oprintf (d.of, " (void *this_obj ATTRIBUTE_UNUSED,\n\tvoid *x_p,\n\tgt_pointer_operator op ATTRIBUTE_UNUSED,\n\tvoid *cookie ATTRIBUTE_UNUSED)\n");
   oprintf (d.of, "{\n");
   oprintf (d.of, "  %s %s * const x ATTRIBUTE_UNUSED = (%s %s *)x_p;\n",
 	   s->kind == TYPE_UNION ? "union" : "struct", s->u.s.tag,
@@ -2380,7 +2306,7 @@ write_local (type_p structures, type_p param_structs)
 /* Write out the 'enum' definition for gt_types_enum.  */
 
 static void
-write_enum_defn (type_p structures, type_p param_structs)
+write_enum_defn  (type_p structures, type_p param_structs)
 {
   type_p s;
 
@@ -2535,7 +2461,7 @@ write_root (outf_p f, pair_p v, type_p type, const char *name, int has_length,
 	      if (strcmp (o->name, "skip") == 0)
 		skip_p = 1;
 	      else if (strcmp (o->name, "desc") == 0)
-		desc = o->info;
+		desc = (const char *)o->info;
 	      else
 		error_at_line (line,
 		       "field `%s' of global `%s' has unknown option `%s'",
@@ -2555,7 +2481,7 @@ write_root (outf_p f, pair_p v, type_p type, const char *name, int has_length,
 
 		    for (oo = ufld->opt; oo; oo = oo->next)
 		      if (strcmp (oo->name, "tag") == 0)
-			tag = oo->info;
+			tag = (const char *)oo->info;
 		    if (tag == NULL || strcmp (tag, desc) != 0)
 		      continue;
 		    if (validf != NULL)
@@ -2697,12 +2623,11 @@ write_array (outf_p f, pair_p v, const struct write_types_data *wtd)
       oprintf (f, "static void gt_%sa_%s\n", wtd->param_prefix, v->name);
       oprintf (f,
        "    (void *, void *, gt_pointer_operator, void *);\n");
-      oprintf (f, "static void gt_%sa_%s (ATTRIBUTE_UNUSED void *this_obj,\n",
+      oprintf (f, "static void gt_%sa_%s (void *this_obj ATTRIBUTE_UNUSED,\n",
 	       wtd->param_prefix, v->name);
-      oprintf (d.of,
-	       "      ATTRIBUTE_UNUSED void *x_p,\n"
-	       "      ATTRIBUTE_UNUSED gt_pointer_operator op,\n"
-	       "      ATTRIBUTE_UNUSED void * cookie)\n");
+      oprintf (d.of, "      void *x_p ATTRIBUTE_UNUSED,\n");
+      oprintf (d.of, "      gt_pointer_operator op ATTRIBUTE_UNUSED,\n");
+      oprintf (d.of, "      void *cookie ATTRIBUTE_UNUSED)\n");
       oprintf (d.of, "{\n");
       d.prev_val[0] = d.prev_val[1] = d.prev_val[2] = d.val = v->name;
       d.process_field = write_types_local_process_field;
@@ -2713,7 +2638,7 @@ write_array (outf_p f, pair_p v, const struct write_types_data *wtd)
   d.opt = v->opt;
   oprintf (f, "static void gt_%sa_%s (void *);\n",
 	   wtd->prefix, v->name);
-  oprintf (f, "static void\ngt_%sa_%s (ATTRIBUTE_UNUSED void *x_p)\n",
+  oprintf (f, "static void\ngt_%sa_%s (void *x_p ATTRIBUTE_UNUSED)\n",
 	   wtd->prefix, v->name);
   oprintf (f, "{\n");
   d.prev_val[0] = d.prev_val[1] = d.prev_val[2] = d.val = v->name;
@@ -2741,7 +2666,7 @@ write_roots (pair_p variables)
 
       for (o = v->opt; o; o = o->next)
 	if (strcmp (o->name, "length") == 0)
-	  length = o->info;
+	  length = (const char *)o->info;
 	else if (strcmp (o->name, "deletable") == 0)
 	  deletable_p = 1;
 	else if (strcmp (o->name, "param_is") == 0)
@@ -2762,7 +2687,7 @@ write_roots (pair_p variables)
 	  break;
       if (fli == NULL)
 	{
-	  fli = XNEW (struct flist);
+	  fli = xmalloc (sizeof (*fli));
 	  fli->f = f;
 	  fli->next = flp;
 	  fli->started_p = 0;
@@ -2866,7 +2791,7 @@ write_roots (pair_p variables)
 	if (strcmp (o->name, "length") == 0)
 	  length_p = 1;
 	else if (strcmp (o->name, "if_marked") == 0)
-	  if_marked = o->info;
+	  if_marked = (const char *) o->info;
 
       if (if_marked == NULL)
 	continue;
@@ -2974,7 +2899,7 @@ write_roots (pair_p variables)
 
 extern int main (int argc, char **argv);
 int
-main(int ARG_UNUSED (argc), char ** ARG_UNUSED (argv))
+main(int argc ATTRIBUTE_UNUSED, char **argv ATTRIBUTE_UNUSED)
 {
   unsigned i;
   static struct fileloc pos = { __FILE__, __LINE__ };
@@ -2989,14 +2914,10 @@ main(int ARG_UNUSED (argc), char ** ARG_UNUSED (argv))
   do_scalar_typedef ("uint8", &pos);
   do_scalar_typedef ("jword", &pos);
   do_scalar_typedef ("JCF_u2", &pos);
-#ifdef USE_MAPPED_LOCATION
-  do_scalar_typedef ("location_t", &pos);
-  do_scalar_typedef ("source_locus", &pos);
-#endif
-  do_scalar_typedef ("void", &pos);
-
-  do_typedef ("PTR", create_pointer (resolve_typedef ("void", &pos)), &pos);
 
+  do_typedef ("PTR", create_pointer (create_scalar_type ("void",
+							 strlen ("void"))),
+	      &pos);
   do_typedef ("HARD_REG_SET", create_array (
 	      create_scalar_type ("unsigned long", strlen ("unsigned long")),
 	      "2"), &pos);
@@ -3015,12 +2936,6 @@ main(int ARG_UNUSED (argc), char ** ARG_UNUSED (argv))
         }
       if (!dupflag)
         parse_file (all_files[i]);
-#ifndef USE_MAPPED_LOCATION
-      /* temporary kludge - gengtype doesn't handle conditionals.
-	 Manually add source_locus *after* we've processed input.h.  */
-      if (i == 0)
-	do_typedef ("source_locus", create_pointer (resolve_typedef ("location_t", &pos)), &pos);
-#endif
     }
 
   if (hit_error != 0)
