@@ -95,7 +95,7 @@ public:
     DEAD        = 1 << 8,   // This node is dead and should not be pointed to
     //#endif
 
-    Composition = AllocaNode | HeapNode | GlobalNode | UnknownNode,
+    Composition = AllocaNode | HeapNode | GlobalNode | UnknownNode
   };
 
   /// NodeType - A union of the above bits.  "Shadow" nodes do not add any flags
@@ -182,7 +182,7 @@ public:
     delete this;
   }
 
-  /// hasLink - Return true if this memory object has a link in slot #LinkNo
+  /// hasLink - Return true if this memory object has a link in slot LinkNo
   ///
   bool hasLink(unsigned Offset) const {
     assert((Offset & ((1 << DS::PointerShift)-1)) == 0 &&
@@ -424,12 +424,16 @@ inline DSNode *DSNodeHandle::getNode() const {
   // Disabling this assertion because it is failing on a "magic" struct
   // in named (from bind).  The fourth field is an array of length 0,
   // presumably used to create struct instances of different sizes.
-  assert((!N ||
+  // In a variable length struct, Offset could exceed Size when getNode()
+  // is called before such a node is folded. In this case, the DS Analysis now 
+  // correctly folds this node after calling getNode.
+  /*  assert((!N ||
           N->isNodeCompletelyFolded() ||
           (N->Size == 0 && Offset == 0) ||
           (int(Offset) >= 0 && Offset < N->Size) ||
           (int(Offset) < 0 && -int(Offset) < int(N->Size)) ||
           N->isForwarding()) && "Node handle offset out of range!");
+  */
   if (N == 0 || !N->isForwarding())
     return N;
 
