@@ -35,65 +35,116 @@ class GlobalValue;
 ///   6. An index into the GOT, if the target uses a GOT
 ///
 class MachineRelocation {
-  /// OffsetTypeExternal - The low 24-bits of this value is the offset from the
-  /// start of the code buffer of the relocation to perform.  Bit 24 of this is
-  /// set if Target should use ExtSym instead of GV, Bit 25 is the CanRewrite
-  /// bit, and the high 6 bits hold the relocation type.
-  // FIXME: with the additional types of relocatable things, rearrange the
-  // storage of things to be a bit more effiecient
-  unsigned OffsetTypeExternal;
+  enum AddressType {
+    isResult,         // Relocation has be transformed into its result pointer.
+    isGV,             // The Target.GV field is valid.
+    isExtSym,         // The Target.ExtSym field is valid.
+    isConstPool,      // Relocation of constant pool address.
+    isJumpTable,      // Relocation of jump table address.
+    isGOTIndex        // The Target.GOTIndex field is valid.
+  };
+  
+  /// Offset - This is the offset from the start of the code buffer of the
+  /// relocation to perform.
+  intptr_t Offset;
+  
+  /// ConstantVal - A field that may be used by the target relocation type.
+  intptr_t ConstantVal;
+
   union {
+    void *Result;        // If this has been resolved to a resolved pointer
     GlobalValue *GV;     // If this is a pointer to an LLVM global
     const char *ExtSym;  // If this is a pointer to a named symbol
-    void *Result;        // If this has been resolved to a resolved pointer
+    unsigned Index;      // Constant pool / jump table index
     unsigned GOTIndex;   // Index in the GOT of this symbol/global
-    unsigned CPool;      // Index in the Constant Pool
   } Target;
-  intptr_t ConstantVal;
-  bool GOTRelative; //out of bits in OffsetTypeExternal
-  bool isConstPool;
+
+  unsigned TargetReloType : 6; // The target relocation ID.
+  AddressType AddrType    : 3; // The field of Target to use.
+  bool DoesntNeedFnStub   : 1; // True if we don't need a fn stub.
+  bool GOTRelative        : 1; // Should this relocation be relative to the GOT?
 
 public:
-  MachineRelocation(unsigned Offset, unsigned RelocationType, GlobalValue *GV,
-                    intptr_t cst = 0, bool DoesntNeedFunctionStub = 0,
-                    bool GOTrelative = 0)
-    : OffsetTypeExternal(Offset + (RelocationType << 26)), ConstantVal(cst),
-      GOTRelative(GOTrelative), isConstPool(0) {
-    assert((Offset & ~((1 << 24)-1)) == 0 && "Code offset too large!");
+  /// MachineRelocation::getGV - Return a relocation entry for a GlobalValue.
+  ///
+  static MachineRelocation getGV(intptr_t offset, unsigned RelocationType, 
+                                 GlobalValue *GV, intptr_t cst = 0,
+                                 bool DoesntNeedFunctionStub = 0,
+                                 bool GOTrelative = 0) {
     assert((RelocationType & ~63) == 0 && "Relocation type too large!");
-    Target.GV = GV;
-    if (DoesntNeedFunctionStub)
-      OffsetTypeExternal |= 1 << 25;
+    MachineRelocation Result;
+    Result.Offset = offset;
+    Result.ConstantVal = cst;
+    Result.TargetReloType = RelocationType;
+    Result.AddrType = isGV;
+    Result.DoesntNeedFnStub = DoesntNeedFunctionStub;
+    Result.GOTRelative = GOTrelative;
+    Result.Target.GV = GV;
+    return Result;
   }
 
-  MachineRelocation(unsigned Offset, unsigned RelocationType, const char *ES,
-                    intptr_t cst = 0, bool GOTrelative = 0)
-    : OffsetTypeExternal(Offset + (1 << 24) + (RelocationType << 26)),
-      ConstantVal(cst), GOTRelative(GOTrelative), isConstPool(0) {
-    assert((Offset & ~((1 << 24)-1)) == 0 && "Code offset too large!");
+  /// MachineRelocation::getExtSym - Return a relocation entry for an external
+  /// symbol, like "free".
+  ///
+  static MachineRelocation getExtSym(intptr_t offset, unsigned RelocationType, 
+                                     const char *ES, intptr_t cst = 0,
+                                     bool GOTrelative = 0) {
     assert((RelocationType & ~63) == 0 && "Relocation type too large!");
-    Target.ExtSym = ES;
+    MachineRelocation Result;
+    Result.Offset = offset;
+    Result.ConstantVal = cst;
+    Result.TargetReloType = RelocationType;
+    Result.AddrType = isExtSym;
+    Result.DoesntNeedFnStub = false;
+    Result.GOTRelative = GOTrelative;
+    Result.Target.ExtSym = ES;
+    return Result;
   }
 
-  MachineRelocation(unsigned Offset, unsigned RelocationType, unsigned CPI,
-                    intptr_t cst = 0)
-    : OffsetTypeExternal(Offset + (RelocationType << 26)),
-      ConstantVal(cst), GOTRelative(0), isConstPool(1) {
-    assert((Offset & ~((1 << 24)-1)) == 0 && "Code offset too large!");
+  /// MachineRelocation::getConstPool - Return a relocation entry for a constant
+  /// pool entry.
+  ///
+  static MachineRelocation getConstPool(intptr_t offset,unsigned RelocationType,
+                                        unsigned CPI, intptr_t cst = 0) {
     assert((RelocationType & ~63) == 0 && "Relocation type too large!");
-    Target.CPool = CPI;
+    MachineRelocation Result;
+    Result.Offset = offset;
+    Result.ConstantVal = cst;
+    Result.TargetReloType = RelocationType;
+    Result.AddrType = isConstPool;
+    Result.DoesntNeedFnStub = false;
+    Result.GOTRelative = false;
+    Result.Target.Index = CPI;
+    return Result;
+  }
+
+  /// MachineRelocation::getJumpTable - Return a relocation entry for a jump
+  /// table entry.
+  ///
+  static MachineRelocation getJumpTable(intptr_t offset,unsigned RelocationType,
+                                        unsigned JTI, intptr_t cst = 0) {
+    assert((RelocationType & ~63) == 0 && "Relocation type too large!");
+    MachineRelocation Result;
+    Result.Offset = offset;
+    Result.ConstantVal = cst;
+    Result.TargetReloType = RelocationType;
+    Result.AddrType = isJumpTable;
+    Result.DoesntNeedFnStub = false;
+    Result.GOTRelative = false;
+    Result.Target.Index = JTI;
+    return Result;
   }
 
   /// getMachineCodeOffset - Return the offset into the code buffer that the
   /// relocation should be performed.
-  unsigned getMachineCodeOffset() const {
-    return OffsetTypeExternal & ((1 << 24)-1);
+  intptr_t getMachineCodeOffset() const {
+    return Offset;
   }
 
   /// getRelocationType - Return the target-specific relocation ID for this
   /// relocation.
   unsigned getRelocationType() const {
-    return OffsetTypeExternal >> 26;
+    return TargetReloType;
   }
 
   /// getConstantVal - Get the constant value associated with this relocation.
@@ -106,19 +157,25 @@ public:
   /// isGlobalValue - Return true if this relocation is a GlobalValue, as
   /// opposed to a constant string.
   bool isGlobalValue() const {
-    return (OffsetTypeExternal & (1 << 24)) == 0 && !isConstantPoolIndex();
+    return AddrType == isGV;
   }
 
   /// isString - Return true if this is a constant string.
   ///
   bool isString() const {
-    return !isGlobalValue() && !isConstantPoolIndex();
+    return AddrType == isExtSym;
   }
 
   /// isConstantPoolIndex - Return true if this is a constant pool reference.
   ///
   bool isConstantPoolIndex() const {
-    return isConstPool;
+    return AddrType == isConstPool;
+  }
+
+  /// isJumpTableIndex - Return true if this is a jump table reference.
+  ///
+  bool isJumpTableIndex() const {
+    return AddrType == isJumpTable;
   }
 
   /// isGOTRelative - Return true the target wants the index into the GOT of
@@ -133,7 +190,7 @@ public:
   /// to be false, but targets can improve their compilation callback functions
   /// to handle more general cases if they want improved performance.
   bool doesntNeedFunctionStub() const {
-    return (OffsetTypeExternal & (1 << 25)) != 0;
+    return DoesntNeedFnStub;
   }
 
   /// getGlobalValue - If this is a global value reference, return the
@@ -154,12 +211,20 @@ public:
   /// the index into the constant pool.
   unsigned getConstantPoolIndex() const {
     assert(isConstantPoolIndex() && "This is not a constant pool reference!");
-    return Target.CPool;
+    return Target.Index;
+  }
+
+  /// getJumpTableIndex - If this is a jump table reference, return
+  /// the index into the jump table.
+  unsigned getJumpTableIndex() const {
+    assert(isJumpTableIndex() && "This is not a jump table reference!");
+    return Target.Index;
   }
 
   /// getResultPointer - Once this has been resolved to point to an actual
   /// address, this returns the pointer.
   void *getResultPointer() const {
+    assert(AddrType == isResult && "Result pointer isn't set yet!");
     return Target.Result;
   }
 
@@ -167,10 +232,12 @@ public:
   ///
   void setResultPointer(void *Ptr) {
     Target.Result = Ptr;
+    AddrType = isResult;
   }
 
   /// setGOTIndex - Set the GOT index to a specific value.
   void setGOTIndex(unsigned idx) {
+    AddrType = isGOTIndex;
     Target.GOTIndex = idx;
   }
 
@@ -178,11 +245,10 @@ public:
   /// this returns that index.  The index is from the lowest address entry
   /// in the GOT.
   unsigned getGOTIndex() const {
+    assert(AddrType == isGOTIndex);
     return Target.GOTIndex;
   }
-
 };
-
 }
 
 #endif
