@@ -18,8 +18,9 @@
 #include "llvm/CodeGen/SelectionDAGNodes.h"
 #include "llvm/ADT/ilist"
 
-#include <map>
 #include <list>
+#include <map>
+#include <set>
 #include <string>
 
 namespace llvm {
@@ -120,6 +121,8 @@ public:
                                    int offset = 0);
   SDOperand getFrameIndex(int FI, MVT::ValueType VT);
   SDOperand getTargetFrameIndex(int FI, MVT::ValueType VT);
+  SDOperand getJumpTable(int JTI, MVT::ValueType VT);
+  SDOperand getTargetJumpTable(int JTI, MVT::ValueType VT);
   SDOperand getConstantPool(Constant *C, MVT::ValueType VT,
                            unsigned Alignment=0,  int offset = 0);
   SDOperand getTargetConstantPool(Constant *C, MVT::ValueType VT,
@@ -427,6 +430,16 @@ public:
   static void InsertISelMapEntry(std::map<SDOperand, SDOperand> &Map,
                                  SDNode *Key, unsigned KeyResNo,
                                  SDNode *Element, unsigned ElementResNo);
+
+  /// InsertInFlightSetEntry - A helper function to insert a SDNode* to a
+  /// SDNode* set. This is added to avoid the set insertion operator from being
+  /// inlined.
+  static void InsertInFlightSetEntry(std::set<SDNode*> &Set, SDNode *N);
+
+  /// RemoveInFlightSetEntry - A helper function to remove a SDNode* from a
+  /// SDNode* set. This is added to avoid the set removal operator from being
+  /// inlined.
+  static void RemoveInFlightSetEntry(std::set<SDNode*> &Set, SDNode *N);
   
 private:
   void RemoveNodeFromCSEMaps(SDNode *N);
@@ -468,7 +481,8 @@ private:
   std::map<std::pair<uint64_t, MVT::ValueType>, SDNode*> TargetConstants;
   std::map<std::pair<uint64_t, MVT::ValueType>, SDNode*> ConstantFPs;
   std::map<std::pair<uint64_t, MVT::ValueType>, SDNode*> TargetConstantFPs;
-  std::map<int, SDNode*> FrameIndices, TargetFrameIndices;
+  std::map<int, SDNode*> FrameIndices, TargetFrameIndices, JumpTableIndices,
+    TargetJumpTableIndices;
   std::map<std::pair<Constant *,
                      std::pair<int, unsigned> >, SDNode*> ConstantPoolIndices;
   std::map<std::pair<Constant *,
