@@ -19,7 +19,7 @@
 #include <map>
 #include <cassert>
 #include <string>
-#include "llvm/Support/MutexGuard.h"
+#include "llvm/System/Mutex.h"
 
 namespace llvm {
 
@@ -32,8 +32,7 @@ class Module;
 class ModuleProvider;
 class TargetData;
 class Type;
-class IntrinsicLowering;
-
+class MutexGuard;
 
 class ExecutionEngineState {
 private:
@@ -48,11 +47,13 @@ private:
   std::map<void *, const GlobalValue*> GlobalAddressReverseMap;
 
 public:
-  std::map<const GlobalValue*, void *>& getGlobalAddressMap(const MutexGuard& locked) {
+  std::map<const GlobalValue*, void *> &
+  getGlobalAddressMap(const MutexGuard &locked) {
     return GlobalAddressMap;
   }
 
-  std::map<void *, const GlobalValue*>& getGlobalAddressReverseMap(const MutexGuard& locked) {
+  std::map<void*, const GlobalValue*> & 
+  getGlobalAddressReverseMap(const MutexGuard& locked) {
     return GlobalAddressReverseMap;
   }
 };
@@ -67,13 +68,20 @@ class ExecutionEngine {
 protected:
   ModuleProvider *MP;
 
-  void setTargetData(const TargetData &td) {
-    TD = &td;
+  void setTargetData(const TargetData *td) {
+    TD = td;
   }
 
+  // To avoid having libexecutionengine depend on the JIT and interpreter
+  // libraries, the JIT and Interpreter set these functions to ctor pointers
+  // at startup time if they are linked in.
+  typedef ExecutionEngine *(*EECtorFn)(ModuleProvider*);
+  static EECtorFn JITCtor, InterpCtor;
+    
 public:
-  /// lock - This lock is protects the ExecutionEngine, JIT, JITResolver and JITEmitter classes.
-  /// It must be held while changing the internal state of any of those classes.
+  /// lock - This lock is protects the ExecutionEngine, JIT, JITResolver and
+  /// JITEmitter classes.  It must be held while changing the internal state of
+  /// any of those classes.
   sys::Mutex lock; // Used to make this class and subclasses thread-safe
 
   ExecutionEngine(ModuleProvider *P);
@@ -81,13 +89,12 @@ public:
   virtual ~ExecutionEngine();
 
   Module &getModule() const { return CurMod; }
-  const TargetData &getTargetData() const { return *TD; }
+  const TargetData *getTargetData() const { return TD; }
 
   /// create - This is the factory method for creating an execution engine which
-  /// is appropriate for the current machine.  If specified, the
-  /// IntrinsicLowering implementation should be allocated on the heap.
-  static ExecutionEngine *create(ModuleProvider *MP, bool ForceInterpreter,
-                                 IntrinsicLowering *IL = 0);
+  /// is appropriate for the current machine.
+  static ExecutionEngine *create(ModuleProvider *MP,
+                                 bool ForceInterpreter = false);
 
   /// runFunction - Execute the specified function with the specified arguments,
   /// and return the result.
@@ -95,6 +102,12 @@ public:
   virtual GenericValue runFunction(Function *F,
                                 const std::vector<GenericValue> &ArgValues) = 0;
 
+  /// runStaticConstructorsDestructors - This method is used to execute all of
+  /// the static constructors or destructors for a module, depending on the
+  /// value of isDtors.
+  void runStaticConstructorsDestructors(bool isDtors);
+  
+  
   /// runFunctionAsMain - This is a helper function which wraps runFunction to
   /// handle the common task of starting up main with the specified argc, argv,
   /// and envp parameters.
@@ -102,57 +115,27 @@ public:
                         const char * const * envp);
 
 
-  void addGlobalMapping(const GlobalValue *GV, void *Addr) {
-    MutexGuard locked(lock);
-
-    void *&CurVal = state.getGlobalAddressMap(locked)[GV];
-    assert((CurVal == 0 || Addr == 0) && "GlobalMapping already established!");
-    CurVal = Addr;
-
-    // If we are using the reverse mapping, add it too
-    if (!state.getGlobalAddressReverseMap(locked).empty()) {
-      const GlobalValue *&V = state.getGlobalAddressReverseMap(locked)[Addr];
-      assert((V == 0 || GV == 0) && "GlobalMapping already established!");
-      V = GV;
-    }
-  }
-
+  /// addGlobalMapping - Tell the execution engine that the specified global is
+  /// at the specified location.  This is used internally as functions are JIT'd
+  /// and as global variables are laid out in memory.  It can and should also be
+  /// used by clients of the EE that want to have an LLVM global overlay
+  /// existing data in memory.
+  void addGlobalMapping(const GlobalValue *GV, void *Addr);
+  
   /// clearAllGlobalMappings - Clear all global mappings and start over again
   /// use in dynamic compilation scenarios when you want to move globals
-  void clearAllGlobalMappings() {
-    MutexGuard locked(lock);
-
-    state.getGlobalAddressMap(locked).clear();
-    state.getGlobalAddressReverseMap(locked).clear();
-  }
-
+  void clearAllGlobalMappings();
+  
   /// updateGlobalMapping - Replace an existing mapping for GV with a new
-  /// address.  This updates both maps as required.
-  void updateGlobalMapping(const GlobalValue *GV, void *Addr) {
-    MutexGuard locked(lock);
-
-    void *&CurVal = state.getGlobalAddressMap(locked)[GV];
-    if (CurVal && !state.getGlobalAddressReverseMap(locked).empty())
-      state.getGlobalAddressReverseMap(locked).erase(CurVal);
-    CurVal = Addr;
-
-    // If we are using the reverse mapping, add it too
-    if (!state.getGlobalAddressReverseMap(locked).empty()) {
-      const GlobalValue *&V = state.getGlobalAddressReverseMap(locked)[Addr];
-      assert((V == 0 || GV == 0) && "GlobalMapping already established!");
-      V = GV;
-    }
-  }
-
+  /// address.  This updates both maps as required.  If "Addr" is null, the
+  /// entry for the global is removed from the mappings.
+  void updateGlobalMapping(const GlobalValue *GV, void *Addr);
+  
   /// getPointerToGlobalIfAvailable - This returns the address of the specified
-  /// global value if it is available, otherwise it returns null.
+  /// global value if it is has already been codegen'd, otherwise it returns
+  /// null.
   ///
-  void *getPointerToGlobalIfAvailable(const GlobalValue *GV) {
-    MutexGuard locked(lock);
-
-    std::map<const GlobalValue*, void*>::iterator I = state.getGlobalAddressMap(locked).find(GV);
-    return I != state.getGlobalAddressMap(locked).end() ? I->second : 0;
-  }
+  void *getPointerToGlobalIfAvailable(const GlobalValue *GV);
 
   /// getPointerToGlobal - This returns the address of the specified global
   /// value.  This may involve code generation if it's a function.
