@@ -48,6 +48,13 @@ void ParseCommandLineOptions(int &argc, char **argv,
 void ParseEnvironmentOptions(const char *progName, const char *envvar,
                              const char *Overview = 0);
 
+///===---------------------------------------------------------------------===//
+/// SetVersionPrinter - Override the default (LLVM specific) version printer
+///                     used to print out the version when --version is given
+///                     on the command line. This gives other systems using the
+///                     CommandLine utilities to print their own version string.
+void SetVersionPrinter(void (*func)());
+
 //===----------------------------------------------------------------------===//
 // Flags permitted to be passed to command line arguments
 //
@@ -143,6 +150,9 @@ class Option {
     return NormalFormatting;
   }
 
+  // Out of line virtual function to provide home for the class.
+  virtual void anchor();
+  
   int NumOccurrences;   // The number of times specified
   int Flags;            // Flags for the argument
   unsigned Position;    // Position of last occurrence of the option
@@ -997,7 +1007,7 @@ class bits_storage {
   
   template<class T>
   static unsigned Bit(const T &V) {
-    unsigned BitPos = (unsigned)V;
+    unsigned BitPos = reinterpret_cast<unsigned>(V);
     assert(BitPos < sizeof(unsigned) * 8 &&
           "enum exceeds width of bit vector!");
     return 1 << BitPos;
@@ -1038,7 +1048,7 @@ class bits_storage<DataType, bool> {
   
   template<class T>
   static unsigned Bit(const T &V) {
-    unsigned BitPos = (unsigned)V;
+    unsigned BitPos = reinterpret_cast<unsigned>(V);
     assert(BitPos < sizeof(unsigned) * 8 &&
           "enum exceeds width of bit vector!");
     return 1 << BitPos;
