@@ -15,6 +15,7 @@
 #define LLVM_SUPPORT_MATHEXTRAS_H
 
 #include "llvm/Support/DataTypes.h"
+#include "llvm/System/IncludeFile.h"
 
 namespace llvm {
 
@@ -25,21 +26,33 @@ namespace llvm {
 
 // Hi_32 - This function returns the high 32 bits of a 64 bit value.
 inline unsigned Hi_32(uint64_t Value) {
-  return (unsigned)(Value >> 32);
+  return static_cast<unsigned>(Value >> 32);
 }
 
 // Lo_32 - This function returns the low 32 bits of a 64 bit value.
 inline unsigned Lo_32(uint64_t Value) {
-  return (unsigned)Value;
+  return static_cast<unsigned>(Value);
 }
 
 // is?Type - these functions produce optimal testing for integer data types.
-inline bool isInt8  (int Value)     { return (  signed char )Value == Value; }
-inline bool isUInt8 (int Value)     { return (unsigned char )Value == Value; }
-inline bool isInt16 (int Value)     { return (  signed short)Value == Value; }
-inline bool isUInt16(int Value)     { return (unsigned short)Value == Value; }
-inline bool isInt32 (int64_t Value) { return (  signed int  )Value == Value; }
-inline bool isUInt32(int64_t Value) { return (unsigned int  )Value == Value; }
+inline bool isInt8  (int Value)     { 
+  return static_cast<signed char>(Value) == Value; 
+}
+inline bool isUInt8 (int Value)     { 
+  return static_cast<unsigned char>(Value) == Value; 
+}
+inline bool isInt16 (int Value)     { 
+  return static_cast<signed short>(Value) == Value; 
+}
+inline bool isUInt16(int Value)     { 
+  return static_cast<unsigned short>(Value) == Value; 
+}
+inline bool isInt32 (int64_t Value) { 
+  return static_cast<signed int>(Value) == Value; 
+}
+inline bool isUInt32(int64_t Value) { 
+  return static_cast<unsigned int>(Value) == Value; 
+}
 
 // isMask_32 - This function returns true if the argument is a sequence of ones  
 // starting at the least significant bit with the remainder zero (32 bit version.)
@@ -76,7 +89,7 @@ inline bool isPowerOf2_32(unsigned Value) {
 // isPowerOf2_64 - This function returns true if the argument is a power of two
 // > 0 (64 bit edition.)
 inline bool isPowerOf2_64(uint64_t Value) {
-  return Value && !(Value & (Value - 1LL));
+  return Value && !(Value & (Value - int64_t(1L)));
 }
 
 // ByteSwap_16 - This function returns a byte-swapped representation of the
@@ -216,7 +229,7 @@ inline unsigned CountPopulation_64(uint64_t Value) {
 
 // Log2_32 - This function returns the floor log base 2 of the specified value, 
 // -1 if the value is zero. (32 bit edition.)
-// Ex. Log2_32(32) == 5, Log2_32(1) == 0, Log2_32(0) == -1
+// Ex. Log2_32(32) == 5, Log2_32(1) == 0, Log2_32(0) == -1, Log2_32(6) == 2
 inline unsigned Log2_32(unsigned Value) {
     return 31 - CountLeadingZeros_32(Value);
 }
@@ -227,6 +240,19 @@ inline unsigned Log2_64(uint64_t Value) {
     return 63 - CountLeadingZeros_64(Value);
 }
 
+// Log2_32_Ceil - This function returns the ceil log base 2 of the specified
+// value, 32 if the value is zero. (32 bit edition).
+// Ex. Log2_32_Ceil(32) == 5, Log2_32_Ceil(1) == 0, Log2_32_Ceil(6) == 3
+inline unsigned Log2_32_Ceil(unsigned Value) {
+  return 32-CountLeadingZeros_32(Value-1);
+}
+
+// Log2_64 - This function returns the ceil log base 2 of the specified value, 
+// 64 if the value is zero. (64 bit edition.)
+inline unsigned Log2_64_Ceil(uint64_t Value) {
+  return 64-CountLeadingZeros_64(Value-1);
+}
+
 // BitsToDouble - This function takes a 64-bit integer and returns the bit
 // equivalent double.
 inline double BitsToDouble(uint64_t Bits) {
@@ -281,4 +307,7 @@ int IsInf (double d);
 
 } // End llvm namespace
 
+FORCE_DEFINING_FILE_TO_BE_LINKED(SupportIsInf)
+FORCE_DEFINING_FILE_TO_BE_LINKED(SupportIsNAN)
+
 #endif
