@@ -22,6 +22,8 @@
 namespace llvm {
   struct PluginLoader {
     void operator=(const std::string &Filename);
+    static unsigned getNumPlugins();
+    static std::string& getPlugin(unsigned num);
   };
 
 #ifndef DONT_GET_PLUGIN_LOADER_OPTION
