@@ -15,6 +15,7 @@
 #define LLVM_SYSTEM_DYNAMIC_LIBRARY_H
 
 #include "llvm/System/Path.h"
+#include "llvm/System/IncludeFile.h"
 #include <string>
 
 namespace llvm {
@@ -26,6 +27,9 @@ namespace sys {
   /// operating system interface, this class provides a portable interface that
   /// allows dynamic libraries to be loaded and and searched for externally
   /// defined symbols. This is typically used to provide "plug-in" support.
+  /// It also allows for symbols to be defined which don't live in any library,
+  /// but rather the main program itself, useful on Windows where the main
+  /// executable cannot be searched.
   /// @since 1.4
   /// @brief Portable dynamic library abstraction.
   class DynamicLibrary {
@@ -42,7 +46,7 @@ namespace sys {
       DynamicLibrary();
 
       /// This is the constructor for DynamicLibrary instances. It will open
-      /// the dynamic library specified by the \filename Path.
+      /// the dynamic library specified by the filename Path.
       /// @throws std::string indicating why the library couldn't be opened.
       /// @brief Open a dynamic library.
       DynamicLibrary(const char* filename);
@@ -62,10 +66,11 @@ namespace sys {
     public:
       /// This function allows a library to be loaded without instantiating a
       /// DynamicLibrary object. Consequently, it is marked as being permanent
-      /// and will only be unloaded when the program terminates.
-      /// @throws std::string on error.
+      /// and will only be unloaded when the program terminates.  This returns
+      /// false on success or returns true and fills in *ErrMsg on failure.
       /// @brief Open a dynamic library permanently.
-      static void LoadLibraryPermanently(const char* filename);
+      static bool LoadLibraryPermanently(const char* filename,
+                                         std::string *ErrMsg = 0);
 
       /// This function will search through all previously loaded dynamic
       /// libraries for the symbol \p symbolName. If it is found, the addressof
@@ -81,6 +86,17 @@ namespace sys {
         return SearchForAddressOfSymbol(symbolName.c_str());
       }
 
+      /// This functions permanently adds the symbol \p symbolName with the
+      /// value \p symbolValue.  These symbols are searched before any
+      /// libraries.
+      /// @brief Add searchable symbol/value pair.
+      static void AddSymbol(const char* symbolName, void *symbolValue);
+
+      /// @brief Convenience function for C++ophiles.
+      static void AddSymbol(const std::string& symbolName, void *symbolValue) {
+        AddSymbol(symbolName.c_str(), symbolValue);
+      }
+
     /// @}
     /// @name Accessors
     /// @{
@@ -110,4 +126,6 @@ namespace sys {
 } // End sys namespace
 } // End llvm namespace
 
+FORCE_DEFINING_FILE_TO_BE_LINKED(SystemDynamicLibrary)
+
 #endif // LLVM_SYSTEM_DYNAMIC_LIBRARY_H
