@@ -15,10 +15,11 @@
 #define LLVM_SYSTEM_PATH_H
 
 #include "llvm/System/TimeValue.h"
+#include "llvm/System/IncludeFile.h"
 #include <set>
 #include <string>
 #include <vector>
-#include <ostream>
+#include <iosfwd>
 
 namespace llvm {
 namespace sys {
@@ -67,7 +68,7 @@ namespace sys {
       struct StatusInfo {
         StatusInfo() : fileSize(0), modTime(0,0), mode(0777), user(999),
                        group(999), isDir(false) { }
-        size_t      fileSize;   ///< Size of the file in bytes
+        uint64_t    fileSize;   ///< Size of the file in bytes
         TimeValue   modTime;    ///< Time of file's modification
         uint32_t    mode;       ///< Mode of the file, if applicable
         uint32_t    user;       ///< User ID of owner, if applicable
@@ -553,7 +554,7 @@ namespace sys {
     UnknownFileType = 0,            ///< Unrecognized file
     BytecodeFileType = 1,           ///< Uncompressed bytecode file
     CompressedBytecodeFileType = 2, ///< Compressed bytecode file
-    ArchiveFileType = 3,            ///< ar style archive file
+    ArchiveFileType = 3             ///< ar style archive file
   };
 
   /// This utility function allows any memory block to be examined in order
@@ -567,11 +568,9 @@ namespace sys {
   void CopyFile(const Path& Dest, const Path& Src);
 }
 
-inline std::ostream& operator<<(std::ostream& strm, const sys::Path& aPath) {
-  strm << aPath.toString();
-  return strm;
-}
+std::ostream& operator<<(std::ostream& strm, const sys::Path& aPath);
 
 }
 
+FORCE_DEFINING_FILE_TO_BE_LINKED(SystemPath)
 #endif
