@@ -78,7 +78,7 @@ public:
   virtual ~TargetLowering();
 
   TargetMachine &getTargetMachine() const { return TM; }
-  const TargetData &getTargetData() const { return TD; }
+  const TargetData *getTargetData() const { return TD; }
 
   bool isLittleEndian() const { return IsLittleEndian; }
   MVT::ValueType getPointerTy() const { return PointerTy; }
@@ -172,7 +172,7 @@ public:
   }
   
   /// getPackedTypeBreakdown - Packed types are broken down into some number of
-  /// legal scalar types.  For example, <8 x float> maps to 2 MVT::v2f32 values
+  /// legal first class types.  For example, <8 x float> maps to 2 MVT::v4f32
   /// with Altivec or SSE1, or 8 promoted MVT::f64 values with the X86 FP stack.
   /// Similarly, <2 x long> turns into 4 MVT::i32 values with both PPC and X86.
   ///
@@ -200,6 +200,16 @@ public:
     return true;
   }
 
+  /// isVectorClearMaskLegal - Similar to isShuffleMaskLegal. This is
+  /// used by Targets can use this to indicate if there is a suitable
+  /// VECTOR_SHUFFLE that can be used to replace a VAND with a constant
+  /// pool entry.
+  virtual bool isVectorClearMaskLegal(std::vector<SDOperand> &BVOps,
+                                      MVT::ValueType EVT,
+                                      SelectionDAG &DAG) const {
+    return false;
+  }
+
   /// getOperationAction - Return how this operation should be treated: either
   /// it is legal, needs to be promoted to a larger size, needs to be
   /// expanded to some other code sequence, or the target has a custom expander
@@ -379,6 +389,19 @@ public:
                                               uint64_t &KnownOne,
                                               unsigned Depth = 0) const;
 
+  /// ComputeNumSignBits - Return the number of times the sign bit of the
+  /// register is replicated into the other bits.  We know that at least 1 bit
+  /// is always equal to the sign bit (itself), but other cases can give us
+  /// information.  For example, immediately after an "SRA X, 2", we know that
+  /// the top 3 bits are all equal to each other, so we return 3.
+  unsigned ComputeNumSignBits(SDOperand Op, unsigned Depth = 0) const;
+  
+  /// ComputeNumSignBitsForTargetNode - This method can be implemented by
+  /// targets that want to expose additional information about sign bits to the
+  /// DAG Combiner.
+  virtual unsigned ComputeNumSignBitsForTargetNode(SDOperand Op,
+                                                   unsigned Depth = 0) const;
+  
   struct DAGCombinerInfo {
     void *DC;  // The DAG Combiner object.
     bool BeforeLegalize;
@@ -488,7 +511,7 @@ protected:
                           LegalizeAction Action) {
     assert(VT < 32 && Op < sizeof(OpActions)/sizeof(OpActions[0]) &&
            "Table isn't big enough!");
-    OpActions[Op] &= ~(3ULL << VT*2);
+    OpActions[Op] &= ~(uint64_t(3UL) << VT*2);
     OpActions[Op] |= (uint64_t)Action << VT*2;
   }
   
@@ -534,7 +557,7 @@ public:
   virtual std::pair<SDOperand, SDOperand>
   LowerCallTo(SDOperand Chain, const Type *RetTy, bool isVarArg,
               unsigned CallingConv, bool isTailCall, SDOperand Callee,
-              ArgListTy &Args, SelectionDAG &DAG) = 0;
+              ArgListTy &Args, SelectionDAG &DAG);
 
   /// LowerFrameReturnAddress - This hook lowers a call to llvm.returnaddress or
   /// llvm.frameaddress (depending on the value of the first argument).  The
@@ -626,6 +649,15 @@ public:
     return LegalAddressScales.end();
   }
 
+  //===--------------------------------------------------------------------===//
+  // Div utility functions
+  //
+  SDOperand BuildSDIV(SDNode *N, SelectionDAG &DAG, 
+		      std::vector<SDNode*>* Created) const;
+  SDOperand BuildUDIV(SDNode *N, SelectionDAG &DAG, 
+		      std::vector<SDNode*>* Created) const;
+
+
 protected:
   /// addLegalAddressScale - Add a integer (> 1) value which can be used as
   /// scale in the target addressing mode. Note: the ordering matters so the
@@ -638,7 +670,7 @@ private:
   std::vector<unsigned> LegalAddressScales;
   
   TargetMachine &TM;
-  const TargetData &TD;
+  const TargetData *TD;
 
   /// IsLittleEndian - True if this is a little endian target.
   ///
