@@ -22,7 +22,6 @@
 namespace llvm {
   class Module;
   class TargetMachine;
-  class IntrinsicLowering;
 
   struct TargetMachineRegistry {
     struct Entry;
@@ -49,8 +48,7 @@ namespace llvm {
     struct Entry {
       const char *Name;
       const char *ShortDesc;
-      TargetMachine *(*CtorFn)(const Module &, IntrinsicLowering*,
-                      const std::string &);
+      TargetMachine *(*CtorFn)(const Module &, const std::string &);
       unsigned (*ModuleMatchQualityFn)(const Module &M);
       unsigned (*JITMatchQualityFn)();
 
@@ -58,8 +56,7 @@ namespace llvm {
 
     protected:
       Entry(const char *N, const char *SD,
-            TargetMachine *(*CF)(const Module &, IntrinsicLowering*,
-                                 const std::string &),
+            TargetMachine *(*CF)(const Module &, const std::string &),
             unsigned (*MMF)(const Module &M), unsigned (*JMF)());
     private:
       const Entry *Next;  // Next entry in the linked list.
@@ -73,7 +70,13 @@ namespace llvm {
   /// RegisterTarget - This class is used to make targets automatically register
   /// themselves with the tool they are linked.  Targets should define an
   /// instance of this and implement the static methods described in the
-  /// TargetMachine comments..
+  /// TargetMachine comments.
+  /// The type 'TargetMachineImpl' should provide a constructor with two 
+  /// parameters:
+  /// - const Module& M: the module that is being compiled:
+  /// - const std::string& FS: target-specific string describing target 
+  ///   flavour.
+  
   template<class TargetMachineImpl>
   struct RegisterTarget : public TargetMachineRegistry::Entry {
     RegisterTarget(const char *Name, const char *ShortDesc) :
@@ -82,9 +85,8 @@ namespace llvm {
                                    &TargetMachineImpl::getJITMatchQuality) {
     }
   private:
-    static TargetMachine *Allocator(const Module &M, IntrinsicLowering *IL,
-                                    const std::string &FS) {
-      return new TargetMachineImpl(M, IL, FS);
+    static TargetMachine *Allocator(const Module &M, const std::string &FS) {
+      return new TargetMachineImpl(M, FS);
     }
   };
 
