@@ -117,6 +117,7 @@ ModulePass *createPruneEHPass();
 /// the main function is found, all other globals are marked as internal.
 ///
 ModulePass *createInternalizePass(bool InternalizeEverything);
+ModulePass *createInternalizePass(const std::vector<const char *> &exportList);
 
 //===----------------------------------------------------------------------===//
 /// createDeadArgEliminationPass - This pass removes arguments from functions
@@ -169,6 +170,11 @@ ModulePass *createBlockExtractorPass(std::vector<BasicBlock*> &BTNE);
 // specific well-known (library) functions.
 ModulePass *createSimplifyLibCallsPass();
 
+
+// createIndMemRemPass - This pass removes potential indirect calls of
+// malloc and free
+ModulePass *createIndMemRemPass();
+
 } // End llvm namespace
 
 #endif
