@@ -21,6 +21,7 @@
 #include "llvm/ADT/SCCIterator.h"
 #include "llvm/ADT/Statistic.h"
 #include "llvm/ADT/STLExtras.h"
+#include <iostream>
 using namespace llvm;
 
 namespace {
@@ -55,8 +56,14 @@ bool CompleteBUDataStructures::runOnModule(Module &M) {
   }
 
   for (Module::iterator I = M.begin(), E = M.end(); I != E; ++I)
-    if (!I->isExternal() && !DSInfo.count(I))
+    if (!I->isExternal() && !DSInfo.count(I)) {
+#ifndef NDEBUG
+      if (MainFunc)
+        std::cerr << "*** CBU: Function unreachable from main: "
+                  << I->getName() << "\n";
+#endif
       calculateSCCGraphs(getOrCreateGraph(*I), Stack, NextID, ValMap);
+    }
 
   GlobalsGraph->removeTriviallyDeadNodes();
 
