@@ -549,16 +549,10 @@ void GraphBuilder::visitCallSite(CallSite CS) {
       case Intrinsic::memcpy_i64:
       case Intrinsic::memmove_i32:
       case Intrinsic::memmove_i64: {
-        //This is over aggressive.  What these functions do is not make the 
-        // targets pointers alias, but rather merge the out edges of the graphs
-        // for the pointers according to the type merging of the graphs.
-        //Simply merging the two graphs is a crude approximation to this.
-        //I might be wrong though.
-
         // Merge the first & second arguments, and mark the memory read and
         // modified.
-        DSNodeHandle RetNH = getValueDest(**CS.arg_begin());
-        RetNH.mergeWith(getValueDest(**(CS.arg_begin()+1)));
+	DSNodeHandle RetNH = getValueDest(**CS.arg_begin());
+	RetNH.mergeWith(getValueDest(**(CS.arg_begin()+1)));
         if (DSNode *N = RetNH.getNode())
           N->setModifiedMarker()->setReadMarker();
         return;
@@ -597,8 +591,11 @@ void GraphBuilder::visitCallSite(CallSite CS) {
           }
         }
 
-        if (F->getName() == "calloc" || F->getName() == "posix_memalign" ||
-            F->getName() == "memalign" || F->getName() == "valloc") {
+	//gets select localtime ioctl
+
+        if ((F->isExternal() && F->getName() == "calloc") 
+            || F->getName() == "posix_memalign"
+            || F->getName() == "memalign" || F->getName() == "valloc") {
           setDestTo(*CS.getInstruction(),
                     createNode()->setHeapNodeMarker()->setModifiedMarker());
           return;
@@ -617,7 +614,10 @@ void GraphBuilder::visitCallSite(CallSite CS) {
           if (DSNode *N = RetNH.getNode())
             N->setModifiedMarker()->setReadMarker();
           return;
-
+        } else if (F->getName() == "free") {
+          // Mark that the node is written to...
+          if (DSNode *N = getValueDest(**CS.arg_begin()).getNode())
+            N->setModifiedMarker()->setHeapNodeMarker();
         } else if (F->getName() == "atoi" || F->getName() == "atof" ||
                    F->getName() == "atol" || F->getName() == "atoll" ||
                    F->getName() == "remove" || F->getName() == "unlink" ||
@@ -629,7 +629,8 @@ void GraphBuilder::visitCallSite(CallSite CS) {
                    F->getName() == "puts" || F->getName() == "write" ||
                    F->getName() == "open" || F->getName() == "create" ||
                    F->getName() == "truncate" || F->getName() == "chdir" ||
-                   F->getName() == "mkdir" || F->getName() == "rmdir") {
+                   F->getName() == "mkdir" || F->getName() == "rmdir" ||
+		   F->getName() == "strlen") {
           // These functions read all of their pointer operands.
           for (CallSite::arg_iterator AI = CS.arg_begin(), E = CS.arg_end();
                AI != E; ++AI) {
@@ -638,8 +639,16 @@ void GraphBuilder::visitCallSite(CallSite CS) {
                 N->setReadMarker();
           }
           return;
+	} else if (F->getName() == "memchr") {
+          DSNodeHandle RetNH = getValueDest(**CS.arg_begin());
+          DSNodeHandle Result = getValueDest(*CS.getInstruction());
+          RetNH.mergeWith(Result);
+          if (DSNode *N = RetNH.getNode())
+            N->setReadMarker();
+          return;
         } else if (F->getName() == "read" || F->getName() == "pipe" ||
-                   F->getName() == "wait" || F->getName() == "time") {
+                   F->getName() == "wait" || F->getName() == "time" ||
+		   F->getName() == "getrusage") {
           // These functions write all of their pointer operands.
           for (CallSite::arg_iterator AI = CS.arg_begin(), E = CS.arg_end();
                AI != E; ++AI) {
@@ -997,6 +1006,17 @@ void GraphBuilder::visitCallSite(CallSite CS) {
           if (DSNode *N = getValueDest(**(CS.arg_begin() + 1)).getNode())
             N->setReadMarker();
           return;
+        } else if (F->getName() == "strcpy" || F->getName() == "strncpy") {
+          //This might be making unsafe assumptions about usage
+          //Merge return and first arg
+          DSNodeHandle RetNH = getValueDest(*CS.getInstruction());
+          RetNH.mergeWith(getValueDest(**CS.arg_begin()));
+          if (DSNode *N = RetNH.getNode())
+            N->setHeapNodeMarker()->setModifiedMarker();
+          //and read second pointer
+          if (DSNode *N = getValueDest(**(CS.arg_begin() + 1)).getNode())
+            N->setReadMarker();
+          return;
         } else {
           // Unknown function, warn if it returns a pointer type or takes a
           // pointer argument.
@@ -1117,13 +1137,12 @@ void GraphBuilder::MergeConstantInitIntoNode(DSNodeHandle &NH, Constant *C) {
     for (unsigned i = 0, e = CS->getNumOperands(); i != e; ++i) {
       DSNode *NHN = NH.getNode();
       //Some programmers think ending a structure with a [0 x sbyte] is cute
-      //This should be ok as the allocation type should grow this type when
-      //it is merged in if it is bigger.
       if (SL->MemberOffsets[i] < SL->StructSize) {
         DSNodeHandle NewNH(NHN, NH.getOffset()+(unsigned)SL->MemberOffsets[i]);
         MergeConstantInitIntoNode(NewNH, cast<Constant>(CS->getOperand(i)));
       } else if (SL->MemberOffsets[i] == SL->StructSize) {
         DEBUG(std::cerr << "Zero size element at end of struct\n");
+        NHN->foldNodeCompletely();
       } else {
         assert(0 && "type was smaller than offsets of of struct layout indicate");
       }
