@@ -20,6 +20,7 @@
 #include "llvm/Support/CommandLine.h"
 #include "llvm/Support/GraphWriter.h"
 #include "llvm/ADT/Statistic.h"
+#include "llvm/Config/config.h"
 #include <fstream>
 #include <sstream>
 using namespace llvm;
@@ -256,17 +257,7 @@ void DSGraph::writeGraphToFile(std::ostream &O,
 /// then cleanup.  For use from the debugger.
 ///
 void DSGraph::viewGraph() const {
-  std::ofstream F("/tmp/tempgraph.dot");
-  if (!F.good()) {
-    std::cerr << "Error opening '/tmp/tempgraph.dot' for temporary graph!\n";
-    return;
-  }
-  print(F);
-  F.close();
-  if (system("dot -Tps -Gsize=10,7.5 -Grotate=90 /tmp/tempgraph.dot > /tmp/tempgraph.ps"))
-    std::cerr << "Error running dot: 'dot' not in path?\n";
-  system("gv /tmp/tempgraph.ps");
-  system("rm /tmp/tempgraph.dot /tmp/tempgraph.ps");
+  ViewGraph(this, "ds.tempgraph", "DataStructures");
 }
 
 
