@@ -86,11 +86,9 @@ void Loop::dump() const {
 //===----------------------------------------------------------------------===//
 // LoopInfo implementation
 //
-void LoopInfo::stub() {}
-
 bool LoopInfo::runOnFunction(Function &) {
   releaseMemory();
-  Calculate(getAnalysis<DominatorSet>());    // Update
+  Calculate(getAnalysis<ETForest>());    // Update
   return false;
 }
 
@@ -104,18 +102,18 @@ void LoopInfo::releaseMemory() {
 }
 
 
-void LoopInfo::Calculate(const DominatorSet &DS) {
-  BasicBlock *RootNode = DS.getRoot();
+void LoopInfo::Calculate(ETForest &EF) {
+  BasicBlock *RootNode = EF.getRoot();
 
   for (df_iterator<BasicBlock*> NI = df_begin(RootNode),
          NE = df_end(RootNode); NI != NE; ++NI)
-    if (Loop *L = ConsiderForLoop(*NI, DS))
+    if (Loop *L = ConsiderForLoop(*NI, EF))
       TopLevelLoops.push_back(L);
 }
 
 void LoopInfo::getAnalysisUsage(AnalysisUsage &AU) const {
   AU.setPreservesAll();
-  AU.addRequired<DominatorSet>();
+  AU.addRequired<ETForest>();
 }
 
 void LoopInfo::print(std::ostream &OS, const Module* ) const {
@@ -135,7 +133,7 @@ static bool isNotAlreadyContainedIn(Loop *SubLoop, Loop *ParentLoop) {
   return isNotAlreadyContainedIn(SubLoop->getParentLoop(), ParentLoop);
 }
 
-Loop *LoopInfo::ConsiderForLoop(BasicBlock *BB, const DominatorSet &DS) {
+Loop *LoopInfo::ConsiderForLoop(BasicBlock *BB, ETForest &EF) {
   if (BBMap.find(BB) != BBMap.end()) return 0;   // Haven't processed this node?
 
   std::vector<BasicBlock *> TodoStack;
@@ -143,7 +141,7 @@ Loop *LoopInfo::ConsiderForLoop(BasicBlock *BB, const DominatorSet &DS) {
   // Scan the predecessors of BB, checking to see if BB dominates any of
   // them.  This identifies backedges which target this node...
   for (pred_iterator I = pred_begin(BB), E = pred_end(BB); I != E; ++I)
-    if (DS.dominates(BB, *I))   // If BB dominates it's predecessor...
+    if (EF.dominates(BB, *I))   // If BB dominates it's predecessor...
       TodoStack.push_back(*I);
 
   if (TodoStack.empty()) return 0;  // No backedges to this block...
@@ -159,7 +157,7 @@ Loop *LoopInfo::ConsiderForLoop(BasicBlock *BB, const DominatorSet &DS) {
     TodoStack.pop_back();
 
     if (!L->contains(X) &&         // As of yet unprocessed??
-        DS.dominates(EntryBlock, X)) {   // X is reachable from entry block?
+        EF.dominates(EntryBlock, X)) {   // X is reachable from entry block?
       // Check to see if this block already belongs to a loop.  If this occurs
       // then we have a case where a loop that is supposed to be a child of the
       // current loop was processed before the current loop.  When this occurs,
@@ -191,7 +189,7 @@ Loop *LoopInfo::ConsiderForLoop(BasicBlock *BB, const DominatorSet &DS) {
   // If there are any loops nested within this loop, create them now!
   for (std::vector<BasicBlock*>::iterator I = L->Blocks.begin(),
          E = L->Blocks.end(); I != E; ++I)
-    if (Loop *NewLoop = ConsiderForLoop(*I, DS)) {
+    if (Loop *NewLoop = ConsiderForLoop(*I, EF)) {
       L->SubLoops.push_back(NewLoop);
       NewLoop->ParentLoop = L;
     }
@@ -367,7 +365,8 @@ BasicBlock *Loop::getLoopPreheader() const {
       Out = *PI;
     }
 
-  // Make sure there is only one exit out of the preheader...
+  // Make sure there is only one exit out of the preheader.
+  assert(Out && "Header of loop has no predecessors from outside loop?");
   succ_iterator SI = succ_begin(Out);
   ++SI;
   if (SI != succ_end(Out))
@@ -480,6 +479,27 @@ Value *Loop::getTripCount() const {
   return 0;
 }
 
+/// isLCSSAForm - Return true if the Loop is in LCSSA form
+bool Loop::isLCSSAForm() const {  
+  for (Loop::block_iterator BB = block_begin(), E = block_end();
+       BB != E; ++BB) {
+    for (BasicBlock::iterator I = (*BB)->begin(), E = (*BB)->end(); I != E; ++I)
+      for (Value::use_iterator UI = I->use_begin(), E = I->use_end(); UI != E;
+           ++UI) {
+        BasicBlock *UserBB = cast<Instruction>(*UI)->getParent();
+        if (PHINode* p = dyn_cast<PHINode>(*UI)) {
+          unsigned OperandNo = UI.getOperandNo();
+          UserBB = p->getIncomingBlock(OperandNo/2);
+        }
+        
+        if (!contains(UserBB)) {
+          return false;
+        }
+      }
+  }
+  
+  return true;
+}
 
 //===-------------------------------------------------------------------===//
 // APIs for updating loop information after changing the CFG
@@ -556,3 +576,6 @@ Loop *Loop::removeChildLoop(iterator I) {
 void Loop::removeBlockFromLoop(BasicBlock *BB) {
   RemoveFromVector(Blocks, BB);
 }
+
+// Ensure this file gets linked when LoopInfo.h is used.
+DEFINING_FILE_FOR(LoopInfo)
