@@ -34,31 +34,32 @@
 #include "llvm/ADT/STLExtras.h"
 #include <algorithm>
 #include <cmath>
+#include <iostream>
 using namespace llvm;
 
 namespace {
   RegisterAnalysis<LiveIntervals> X("liveintervals", "Live Interval Analysis");
 
-  Statistic<> numIntervals
+  static Statistic<> numIntervals
   ("liveintervals", "Number of original intervals");
 
-  Statistic<> numIntervalsAfter
+  static Statistic<> numIntervalsAfter
   ("liveintervals", "Number of intervals after coalescing");
 
-  Statistic<> numJoins
+  static Statistic<> numJoins
   ("liveintervals", "Number of interval joins performed");
 
-  Statistic<> numPeep
+  static Statistic<> numPeep
   ("liveintervals", "Number of identity moves eliminated after coalescing");
 
-  Statistic<> numFolded
+  static Statistic<> numFolded
   ("liveintervals", "Number of loads/stores folded into instructions");
 
-  cl::opt<bool>
+  static cl::opt<bool>
   EnableJoining("join-liveintervals",
                 cl::desc("Join compatible live intervals"),
                 cl::init(true));
-};
+}
 
 void LiveIntervals::getAnalysisUsage(AnalysisUsage &AU) const
 {
@@ -79,6 +80,15 @@ void LiveIntervals::releaseMemory()
 }
 
 
+static bool isZeroLengthInterval(LiveInterval *li) {
+  for (LiveInterval::Ranges::const_iterator
+         i = li->ranges.begin(), e = li->ranges.end(); i != e; ++i)
+    if (i->end - i->start > LiveIntervals::InstrSlots::NUM)
+      return false;
+  return true;
+}
+
+
 /// runOnMachineFunction - Register allocate the whole function
 ///
 bool LiveIntervals::runOnMachineFunction(MachineFunction &fn) {
@@ -186,7 +196,7 @@ bool LiveIntervals::runOnMachineFunction(MachineFunction &fn) {
               MRegisterInfo::isVirtualRegister(mop.getReg())) {
             // replace register with representative register
             unsigned reg = rep(mop.getReg());
-            mii->SetMachineOperandReg(i, reg);
+            mii->getOperand(i).setReg(reg);
 
             LiveInterval &RegInt = getInterval(reg);
             RegInt.weight +=
@@ -198,6 +208,16 @@ bool LiveIntervals::runOnMachineFunction(MachineFunction &fn) {
     }
   }
 
+  for (iterator I = begin(), E = end(); I != E; ++I) {
+    LiveInterval &li = I->second;
+    if (MRegisterInfo::isVirtualRegister(li.reg))
+      // If the live interval legnth is essentially zero, i.e. in every live
+      // range the use follows def immediately, it doesn't make sense to spill
+      // it and hope it will be easier to allocate for this li.
+      if (isZeroLengthInterval(&li))
+        li.weight = float(HUGE_VAL);
+  }
+
   DEBUG(dump());
   return true;
 }
@@ -247,7 +267,7 @@ addIntervalsForSpills(const LiveInterval &li, VirtRegMap &vrm, int slot) {
         index += InstrSlots::NUM;
       if (index == end) break;
 
-      MachineBasicBlock::iterator mi = getInstructionFromIndex(index);
+      MachineInstr *MI = getInstructionFromIndex(index);
 
       // NewRegLiveIn - This instruction might have multiple uses of the spilled
       // register.  In this case, for the first use, keep track of the new vreg
@@ -256,28 +276,27 @@ addIntervalsForSpills(const LiveInterval &li, VirtRegMap &vrm, int slot) {
       unsigned NewRegLiveIn = 0;
 
     for_operand:
-      for (unsigned i = 0; i != mi->getNumOperands(); ++i) {
-        MachineOperand& mop = mi->getOperand(i);
+      for (unsigned i = 0; i != MI->getNumOperands(); ++i) {
+        MachineOperand& mop = MI->getOperand(i);
         if (mop.isRegister() && mop.getReg() == li.reg) {
           if (NewRegLiveIn && mop.isUse()) {
             // We already emitted a reload of this value, reuse it for
             // subsequent operands.
-            mi->SetMachineOperandReg(i, NewRegLiveIn);
+            MI->getOperand(i).setReg(NewRegLiveIn);
             DEBUG(std::cerr << "\t\t\t\treused reload into reg" << NewRegLiveIn
                             << " for operand #" << i << '\n');
-          } else if (MachineInstr* fmi = mri_->foldMemoryOperand(mi, i, slot)) {
+          } else if (MachineInstr* fmi = mri_->foldMemoryOperand(MI, i, slot)) {
             // Attempt to fold the memory reference into the instruction.  If we
             // can do this, we don't need to insert spill code.
             if (lv_)
-              lv_->instructionChanged(mi, fmi);
-            vrm.virtFolded(li.reg, mi, i, fmi);
-            mi2iMap_.erase(mi);
+              lv_->instructionChanged(MI, fmi);
+            MachineBasicBlock &MBB = *MI->getParent();
+            vrm.virtFolded(li.reg, MI, i, fmi);
+            mi2iMap_.erase(MI);
             i2miMap_[index/InstrSlots::NUM] = fmi;
             mi2iMap_[fmi] = index;
-            MachineBasicBlock &MBB = *mi->getParent();
-            mi = MBB.insert(MBB.erase(mi), fmi);
+            MI = MBB.insert(MBB.erase(MI), fmi);
             ++numFolded;
-
             // Folding the load/store can completely change the instruction in
             // unpredictable ways, rescan it from the beginning.
             goto for_operand;
@@ -300,7 +319,7 @@ addIntervalsForSpills(const LiveInterval &li, VirtRegMap &vrm, int slot) {
 
             // create a new register for this spill
             NewRegLiveIn = mf_->getSSARegMap()->createVirtualRegister(rc);
-            mi->SetMachineOperandReg(i, NewRegLiveIn);
+            MI->getOperand(i).setReg(NewRegLiveIn);
             vrm.grow();
             vrm.assignVirt2StackSlot(NewRegLiveIn, slot);
             LiveInterval& nI = getOrCreateInterval(NewRegLiveIn);
@@ -316,7 +335,7 @@ addIntervalsForSpills(const LiveInterval &li, VirtRegMap &vrm, int slot) {
 
             // update live variables if it is available
             if (lv_)
-              lv_->addVirtualRegisterKilled(NewRegLiveIn, mi);
+              lv_->addVirtualRegisterKilled(NewRegLiveIn, MI);
             
             // If this is a live in, reuse it for subsequent live-ins.  If it's
             // a def, we can't do this.
@@ -585,7 +604,8 @@ void LiveIntervals::handleRegisterDef(MachineBasicBlock *MBB,
     handleVirtualRegisterDef(MBB, MI, getOrCreateInterval(reg));
   else if (allocatableRegs_[reg]) {
     unsigned SrcReg = 0, DestReg = 0;
-    bool IsMove = tii_->isMoveInstr(*MI, SrcReg, DestReg);
+    if (!tii_->isMoveInstr(*MI, SrcReg, DestReg))
+      SrcReg = DestReg = 0;
 
     handlePhysicalRegisterDef(MBB, MI, getOrCreateInterval(reg),
                               SrcReg, DestReg);
@@ -619,8 +639,10 @@ void LiveIntervals::computeIntervals()
       DEBUG(std::cerr << getInstructionIndex(mi) << "\t" << *mi);
 
       // handle implicit defs
-      for (const unsigned* id = tid.ImplicitDefs; *id; ++id)
-        handleRegisterDef(mbb, mi, *id);
+      if (tid.ImplicitDefs) {
+        for (const unsigned* id = tid.ImplicitDefs; *id; ++id)
+          handleRegisterDef(mbb, mi, *id);
+      }
 
       // handle explicit defs
       for (int i = mi->getNumOperands() - 1; i >= 0; --i) {
