@@ -223,16 +223,21 @@ public:
     Field = getGlobalVariable(C);
   }
   virtual void Apply(std::vector<DebugInfoDesc *> &Field) {
+    Field.resize(0);
     Constant *C = CI->getOperand(I++);
     GlobalVariable *GV = getGlobalVariable(C);
-    Field.resize(0);
-    // Have to be able to deal with the empty array case (zero initializer)
-    if (!GV->hasInitializer()) return;
-    if (ConstantArray *CA = dyn_cast<ConstantArray>(GV->getInitializer())) {
-      for (unsigned i = 0, N = CA->getNumOperands(); i < N; ++i) {
-        GlobalVariable *GVE = getGlobalVariable(CA->getOperand(i));
-        DebugInfoDesc *DE = DR.Deserialize(GVE);
-        Field.push_back(DE);
+    if (GV->hasInitializer()) {
+      if (ConstantArray *CA = dyn_cast<ConstantArray>(GV->getInitializer())) {
+        for (unsigned i = 0, N = CA->getNumOperands(); i < N; ++i) {
+          GlobalVariable *GVE = getGlobalVariable(CA->getOperand(i));
+          DebugInfoDesc *DE = DR.Deserialize(GVE);
+          Field.push_back(DE);
+        }
+      } else if (GV->getInitializer()->isNullValue()) {
+        if (const ArrayType *T =
+            dyn_cast<ArrayType>(GV->getType()->getElementType())) {
+          Field.resize(T->getNumElements());
+        }
       }
     }
   }
@@ -262,10 +267,10 @@ public:
     Elements.push_back(ConstantUInt::get(Type::UIntTy, Field));
   }
   virtual void Apply(int64_t &Field) {
-    Elements.push_back(ConstantSInt::get(Type::IntTy, Field));
+    Elements.push_back(ConstantSInt::get(Type::LongTy, Field));
   }
   virtual void Apply(uint64_t &Field) {
-    Elements.push_back(ConstantUInt::get(Type::UIntTy, Field));
+    Elements.push_back(ConstantUInt::get(Type::ULongTy, Field));
   }
   virtual void Apply(bool &Field) {
     Elements.push_back(ConstantBool::get(Field));
@@ -305,9 +310,13 @@ public:
     std::vector<Constant *> ArrayElements;
 
     for (unsigned i = 0, N = Field.size(); i < N; ++i) {
-      GlobalVariable *GVE = SR.Serialize(Field[i]);
-      Constant *CE = ConstantExpr::getCast(GVE, EmptyTy);
-      ArrayElements.push_back(cast<Constant>(CE));
+      if (DebugInfoDesc *Element = Field[i]) {
+        GlobalVariable *GVE = SR.Serialize(Element);
+        Constant *CE = ConstantExpr::getCast(GVE, EmptyTy);
+        ArrayElements.push_back(cast<Constant>(CE));
+      } else {
+        ArrayElements.push_back(ConstantPointerNull::get(EmptyTy));
+      }
     }
     
     Constant *CA = ConstantArray::get(AT, ArrayElements);
@@ -345,10 +354,10 @@ public:
     Fields.push_back(Type::UIntTy);
   }
   virtual void Apply(int64_t &Field) {
-    Fields.push_back(Type::IntTy);
+    Fields.push_back(Type::LongTy);
   }
   virtual void Apply(uint64_t &Field) {
-    Fields.push_back(Type::UIntTy);
+    Fields.push_back(Type::ULongTy);
   }
   virtual void Apply(bool &Field) {
     Fields.push_back(Type::BoolTy);
@@ -455,11 +464,21 @@ public:
 
 //===----------------------------------------------------------------------===//
 
-/// TagFromGlobal - Returns the Tag number from a debug info descriptor
-/// GlobalVariable.  
+/// TagFromGlobal - Returns the tag number from a debug info descriptor
+/// GlobalVariable.   Return DIIValid if operand is not an unsigned int. 
 unsigned DebugInfoDesc::TagFromGlobal(GlobalVariable *GV) {
   ConstantUInt *C = getUIntOperand(GV, 0);
-  return C ? (unsigned)C->getValue() : (unsigned)DW_TAG_invalid;
+  return C ? ((unsigned)C->getValue() & ~LLVMDebugVersionMask) :
+             (unsigned)DW_TAG_invalid;
+}
+
+/// VersionFromGlobal - Returns the version number from a debug info
+/// descriptor GlobalVariable.  Return DIIValid if operand is not an unsigned
+/// int.
+unsigned  DebugInfoDesc::VersionFromGlobal(GlobalVariable *GV) {
+  ConstantUInt *C = getUIntOperand(GV, 0);
+  return C ? ((unsigned)C->getValue() & LLVMDebugVersionMask) :
+             (unsigned)DW_TAG_invalid;
 }
 
 /// DescFactory - Create an instance of debug info descriptor based on Tag.
@@ -482,7 +501,9 @@ DebugInfoDesc *DebugInfoDesc::DescFactory(unsigned Tag) {
   case DW_TAG_array_type:
   case DW_TAG_structure_type:
   case DW_TAG_union_type:
-  case DW_TAG_enumeration_type: return new CompositeTypeDesc(Tag);
+  case DW_TAG_enumeration_type:
+  case DW_TAG_vector_type:
+  case DW_TAG_subroutine_type:  return new CompositeTypeDesc(Tag);
   case DW_TAG_subrange_type:    return new SubrangeDesc();
   case DW_TAG_enumerator:       return new EnumeratorDesc();
   case DW_TAG_return_variable:
@@ -563,6 +584,7 @@ const char *AnchorDesc::getTypeString() const {
 #ifndef NDEBUG
 void AnchorDesc::dump() {
   std::cerr << getDescString() << " "
+            << "Version(" << getVersion() << "), "
             << "Tag(" << getTag() << "), "
             << "AnchorTag(" << AnchorTag << ")\n";
 }
@@ -580,16 +602,13 @@ AnchoredDesc::AnchoredDesc(unsigned T)
 void AnchoredDesc::ApplyToFields(DIVisitor *Visitor) {
   DebugInfoDesc::ApplyToFields(Visitor);
 
-  DebugInfoDesc *Tmp = Anchor;
-  Visitor->Apply(Tmp);
-  Anchor = (AnchorDesc*)Tmp;
+  Visitor->Apply(Anchor);
 }
 
 //===----------------------------------------------------------------------===//
 
 CompileUnitDesc::CompileUnitDesc()
 : AnchoredDesc(DW_TAG_compile_unit)
-, DebugVersion(LLVMDebugVersion)
 , Language(0)
 , FileName("")
 , Directory("")
@@ -601,19 +620,17 @@ bool CompileUnitDesc::classof(const DebugInfoDesc *D) {
   return D->getTag() == DW_TAG_compile_unit;
 }
 
-/// DebugVersionFromGlobal - Returns the version number from a compile unit
-/// GlobalVariable.
-unsigned CompileUnitDesc::DebugVersionFromGlobal(GlobalVariable *GV) {
-  ConstantUInt *C = getUIntOperand(GV, 2);
-  return C ? (unsigned)C->getValue() : (unsigned)DW_TAG_invalid;
-}
-  
 /// ApplyToFields - Target the visitor to the fields of the CompileUnitDesc.
 ///
 void CompileUnitDesc::ApplyToFields(DIVisitor *Visitor) {
   AnchoredDesc::ApplyToFields(Visitor);
+  
+  // Handle cases out of sync with compiler.
+  if (getVersion() == 0) {
+    unsigned DebugVersion;
+    Visitor->Apply(DebugVersion);
+  }
 
-  Visitor->Apply(DebugVersion);
   Visitor->Apply(Language);
   Visitor->Apply(FileName);
   Visitor->Apply(Directory);
@@ -642,9 +659,9 @@ const char *CompileUnitDesc::getAnchorString() const {
 #ifndef NDEBUG
 void CompileUnitDesc::dump() {
   std::cerr << getDescString() << " "
+            << "Version(" << getVersion() << "), "
             << "Tag(" << getTag() << "), "
             << "Anchor(" << getAnchor() << "), "
-            << "DebugVersion(" << DebugVersion << "), "
             << "Language(" << Language << "), "
             << "FileName(\"" << FileName << "\"), "
             << "Directory(\"" << Directory << "\"), "
@@ -663,6 +680,7 @@ TypeDesc::TypeDesc(unsigned T)
 , Size(0)
 , Align(0)
 , Offset(0)
+, Flags(0)
 {}
 
 /// ApplyToFields - Target the visitor to the fields of the TypeDesc.
@@ -672,13 +690,12 @@ void TypeDesc::ApplyToFields(DIVisitor *Visitor) {
   
   Visitor->Apply(Context);
   Visitor->Apply(Name);
-  DebugInfoDesc* Tmp = File;
-  Visitor->Apply(Tmp);
-  File = (CompileUnitDesc*)Tmp;
+  Visitor->Apply(File);
   Visitor->Apply(Line);
   Visitor->Apply(Size);
   Visitor->Apply(Align);
   Visitor->Apply(Offset);
+  if (getVersion() > LLVMDebugVersion4) Visitor->Apply(Flags);
 }
 
 /// getDescString - Return a string used to compose global names and labels.
@@ -696,6 +713,7 @@ const char *TypeDesc::getTypeString() const {
 #ifndef NDEBUG
 void TypeDesc::dump() {
   std::cerr << getDescString() << " "
+            << "Version(" << getVersion() << "), "
             << "Tag(" << getTag() << "), "
             << "Context(" << Context << "), "
             << "Name(\"" << Name << "\"), "
@@ -703,7 +721,8 @@ void TypeDesc::dump() {
             << "Line(" << Line << "), "
             << "Size(" << Size << "), "
             << "Align(" << Align << "), "
-            << "Offset(" << Offset << ")\n";
+            << "Offset(" << Offset << "), "
+            << "Flags(" << Flags << ")\n";
 }
 #endif
 
@@ -742,6 +761,7 @@ const char *BasicTypeDesc::getTypeString() const {
 #ifndef NDEBUG
 void BasicTypeDesc::dump() {
   std::cerr << getDescString() << " "
+            << "Version(" << getVersion() << "), "
             << "Tag(" << getTag() << "), "
             << "Context(" << getContext() << "), "
             << "Name(\"" << getName() << "\"), "
@@ -779,9 +799,7 @@ bool DerivedTypeDesc::classof(const DebugInfoDesc *D) {
 void DerivedTypeDesc::ApplyToFields(DIVisitor *Visitor) {
   TypeDesc::ApplyToFields(Visitor);
   
-  DebugInfoDesc* Tmp = FromType;
-  Visitor->Apply(Tmp);
-  FromType = (TypeDesc*)Tmp;
+  Visitor->Apply(FromType);
 }
 
 /// getDescString - Return a string used to compose global names and labels.
@@ -799,6 +817,7 @@ const char *DerivedTypeDesc::getTypeString() const {
 #ifndef NDEBUG
 void DerivedTypeDesc::dump() {
   std::cerr << getDescString() << " "
+            << "Version(" << getVersion() << "), "
             << "Tag(" << getTag() << "), "
             << "Context(" << getContext() << "), "
             << "Name(\"" << getName() << "\"), "
@@ -824,6 +843,8 @@ bool CompositeTypeDesc::classof(const DebugInfoDesc *D) {
   case DW_TAG_structure_type:
   case DW_TAG_union_type:
   case DW_TAG_enumeration_type:
+  case DW_TAG_vector_type:
+  case DW_TAG_subroutine_type:
     return true;
   default: break;
   }
@@ -833,8 +854,8 @@ bool CompositeTypeDesc::classof(const DebugInfoDesc *D) {
 /// ApplyToFields - Target the visitor to the fields of the CompositeTypeDesc.
 ///
 void CompositeTypeDesc::ApplyToFields(DIVisitor *Visitor) {
-  DerivedTypeDesc::ApplyToFields(Visitor);
-  
+  DerivedTypeDesc::ApplyToFields(Visitor);  
+
   Visitor->Apply(Elements);
 }
 
@@ -853,6 +874,7 @@ const char *CompositeTypeDesc::getTypeString() const {
 #ifndef NDEBUG
 void CompositeTypeDesc::dump() {
   std::cerr << getDescString() << " "
+            << "Version(" << getVersion() << "), "
             << "Tag(" << getTag() << "), "
             << "Context(" << getContext() << "), "
             << "Name(\"" << getName() << "\"), "
@@ -901,6 +923,7 @@ const char *SubrangeDesc::getTypeString() const {
 #ifndef NDEBUG
 void SubrangeDesc::dump() {
   std::cerr << getDescString() << " "
+            << "Version(" << getVersion() << "), "
             << "Tag(" << getTag() << "), "
             << "Lo(" << Lo << "), "
             << "Hi(" << Hi << ")\n";
@@ -944,6 +967,7 @@ const char *EnumeratorDesc::getTypeString() const {
 #ifndef NDEBUG
 void EnumeratorDesc::dump() {
   std::cerr << getDescString() << " "
+            << "Version(" << getVersion() << "), "
             << "Tag(" << getTag() << "), "
             << "Name(" << Name << "), "
             << "Value(" << Value << ")\n";
@@ -981,13 +1005,9 @@ void VariableDesc::ApplyToFields(DIVisitor *Visitor) {
   
   Visitor->Apply(Context);
   Visitor->Apply(Name);
-  DebugInfoDesc* Tmp1 = File;
-  Visitor->Apply(Tmp1);
-  File = (CompileUnitDesc*)Tmp1;
+  Visitor->Apply(File);
   Visitor->Apply(Line);
-  DebugInfoDesc* Tmp2 = TyDesc;
-  Visitor->Apply(Tmp2);
-  TyDesc = (TypeDesc*)Tmp2;
+  Visitor->Apply(TyDesc);
 }
 
 /// getDescString - Return a string used to compose global names and labels.
@@ -1005,6 +1025,7 @@ const char *VariableDesc::getTypeString() const {
 #ifndef NDEBUG
 void VariableDesc::dump() {
   std::cerr << getDescString() << " "
+            << "Version(" << getVersion() << "), "
             << "Tag(" << getTag() << "), "
             << "Context(" << Context << "), "
             << "Name(\"" << Name << "\"), "
@@ -1020,6 +1041,7 @@ GlobalDesc::GlobalDesc(unsigned T)
 : AnchoredDesc(T)
 , Context(0)
 , Name("")
+, DisplayName("")
 , File(NULL)
 , Line(0)
 , TyDesc(NULL)
@@ -1034,13 +1056,10 @@ void GlobalDesc::ApplyToFields(DIVisitor *Visitor) {
 
   Visitor->Apply(Context);
   Visitor->Apply(Name);
-  DebugInfoDesc* Tmp1 = File;
-  Visitor->Apply(Tmp1);
-  File = (CompileUnitDesc*)Tmp1;
+  if (getVersion() > LLVMDebugVersion4) Visitor->Apply(DisplayName);
+  Visitor->Apply(File);
   Visitor->Apply(Line);
-  DebugInfoDesc* Tmp2 = TyDesc;
-  Visitor->Apply(Tmp2);
-  TyDesc = (TypeDesc*)Tmp2;
+  Visitor->Apply(TyDesc);
   Visitor->Apply(IsStatic);
   Visitor->Apply(IsDefinition);
 }
@@ -1087,9 +1106,11 @@ const char *GlobalVariableDesc::getAnchorString() const {
 #ifndef NDEBUG
 void GlobalVariableDesc::dump() {
   std::cerr << getDescString() << " "
+            << "Version(" << getVersion() << "), "
             << "Tag(" << getTag() << "), "
             << "Anchor(" << getAnchor() << "), "
             << "Name(\"" << getName() << "\"), "
+            << "DisplayName(\"" << getDisplayName() << "\"), "
             << "File(" << getFile() << "),"
             << "Line(" << getLine() << "),"
             << "Type(\"" << getType() << "\"), "
@@ -1138,9 +1159,11 @@ const char *SubprogramDesc::getAnchorString() const {
 #ifndef NDEBUG
 void SubprogramDesc::dump() {
   std::cerr << getDescString() << " "
+            << "Version(" << getVersion() << "), "
             << "Tag(" << getTag() << "), "
             << "Anchor(" << getAnchor() << "), "
             << "Name(\"" << getName() << "\"), "
+            << "DisplayName(\"" << getDisplayName() << "\"), "
             << "File(" << getFile() << "),"
             << "Line(" << getLine() << "),"
             << "Type(\"" << getType() << "\"), "
@@ -1184,6 +1207,7 @@ const char *BlockDesc::getTypeString() const {
 #ifndef NDEBUG
 void BlockDesc::dump() {
   std::cerr << getDescString() << " "
+            << "Version(" << getVersion() << "), "
             << "Tag(" << getTag() << "),"
             << "Context(" << Context << ")\n";
 }
@@ -1205,11 +1229,6 @@ DebugInfoDesc *DIDeserializer::Deserialize(GlobalVariable *GV) {
   // Get the Tag from the global.
   unsigned Tag = DebugInfoDesc::TagFromGlobal(GV);
   
-  // Get the debug version if a compile unit.
-  if (Tag == DW_TAG_compile_unit) {
-    DebugVersion = CompileUnitDesc::DebugVersionFromGlobal(GV);
-  }
-  
   // Create an empty instance of the correct sort.
   Slot = DebugInfoDesc::DescFactory(Tag);
   
@@ -1366,16 +1385,6 @@ bool DIVerifier::Verify(GlobalVariable *GV) {
   // Check for user defined descriptors.
   if (Tag == DW_TAG_invalid) return true;
 
-  // If a compile unit we need the debug version.
-  if (Tag == DW_TAG_compile_unit) {
-    DebugVersion = CompileUnitDesc::DebugVersionFromGlobal(GV);
-    // FIXME - In the short term, changes are too drastic to continue.
-    if (DebugVersion != LLVMDebugVersion) {
-      ValiditySlot = Invalid;
-      return false;
-    }
-  }
-  
   // Construct an empty DebugInfoDesc.
   DebugInfoDesc *DD = DebugInfoDesc::DescFactory(Tag);
   
