@@ -18,13 +18,16 @@
 #include "llvm/CodeGen/SSARegMap.h"
 #include "llvm/CodeGen/MachineFrameInfo.h"
 #include "llvm/CodeGen/MachineConstantPool.h"
+#include "llvm/CodeGen/MachineJumpTableInfo.h"
 #include "llvm/CodeGen/Passes.h"
+#include "llvm/Target/TargetData.h"
 #include "llvm/Target/TargetMachine.h"
 #include "llvm/Target/TargetFrameInfo.h"
 #include "llvm/Function.h"
 #include "llvm/Instructions.h"
 #include "llvm/Support/LeakDetector.h"
 #include "llvm/Support/GraphWriter.h"
+#include "llvm/Support/Visibility.h"
 #include "llvm/Config/config.h"
 #include <fstream>
 #include <iostream>
@@ -35,9 +38,11 @@ using namespace llvm;
 static AnnotationID MF_AID(
   AnnotationManager::getID("CodeGen::MachineCodeForFunction"));
 
+// Out of line virtual function to home classes.
+void MachineFunctionPass::virtfn() {}
 
 namespace {
-  struct Printer : public MachineFunctionPass {
+  struct VISIBILITY_HIDDEN Printer : public MachineFunctionPass {
     std::ostream *OS;
     const std::string Banner;
 
@@ -67,7 +72,7 @@ FunctionPass *llvm::createMachineFunctionPrinterPass(std::ostream *OS,
 }
 
 namespace {
-  struct Deleter : public MachineFunctionPass {
+  struct VISIBILITY_HIDDEN Deleter : public MachineFunctionPass {
     const char *getPassName() const { return "Machine Code Deleter"; }
 
     bool runOnMachineFunction(MachineFunction &MF) {
@@ -113,6 +118,7 @@ MachineFunction::MachineFunction(const Function *F,
   MFInfo = 0;
   FrameInfo = new MachineFrameInfo();
   ConstantPool = new MachineConstantPool(TM.getTargetData());
+  JumpTableInfo = new MachineJumpTableInfo(TM.getTargetData());
   BasicBlocks.Parent = this;
 }
 
@@ -122,6 +128,7 @@ MachineFunction::~MachineFunction() {
   delete MFInfo;
   delete FrameInfo;
   delete ConstantPool;
+  delete JumpTableInfo;
   delete[] UsedPhysRegs;
 }
 
@@ -132,6 +139,9 @@ void MachineFunction::print(std::ostream &OS) const {
 
   // Print Frame Information
   getFrameInfo()->print(*this, OS);
+  
+  // Print JumpTable Information
+  getJumpTableInfo()->print(OS);
 
   // Print Constant Pool
   getConstantPool()->print(OS);
@@ -145,6 +155,9 @@ void MachineFunction::print(std::ostream &OS) const {
         OS << " " << MRI->getName(I->first);
       else
         OS << " Reg #" << I->first;
+      
+      if (I->second)
+        OS << " in VR#" << I->second << " ";
     }
     OS << "\n";
   }
@@ -208,48 +221,11 @@ namespace llvm {
 void MachineFunction::viewCFG() const
 {
 #ifndef NDEBUG
-  std::string Filename = "/tmp/cfg." + getFunction()->getName() + ".dot";
-  std::cerr << "Writing '" << Filename << "'... ";
-  std::ofstream F(Filename.c_str());
-
-  if (!F) {
-    std::cerr << "  error opening file for writing!\n";
-    return;
-  }
-
-  WriteGraph(F, this);
-  F.close();
-  std::cerr << "\n";
-
-#ifdef HAVE_GRAPHVIZ
-  std::cerr << "Running 'Graphviz' program... " << std::flush;
-  if (system((LLVM_PATH_GRAPHVIZ " " + Filename).c_str())) {
-    std::cerr << "Error viewing graph: 'Graphviz' not in path?\n";
-  } else {
-    system(("rm " + Filename).c_str());
-    return;
-  }
-#endif  // HAVE_GRAPHVIZ
-
-#ifdef HAVE_GV
-  std::cerr << "Running 'dot' program... " << std::flush;
-  if (system(("dot -Tps -Nfontname=Courier -Gsize=7.5,10 " + Filename
-              + " > /tmp/cfg.tempgraph.ps").c_str())) {
-    std::cerr << "Error running dot: 'dot' not in path?\n";
-  } else {
-    std::cerr << "\n";
-    system("gv /tmp/cfg.tempgraph.ps");
-  }
-  system(("rm " + Filename + " /tmp/cfg.tempgraph.ps").c_str());
-  return;
-#endif  // HAVE_GV
-#endif  // NDEBUG
-  std::cerr << "MachineFunction::viewCFG is only available in debug builds on "
+  ViewGraph(this, "mf" + getFunction()->getName());
+#else
+  std::cerr << "SelectionDAG::viewGraph is only available in debug builds on "
             << "systems with Graphviz or gv!\n";
-
-#ifndef NDEBUG
-  system(("rm " + Filename).c_str());
-#endif
+#endif // NDEBUG
 }
 
 void MachineFunction::viewCFGOnly() const
@@ -333,6 +309,44 @@ void MachineFrameInfo::dump(const MachineFunction &MF) const {
 }
 
 
+//===----------------------------------------------------------------------===//
+//  MachineJumpTableInfo implementation
+//===----------------------------------------------------------------------===//
+
+/// getJumpTableIndex - Create a new jump table entry in the jump table info
+/// or return an existing one.
+///
+unsigned MachineJumpTableInfo::getJumpTableIndex(
+                                     std::vector<MachineBasicBlock*> &DestBBs) {
+  for (unsigned i = 0, e = JumpTables.size(); i != e; ++i)
+    if (JumpTables[i].MBBs == DestBBs)
+      return i;
+  
+  JumpTables.push_back(MachineJumpTableEntry(DestBBs));
+  return JumpTables.size()-1;
+}
+
+
+void MachineJumpTableInfo::print(std::ostream &OS) const {
+  // FIXME: this is lame, maybe we could print out the MBB numbers or something
+  // like {1, 2, 4, 5, 3, 0}
+  for (unsigned i = 0, e = JumpTables.size(); i != e; ++i) {
+    OS << "  <jt #" << i << "> has " << JumpTables[i].MBBs.size() 
+       << " entries\n";
+  }
+}
+
+unsigned MachineJumpTableInfo::getEntrySize() const { 
+  return TD->getPointerSize(); 
+}
+
+unsigned MachineJumpTableInfo::getAlignment() const { 
+  return TD->getPointerAlignment(); 
+}
+
+void MachineJumpTableInfo::dump() const { print(std::cerr); }
+
+
 //===----------------------------------------------------------------------===//
 //  MachineConstantPool implementation
 //===----------------------------------------------------------------------===//
@@ -356,7 +370,7 @@ unsigned MachineConstantPool::getConstantPoolIndex(Constant *C,
   unsigned Offset = 0;
   if (!Constants.empty()) {
     Offset = Constants.back().Offset;
-    Offset += TD.getTypeSize(Constants.back().Val->getType());
+    Offset += TD->getTypeSize(Constants.back().Val->getType());
     Offset = (Offset+AlignMask)&~AlignMask;
   }
   
