@@ -26,22 +26,26 @@
 #include "llvm/ADT/Statistic.h"
 #include "llvm/ADT/STLExtras.h"
 #include "llvm/Support/Debug.h"
+#include "llvm/Support/Visibility.h"
 #include <algorithm>
 #include <cmath>
+#include <iostream>
 #include <set>
 #include <queue>
+#include <memory>
 using namespace llvm;
 
 namespace {
 
-  Statistic<double> efficiency
+  static Statistic<double> efficiency
   ("regalloc", "Ratio of intervals processed over total intervals");
-  Statistic<> NumBacktracks("regalloc", "Number of times we had to backtrack");
+  static Statistic<> NumBacktracks
+  ("regalloc", "Number of times we had to backtrack");
 
   static unsigned numIterations = 0;
   static unsigned numIntervals = 0;
 
-  struct RA : public MachineFunctionPass {
+  struct VISIBILITY_HIDDEN RA : public MachineFunctionPass {
     typedef std::pair<LiveInterval*, LiveInterval::iterator> IntervalPtr;
     typedef std::vector<IntervalPtr> IntervalPtrs;
   private:
@@ -404,7 +408,6 @@ static void RevertVectorIteratorsTo(RA::IntervalPtrs &V, unsigned Point) {
   }
 }
 
-
 /// assignRegOrStackSlotAtInterval - assign a register if one is available, or
 /// spill.
 void RA::assignRegOrStackSlotAtInterval(LiveInterval* cur)
@@ -537,6 +540,7 @@ void RA::assignRegOrStackSlotAtInterval(LiveInterval* cur)
  
   DEBUG(std::cerr << "\tassigning stack slot at interval "<< *cur << ":\n");
 
+  // Find a register to spill.
   float minWeight = float(HUGE_VAL);
   unsigned minReg = 0;
   for (TargetRegisterClass::iterator i = RC->allocation_order_begin(*mf_),
@@ -547,13 +551,34 @@ void RA::assignRegOrStackSlotAtInterval(LiveInterval* cur)
       minReg = reg;
     }
   }
+  
+  // If we didn't find a register that is spillable, try aliases?
+  if (!minReg) {
+    for (TargetRegisterClass::iterator i = RC->allocation_order_begin(*mf_),
+           e = RC->allocation_order_end(*mf_); i != e; ++i) {
+      unsigned reg = *i;
+      // No need to worry about if the alias register size < regsize of RC.
+      // We are going to spill all registers that alias it anyway.
+      for (const unsigned* as = mri_->getAliasSet(reg); *as; ++as) {
+        if (minWeight > SpillWeights[*as]) {
+          minWeight = SpillWeights[*as];
+          minReg = *as;
+        }
+      }
+    }
+
+    // All registers must have inf weight. Just grab one!
+    if (!minReg)
+      minReg = *RC->allocation_order_begin(*mf_);
+  }
+  
   DEBUG(std::cerr << "\t\tregister with min weight: "
         << mri_->getName(minReg) << " (" << minWeight << ")\n");
 
   // if the current has the minimum weight, we need to spill it and
   // add any added intervals back to unhandled, and restart
   // linearscan.
-  if (cur->weight <= minWeight) {
+  if (cur->weight != float(HUGE_VAL) && cur->weight <= minWeight) {
     DEBUG(std::cerr << "\t\t\tspilling(c): " << *cur << '\n';);
     int slot = vrm_->assignVirt2StackSlot(cur->reg);
     std::vector<LiveInterval*> added =
@@ -648,26 +673,17 @@ void RA::assignRegOrStackSlotAtInterval(LiveInterval* cur)
     IntervalPtrs::iterator it;
     if ((it = FindIntervalInVector(active_, i)) != active_.end()) {
       active_.erase(it);
-      if (MRegisterInfo::isPhysicalRegister(i->reg)) {
-        assert(0 && "daksjlfd");
-        prt_->delRegUse(i->reg);
+      assert(!MRegisterInfo::isPhysicalRegister(i->reg));
+      if (!spilled.count(i->reg))
         unhandled_.push(i);
-      } else {
-        if (!spilled.count(i->reg))
-          unhandled_.push(i);
-        prt_->delRegUse(vrm_->getPhys(i->reg));
-        vrm_->clearVirt(i->reg);
-      }
+      prt_->delRegUse(vrm_->getPhys(i->reg));
+      vrm_->clearVirt(i->reg);
     } else if ((it = FindIntervalInVector(inactive_, i)) != inactive_.end()) {
       inactive_.erase(it);
-      if (MRegisterInfo::isPhysicalRegister(i->reg)) {
-        assert(0 && "daksjlfd");
+      assert(!MRegisterInfo::isPhysicalRegister(i->reg));
+      if (!spilled.count(i->reg))
         unhandled_.push(i);
-      } else {
-        if (!spilled.count(i->reg))
-          unhandled_.push(i);
-        vrm_->clearVirt(i->reg);
-      }
+      vrm_->clearVirt(i->reg);
     } else {
       assert(MRegisterInfo::isVirtualRegister(i->reg) &&
              "Can only allocate virtual registers!");
@@ -691,11 +707,8 @@ void RA::assignRegOrStackSlotAtInterval(LiveInterval* cur)
         HI->expiredAt(cur->beginNumber())) {
       DEBUG(std::cerr << "\t\t\tundo changes for: " << *HI << '\n');
       active_.push_back(std::make_pair(HI, HI->begin()));
-      if (MRegisterInfo::isPhysicalRegister(HI->reg)) {
-        assert(0 &&"sdflkajsdf");
-        prt_->addRegUse(HI->reg);
-      } else
-        prt_->addRegUse(vrm_->getPhys(HI->reg));
+      assert(!MRegisterInfo::isPhysicalRegister(HI->reg));
+      prt_->addRegUse(vrm_->getPhys(HI->reg));
     }
   }
 
@@ -706,8 +719,7 @@ void RA::assignRegOrStackSlotAtInterval(LiveInterval* cur)
 
 /// getFreePhysReg - return a free physical register for this virtual register
 /// interval if we have one, otherwise return 0.
-unsigned RA::getFreePhysReg(LiveInterval* cur)
-{
+unsigned RA::getFreePhysReg(LiveInterval *cur) {
   std::vector<unsigned> inactiveCounts(mri_->getNumRegs(), 0);
   unsigned MaxInactiveCount = 0;
   
