@@ -136,6 +136,12 @@ namespace llvm {
     getFoldedVirts(MachineInstr* MI) const {
       return MI2VirtMap.equal_range(MI);
     }
+    
+    /// RemoveFromFoldedVirtMap - If the specified machine instruction is in
+    /// the folded instruction map, remove its entry from the map.
+    void RemoveFromFoldedVirtMap(MachineInstr *MI) {
+      MI2VirtMap.erase(MI);
+    }
 
     void print(std::ostream &OS) const;
     void dump() const;
@@ -151,7 +157,7 @@ namespace llvm {
   struct Spiller {
     virtual ~Spiller();
     virtual bool runOnMachineFunction(MachineFunction &MF,
-                                      const VirtRegMap &VRM) = 0;
+                                      VirtRegMap &VRM) = 0;
   };
 
   /// createSpiller - Create an return a spiller object, as specified on the
