@@ -46,20 +46,20 @@ static void runAtExitHandlers() {
 // strategy of making these functions work differently when inlined vs. when
 // not inlined, and hiding their real definitions in a separate archive file
 // that the dynamic linker can't see. For more info, search for
-// 'libc_nonshared.a' on Google, or read http://llvm.cs.uiuc.edu/PR274.
+// 'libc_nonshared.a' on Google, or read http://llvm.org/PR274.
 #if defined(__linux__)
 #if defined(HAVE_SYS_STAT_H)
 #include <sys/stat.h>
 #endif
 void *FunctionPointers[] = {
-  (void *) stat,
-  (void *) fstat,
-  (void *) lstat,
-  (void *) stat64,
-  (void *) fstat64,
-  (void *) lstat64,
-  (void *) atexit,
-  (void *) mknod
+  (void *)(intptr_t) stat,
+  (void *)(intptr_t) fstat,
+  (void *)(intptr_t) lstat,
+  (void *)(intptr_t) stat64,
+  (void *)(intptr_t) fstat64,
+  (void *)(intptr_t) lstat64,
+  (void *)(intptr_t) atexit,
+  (void *)(intptr_t) mknod
 };
 #endif // __linux__
 
@@ -90,13 +90,15 @@ static int jit_atexit(void (*Fn)(void)) {
 /// for resolving library symbols, not code generated symbols.
 ///
 void *JIT::getPointerToNamedFunction(const std::string &Name) {
-  // Check to see if this is one of the functions we want to intercept...
-  if (Name == "exit") return (void*)&jit_exit;
-  if (Name == "atexit") return (void*)&jit_atexit;
+  // Check to see if this is one of the functions we want to intercept.  Note,
+  // we cast to intptr_t here to silence a -pedantic warning that complains
+  // about casting a function pointer to a normal pointer.
+  if (Name == "exit") return (void*)(intptr_t)&jit_exit;
+  if (Name == "atexit") return (void*)(intptr_t)&jit_atexit;
 
   // If the program does not have a linked in __main function, allow it to run,
   // but print a warning.
-  if (Name == "__main") return (void*)&__mainFunc;
+  if (Name == "__main") return (void*)(intptr_t)&__mainFunc;
 
   // If it's an external function, look it up in the process image...
   void *Ptr = sys::DynamicLibrary::SearchForAddressOfSymbol(Name);
