@@ -12,6 +12,8 @@
 //===----------------------------------------------------------------------===//
 
 #include "llvm/Config/config.h"
+#include "llvm/System/IncludeFile.h"
+
 #if HAVE_ISNAN_IN_MATH_H
 # include <math.h>
 #elif HAVE_ISNAN_IN_CMATH
@@ -31,4 +33,6 @@ namespace llvm {
 int IsNAN (float f)  { return isnan (f); }
 int IsNAN (double d) { return isnan (d); }
 
-}; // end namespace llvm;
+} // end namespace llvm;
+
+DEFINING_FILE_FOR(SupportIsNAN)
