@@ -14,8 +14,8 @@
 #include "llvm/System/Path.h"
 #include "llvm/Config/config.h"
 #include <cassert>
-
-namespace llvm {
+#include <ostream>
+using namespace llvm;
 using namespace sys;
 
 //===----------------------------------------------------------------------===//
@@ -23,6 +23,11 @@ using namespace sys;
 //===          independent code.
 //===----------------------------------------------------------------------===//
 
+std::ostream& llvm::operator<<(std::ostream &strm, const sys::Path &aPath) {
+  strm << aPath.toString();
+  return strm;
+}
+
 Path
 Path::GetLLVMConfigDir() {
   Path result;
@@ -91,15 +96,11 @@ Path::FindLibrary(std::string& name) {
   return sys::Path();
 }
 
-std::string
-Path::GetDLLSuffix() {
+std::string Path::GetDLLSuffix() {
   return LTDL_SHLIB_EXT;
 }
 
-}
-
 // Include the truly platform-specific parts of this class.
-
 #if defined(LLVM_ON_UNIX)
 #include "Unix/Path.inc"
 #endif
@@ -107,3 +108,4 @@ Path::GetDLLSuffix() {
 #include "Win32/Path.inc"
 #endif
 
+DEFINING_FILE_FOR(SystemPath)
