@@ -1,10 +1,10 @@
 //===-- Process.cpp - Implement OS Process Concept --------------*- C++ -*-===//
-// 
+//
 //                     The LLVM Compiler Infrastructure
 //
-// This file was developed by Reid Spencer and is distributed under the 
+// This file was developed by Reid Spencer and is distributed under the
 // University of Illinois Open Source License. See LICENSE.TXT for details.
-// 
+//
 //===----------------------------------------------------------------------===//
 //
 //  This header file implements the operating system Process concept.
@@ -12,18 +12,24 @@
 //===----------------------------------------------------------------------===//
 
 #include "llvm/System/Process.h"
+#include "llvm/Config/config.h"
 
 namespace llvm {
 using namespace sys;
 
 //===----------------------------------------------------------------------===//
 //=== WARNING: Implementation here must contain only TRULY operating system
-//===          independent code. 
+//===          independent code.
 //===----------------------------------------------------------------------===//
 
 }
 
 // Include the platform-specific parts of this class.
-#include "platform/Process.cpp"
+#ifdef LLVM_ON_UNIX
+#include "Unix/Process.inc"
+#endif
+#ifdef LLVM_ON_WIN32
+#include "Win32/Process.inc"
+#endif
 
-// vim: sw=2 smartindent smarttab tw=80 autoindent expandtab
+DEFINING_FILE_FOR(SystemProcess)
