@@ -1,10 +1,10 @@
 //===- Signals.cpp - Signal Handling support --------------------*- C++ -*-===//
-// 
+//
 //                     The LLVM Compiler Infrastructure
 //
 // This file was developed by the LLVM research group and is distributed under
 // the University of Illinois Open Source License. See LICENSE.TXT for details.
-// 
+//
 //===----------------------------------------------------------------------===//
 //
 // This file defines some helpful functions for dealing with the possibility of
@@ -13,18 +13,24 @@
 //===----------------------------------------------------------------------===//
 
 #include "llvm/System/Signals.h"
+#include "llvm/Config/config.h"
 
 namespace llvm {
 using namespace sys;
 
 //===----------------------------------------------------------------------===//
 //=== WARNING: Implementation here must contain only TRULY operating system
-//===          independent code. 
+//===          independent code.
 //===----------------------------------------------------------------------===//
 
 }
 
 // Include the platform-specific parts of this class.
-#include "platform/Signals.cpp"
+#ifdef LLVM_ON_UNIX
+#include "Unix/Signals.inc"
+#endif
+#ifdef LLVM_ON_WIN32
+#include "Win32/Signals.inc"
+#endif
 
-// vim: sw=2 smartindent smarttab tw=80 autoindent expandtab
+DEFINING_FILE_FOR(SystemSignals)
