@@ -161,8 +161,8 @@ void IndVarSimplify::EliminatePointerRecurrence(PHINode *PN,
           assert(NumOps > 1 && "CE folding didn't work!");
           if (CE->getOperand(NumOps-1)->isNullValue()) {
             // Check to make sure the last index really is an array index.
-            gep_type_iterator GTI = gep_type_begin(GEPI);
-            for (unsigned i = 1, e = GEPI->getNumOperands()-1;
+            gep_type_iterator GTI = gep_type_begin(CE);
+            for (unsigned i = 1, e = CE->getNumOperands()-1;
                  i != e; ++i, ++GTI)
               /*empty*/;
             if (isa<SequentialType>(*GTI)) {
@@ -321,11 +321,26 @@ void IndVarSimplify::RewriteLoopExitValues(Loop *L) {
               HasConstantItCount) {
             // Find out if this predictably varying value is actually used
             // outside of the loop.  "extra" as opposed to "intra".
-            std::vector<User*> ExtraLoopUsers;
+            std::vector<Instruction*> ExtraLoopUsers;
             for (Value::use_iterator UI = I->use_begin(), E = I->use_end();
-                 UI != E; ++UI)
-              if (!L->contains(cast<Instruction>(*UI)->getParent()))
-                ExtraLoopUsers.push_back(*UI);
+                 UI != E; ++UI) {
+              Instruction *User = cast<Instruction>(*UI);
+              if (!L->contains(User->getParent())) {
+                // If this is a PHI node in the exit block and we're inserting,
+                // into the exit block, it must have a single entry.  In this
+                // case, we can't insert the code after the PHI and have the PHI
+                // still use it.  Instead, don't insert the the PHI.
+                if (PHINode *PN = dyn_cast<PHINode>(User)) {
+                  // FIXME: This is a case where LCSSA pessimizes code, this
+                  // should be fixed better.
+                  if (PN->getNumOperands() == 2 && 
+                      PN->getParent() == BlockToInsertInto)
+                    continue;
+                }
+                ExtraLoopUsers.push_back(User);
+              }
+            }
+            
             if (!ExtraLoopUsers.empty()) {
               // Okay, this instruction has a user outside of the current loop
               // and varies predictably in this loop.  Evaluate the value it
@@ -343,8 +358,36 @@ void IndVarSimplify::RewriteLoopExitValues(Loop *L) {
 
                 // Rewrite any users of the computed value outside of the loop
                 // with the newly computed value.
-                for (unsigned i = 0, e = ExtraLoopUsers.size(); i != e; ++i)
-                  ExtraLoopUsers[i]->replaceUsesOfWith(I, NewVal);
+                for (unsigned i = 0, e = ExtraLoopUsers.size(); i != e; ++i) {
+                  PHINode* PN = dyn_cast<PHINode>(ExtraLoopUsers[i]);
+                  if (PN && PN->getNumOperands() == 2 &&
+                      !L->contains(PN->getParent())) {
+                    // We're dealing with an LCSSA Phi.  Handle it specially.
+                    Instruction* LCSSAInsertPt = BlockToInsertInto->begin();
+                    
+                    Instruction* NewInstr = dyn_cast<Instruction>(NewVal);
+                    if (NewInstr && !isa<PHINode>(NewInstr) &&
+                        !L->contains(NewInstr->getParent()))
+                      for (unsigned j = 0; j < NewInstr->getNumOperands(); ++j){
+                        Instruction* PredI = 
+                                 dyn_cast<Instruction>(NewInstr->getOperand(j));
+                        if (PredI && L->contains(PredI->getParent())) {
+                          PHINode* NewLCSSA = new PHINode(PredI->getType(),
+                                                    PredI->getName() + ".lcssa",
+                                                    LCSSAInsertPt);
+                          NewLCSSA->addIncoming(PredI, 
+                                     BlockToInsertInto->getSinglePredecessor());
+                        
+                          NewInstr->replaceUsesOfWith(PredI, NewLCSSA);
+                        }
+                      }
+                    
+                    PN->replaceAllUsesWith(NewVal);
+                    PN->eraseFromParent();
+                  } else {
+                    ExtraLoopUsers[i]->replaceUsesOfWith(I, NewVal);
+                  }
+                }
 
                 // If this instruction is dead now, schedule it to be removed.
                 if (I->use_empty())
