@@ -31,7 +31,10 @@
 #include "llvm/Target/TargetData.h"
 #include "llvm/ADT/Statistic.h"
 #include "llvm/Support/Debug.h"
+#include "llvm/Support/Visibility.h"
+#include "llvm/Target/TargetLowering.h"
 #include <algorithm>
+#include <iostream>
 #include <set>
 using namespace llvm;
 
@@ -74,23 +77,50 @@ namespace {
     }
   };
 
+  /// IVInfo - This structure keeps track of one IV expression inserted during
+  /// StrengthReduceStridedIVUsers. It contains the stride, the common base, as
+  /// well as the PHI node and increment value created for rewrite.
+  struct IVExpr {
+    SCEVHandle  Stride;
+    SCEVHandle  Base;
+    PHINode    *PHI;
+    Value      *IncV;
 
-  class LoopStrengthReduce : public FunctionPass {
+    IVExpr()
+      : Stride(SCEVUnknown::getIntegerSCEV(0, Type::UIntTy)),
+        Base  (SCEVUnknown::getIntegerSCEV(0, Type::UIntTy)) {}
+    IVExpr(const SCEVHandle &stride, const SCEVHandle &base, PHINode *phi,
+           Value *incv)
+      : Stride(stride), Base(base), PHI(phi), IncV(incv) {}
+  };
+
+  /// IVsOfOneStride - This structure keeps track of all IV expression inserted
+  /// during StrengthReduceStridedIVUsers for a particular stride of the IV.
+  struct IVsOfOneStride {
+    std::vector<IVExpr> IVs;
+
+    void addIV(const SCEVHandle &Stride, const SCEVHandle &Base, PHINode *PHI,
+               Value *IncV) {
+      IVs.push_back(IVExpr(Stride, Base, PHI, IncV));
+    }
+  };
+
+  class VISIBILITY_HIDDEN LoopStrengthReduce : public FunctionPass {
     LoopInfo *LI;
-    DominatorSet *DS;
+    ETForest *EF;
     ScalarEvolution *SE;
     const TargetData *TD;
     const Type *UIntPtrTy;
     bool Changed;
 
-    /// MaxTargetAMSize - This is the maximum power-of-two scale value that the
-    /// target can handle for free with its addressing modes.
-    unsigned MaxTargetAMSize;
-
     /// IVUsesByStride - Keep track of all uses of induction variables that we
     /// are interested in.  The key of the map is the stride of the access.
     std::map<SCEVHandle, IVUsersOfOneStride> IVUsesByStride;
 
+    /// IVsByStride - Keep track of all IVs that have been inserted for a
+    /// particular stride.
+    std::map<SCEVHandle, IVsOfOneStride> IVsByStride;
+
     /// StrideOrder - An ordering of the keys in IVUsesByStride that is stable:
     /// We use this to iterate over the IVUsesByStride collection without being
     /// dependent on random ordering of pointers in the process.
@@ -104,14 +134,19 @@ namespace {
     /// DeadInsts - Keep track of instructions we may have made dead, so that
     /// we can remove them after we are done working.
     std::set<Instruction*> DeadInsts;
+
+    /// TLI - Keep a pointer of a TargetLowering to consult for determining
+    /// transformation profitability.
+    const TargetLowering *TLI;
+
   public:
-    LoopStrengthReduce(unsigned MTAMS = 1)
-      : MaxTargetAMSize(MTAMS) {
+    LoopStrengthReduce(const TargetLowering *tli = NULL)
+      : TLI(tli) {
     }
 
     virtual bool runOnFunction(Function &) {
       LI = &getAnalysis<LoopInfo>();
-      DS = &getAnalysis<DominatorSet>();
+      EF = &getAnalysis<ETForest>();
       SE = &getAnalysis<ScalarEvolution>();
       TD = &getAnalysis<TargetData>();
       UIntPtrTy = TD->getIntPtrType();
@@ -129,13 +164,14 @@ namespace {
       AU.addPreservedID(LoopSimplifyID);
       AU.addPreserved<LoopInfo>();
       AU.addPreserved<DominatorSet>();
+      AU.addPreserved<ETForest>();
       AU.addPreserved<ImmediateDominators>();
       AU.addPreserved<DominanceFrontier>();
       AU.addPreserved<DominatorTree>();
 
       AU.addRequiredID(LoopSimplifyID);
       AU.addRequired<LoopInfo>();
-      AU.addRequired<DominatorSet>();
+      AU.addRequired<ETForest>();
       AU.addRequired<TargetData>();
       AU.addRequired<ScalarEvolution>();
     }
@@ -151,6 +187,8 @@ private:
 
     void OptimizeIndvars(Loop *L);
 
+    unsigned CheckForIVReuse(const SCEVHandle&, IVExpr&, const Type*);
+
     void StrengthReduceStridedIVUsers(const SCEVHandle &Stride,
                                       IVUsersOfOneStride &Uses,
                                       Loop *L, bool isOnlyStride);
@@ -160,8 +198,8 @@ private:
                                     "Loop Strength Reduction");
 }
 
-FunctionPass *llvm::createLoopStrengthReducePass(unsigned MaxTargetAMSize) {
-  return new LoopStrengthReduce(MaxTargetAMSize);
+FunctionPass *llvm::createLoopStrengthReducePass(const TargetLowering *TLI) {
+  return new LoopStrengthReduce(TLI);
 }
 
 /// getCastedVersionOf - Return the specified value casted to uintptr_t.
@@ -174,24 +212,7 @@ Value *LoopStrengthReduce::getCastedVersionOf(Value *V) {
   Value *&New = CastedPointers[V];
   if (New) return New;
   
-  BasicBlock::iterator InsertPt;
-  if (Argument *Arg = dyn_cast<Argument>(V)) {
-    // Insert into the entry of the function, after any allocas.
-    InsertPt = Arg->getParent()->begin()->begin();
-    while (isa<AllocaInst>(InsertPt)) ++InsertPt;
-  } else {
-    if (InvokeInst *II = dyn_cast<InvokeInst>(V)) {
-      InsertPt = II->getNormalDest()->begin();
-    } else {
-      InsertPt = cast<Instruction>(V);
-      ++InsertPt;
-    }
-
-    // Do not insert casts into the middle of PHI node blocks.
-    while (isa<PHINode>(InsertPt)) ++InsertPt;
-  }
-  
-  New = new CastInst(V, UIntPtrTy, V->getName(), InsertPt);
+  New = SCEVExpander::InsertCastOfTo(V, UIntPtrTy);
   DeadInsts.insert(cast<Instruction>(New));
   return New;
 }
@@ -324,7 +345,7 @@ static bool getSCEVStartAndStride(const SCEVHandle &SH, Loop *L,
 /// the loop, resulting in reg-reg copies (if we use the pre-inc value when we
 /// should use the post-inc value).
 static bool IVUseShouldUsePostIncValue(Instruction *User, Instruction *IV,
-                                       Loop *L, DominatorSet *DS, Pass *P) {
+                                       Loop *L, ETForest *EF, Pass *P) {
   // If the user is in the loop, use the preinc value.
   if (L->contains(User->getParent())) return false;
   
@@ -332,7 +353,7 @@ static bool IVUseShouldUsePostIncValue(Instruction *User, Instruction *IV,
   
   // Ok, the user is outside of the loop.  If it is dominated by the latch
   // block, use the post-inc value.
-  if (DS->dominates(LatchBlock, User->getParent()))
+  if (EF->dominates(LatchBlock, User->getParent()))
     return true;
 
   // There is one case we have to be careful of: PHI nodes.  These little guys
@@ -349,7 +370,7 @@ static bool IVUseShouldUsePostIncValue(Instruction *User, Instruction *IV,
   for (unsigned i = 0, e = PN->getNumIncomingValues(); i != e; ++i)
     if (PN->getIncomingValue(i) == IV) {
       ++NumUses;
-      if (!DS->dominates(LatchBlock, PN->getIncomingBlock(i)))
+      if (!EF->dominates(LatchBlock, PN->getIncomingBlock(i)))
         return false;
     }
 
@@ -415,7 +436,7 @@ bool LoopStrengthReduce::AddUsersIfInteresting(Instruction *I, Loop *L,
       // Okay, we found a user that we cannot reduce.  Analyze the instruction
       // and decide what to do with it.  If we are a use inside of the loop, use
       // the value before incrementation, otherwise use it after incrementation.
-      if (IVUseShouldUsePostIncValue(User, I, L, DS, this)) {
+      if (IVUseShouldUsePostIncValue(User, I, L, EF, this)) {
         // The value used will be incremented by the stride more than we are
         // expecting, so subtract this off.
         SCEVHandle NewStart = SCEV::getMinusSCEV(Start, Stride);
@@ -475,6 +496,10 @@ namespace {
     void RewriteInstructionToUseNewBase(const SCEVHandle &NewBase,
                                         SCEVExpander &Rewriter, Loop *L,
                                         Pass *P);
+    
+    Value *InsertCodeForBaseAtPosition(const SCEVHandle &NewBase, 
+                                       SCEVExpander &Rewriter,
+                                       Instruction *IP, Loop *L);
     void dump() const;
   };
 }
@@ -488,6 +513,41 @@ void BasedUser::dump() const {
   std::cerr << "   Inst: " << *Inst;
 }
 
+Value *BasedUser::InsertCodeForBaseAtPosition(const SCEVHandle &NewBase, 
+                                              SCEVExpander &Rewriter,
+                                              Instruction *IP, Loop *L) {
+  // Figure out where we *really* want to insert this code.  In particular, if
+  // the user is inside of a loop that is nested inside of L, we really don't
+  // want to insert this expression before the user, we'd rather pull it out as
+  // many loops as possible.
+  LoopInfo &LI = Rewriter.getLoopInfo();
+  Instruction *BaseInsertPt = IP;
+  
+  // Figure out the most-nested loop that IP is in.
+  Loop *InsertLoop = LI.getLoopFor(IP->getParent());
+  
+  // If InsertLoop is not L, and InsertLoop is nested inside of L, figure out
+  // the preheader of the outer-most loop where NewBase is not loop invariant.
+  while (InsertLoop && NewBase->isLoopInvariant(InsertLoop)) {
+    BaseInsertPt = InsertLoop->getLoopPreheader()->getTerminator();
+    InsertLoop = InsertLoop->getParentLoop();
+  }
+  
+  // If there is no immediate value, skip the next part.
+  if (SCEVConstant *SC = dyn_cast<SCEVConstant>(Imm))
+    if (SC->getValue()->isNullValue())
+      return Rewriter.expandCodeFor(NewBase, BaseInsertPt,
+                                    OperandValToReplace->getType());
+
+  Value *Base = Rewriter.expandCodeFor(NewBase, BaseInsertPt);
+  
+  // Always emit the immediate (if non-zero) into the same block as the user.
+  SCEVHandle NewValSCEV = SCEVAddExpr::get(SCEVUnknown::get(Base), Imm);
+  return Rewriter.expandCodeFor(NewValSCEV, IP,
+                                OperandValToReplace->getType());
+}
+
+
 // Once we rewrite the code to insert the new IVs we want, update the
 // operands of Inst to use the new expression 'NewBase', with 'Imm' added
 // to it.
@@ -495,9 +555,7 @@ void BasedUser::RewriteInstructionToUseNewBase(const SCEVHandle &NewBase,
                                                SCEVExpander &Rewriter,
                                                Loop *L, Pass *P) {
   if (!isa<PHINode>(Inst)) {
-    SCEVHandle NewValSCEV = SCEVAddExpr::get(NewBase, Imm);
-    Value *NewVal = Rewriter.expandCodeFor(NewValSCEV, Inst,
-                                           OperandValToReplace->getType());
+    Value *NewVal = InsertCodeForBaseAtPosition(NewBase, Rewriter, Inst, L);
     // Replace the use of the operand Value with the new Phi we just created.
     Inst->replaceUsesOfWith(OperandValToReplace, NewVal);
     DEBUG(std::cerr << "    CHANGED: IMM =" << *Imm << "  Inst = " << *Inst);
@@ -537,11 +595,8 @@ void BasedUser::RewriteInstructionToUseNewBase(const SCEVHandle &NewBase,
       Value *&Code = InsertedCode[PN->getIncomingBlock(i)];
       if (!Code) {
         // Insert the code into the end of the predecessor block.
-        BasicBlock::iterator InsertPt =PN->getIncomingBlock(i)->getTerminator();
-      
-        SCEVHandle NewValSCEV = SCEVAddExpr::get(NewBase, Imm);
-        Code = Rewriter.expandCodeFor(NewValSCEV, InsertPt,
-                                      OperandValToReplace->getType());
+        Instruction *InsertPt = PN->getIncomingBlock(i)->getTerminator();
+        Code = InsertCodeForBaseAtPosition(NewBase, Rewriter, InsertPt, L);
       }
       
       // Replace the use of the operand Value with the new Phi we just created.
@@ -555,25 +610,25 @@ void BasedUser::RewriteInstructionToUseNewBase(const SCEVHandle &NewBase,
 
 /// isTargetConstant - Return true if the following can be referenced by the
 /// immediate field of a target instruction.
-static bool isTargetConstant(const SCEVHandle &V) {
-
-  // FIXME: Look at the target to decide if &GV is a legal constant immediate.
+static bool isTargetConstant(const SCEVHandle &V, const TargetLowering *TLI) {
   if (SCEVConstant *SC = dyn_cast<SCEVConstant>(V)) {
-    // PPC allows a sign-extended 16-bit immediate field.
-    if ((int64_t)SC->getValue()->getRawValue() > -(1 << 16) &&
-        (int64_t)SC->getValue()->getRawValue() < (1 << 16)-1)
-      return true;
-    return false;
+    int64_t V = SC->getValue()->getSExtValue();
+    if (TLI)
+      return TLI->isLegalAddressImmediate(V);
+    else
+      // Defaults to PPC. PPC allows a sign-extended 16-bit immediate field.
+      return (V > -(1 << 16) && V < (1 << 16)-1);
   }
 
-  return false;     // ENABLE this for x86
-
   if (SCEVUnknown *SU = dyn_cast<SCEVUnknown>(V))
     if (ConstantExpr *CE = dyn_cast<ConstantExpr>(SU->getValue()))
-      if (CE->getOpcode() == Instruction::Cast)
-        if (isa<GlobalValue>(CE->getOperand(0)))
-          // FIXME: should check to see that the dest is uintptr_t!
+      if (CE->getOpcode() == Instruction::Cast) {
+        Constant *Op0 = CE->getOperand(0);
+        if (isa<GlobalValue>(Op0) &&
+            TLI &&
+            TLI->isLegalAddressImmediate(cast<GlobalValue>(Op0)))
           return true;
+      }
   return false;
 }
 
@@ -619,22 +674,25 @@ static void MoveLoopVariantsToImediateField(SCEVHandle &Val, SCEVHandle &Imm,
 /// MoveImmediateValues - Look at Val, and pull out any additions of constants
 /// that can fit into the immediate field of instructions in the target.
 /// Accumulate these immediate values into the Imm value.
-static void MoveImmediateValues(SCEVHandle &Val, SCEVHandle &Imm,
+static void MoveImmediateValues(const TargetLowering *TLI,
+                                SCEVHandle &Val, SCEVHandle &Imm,
                                 bool isAddress, Loop *L) {
   if (SCEVAddExpr *SAE = dyn_cast<SCEVAddExpr>(Val)) {
     std::vector<SCEVHandle> NewOps;
     NewOps.reserve(SAE->getNumOperands());
     
-    for (unsigned i = 0; i != SAE->getNumOperands(); ++i)
-      if (isAddress && isTargetConstant(SAE->getOperand(i))) {
-        Imm = SCEVAddExpr::get(Imm, SAE->getOperand(i));
-      } else if (!SAE->getOperand(i)->isLoopInvariant(L)) {
+    for (unsigned i = 0; i != SAE->getNumOperands(); ++i) {
+      SCEVHandle NewOp = SAE->getOperand(i);
+      MoveImmediateValues(TLI, NewOp, Imm, isAddress, L);
+      
+      if (!NewOp->isLoopInvariant(L)) {
         // If this is a loop-variant expression, it must stay in the immediate
         // field of the expression.
-        Imm = SCEVAddExpr::get(Imm, SAE->getOperand(i));
+        Imm = SCEVAddExpr::get(Imm, NewOp);
       } else {
-        NewOps.push_back(SAE->getOperand(i));
+        NewOps.push_back(NewOp);
       }
+    }
 
     if (NewOps.empty())
       Val = SCEVUnknown::getIntegerSCEV(0, Val->getType());
@@ -644,7 +702,7 @@ static void MoveImmediateValues(SCEVHandle &Val, SCEVHandle &Imm,
   } else if (SCEVAddRecExpr *SARE = dyn_cast<SCEVAddRecExpr>(Val)) {
     // Try to pull immediates out of the start value of nested addrec's.
     SCEVHandle Start = SARE->getStart();
-    MoveImmediateValues(Start, Imm, isAddress, L);
+    MoveImmediateValues(TLI, Start, Imm, isAddress, L);
     
     if (Start != SARE->getStart()) {
       std::vector<SCEVHandle> Ops(SARE->op_begin(), SARE->op_end());
@@ -652,11 +710,36 @@ static void MoveImmediateValues(SCEVHandle &Val, SCEVHandle &Imm,
       Val = SCEVAddRecExpr::get(Ops, SARE->getLoop());
     }
     return;
+  } else if (SCEVMulExpr *SME = dyn_cast<SCEVMulExpr>(Val)) {
+    // Transform "8 * (4 + v)" -> "32 + 8*V" if "32" fits in the immed field.
+    if (isAddress && isTargetConstant(SME->getOperand(0), TLI) &&
+        SME->getNumOperands() == 2 && SME->isLoopInvariant(L)) {
+
+      SCEVHandle SubImm = SCEVUnknown::getIntegerSCEV(0, Val->getType());
+      SCEVHandle NewOp = SME->getOperand(1);
+      MoveImmediateValues(TLI, NewOp, SubImm, isAddress, L);
+      
+      // If we extracted something out of the subexpressions, see if we can 
+      // simplify this!
+      if (NewOp != SME->getOperand(1)) {
+        // Scale SubImm up by "8".  If the result is a target constant, we are
+        // good.
+        SubImm = SCEVMulExpr::get(SubImm, SME->getOperand(0));
+        if (isTargetConstant(SubImm, TLI)) {
+          // Accumulate the immediate.
+          Imm = SCEVAddExpr::get(Imm, SubImm);
+          
+          // Update what is left of 'Val'.
+          Val = SCEVMulExpr::get(SME->getOperand(0), NewOp);
+          return;
+        }
+      }
+    }
   }
 
   // Loop-variant expressions must stay in the immediate field of the
   // expression.
-  if ((isAddress && isTargetConstant(Val)) ||
+  if ((isAddress && isTargetConstant(Val, TLI)) ||
       !Val->isLoopInvariant(L)) {
     Imm = SCEVAddExpr::get(Imm, Val);
     Val = SCEVUnknown::getIntegerSCEV(0, Val->getType());
@@ -775,6 +858,52 @@ RemoveCommonExpressionsFromUseBases(std::vector<BasedUser> &Uses) {
   return Result;
 }
 
+/// isZero - returns true if the scalar evolution expression is zero.
+///
+static bool isZero(SCEVHandle &V) {
+  if (SCEVConstant *SC = dyn_cast<SCEVConstant>(V))
+    return SC->getValue()->getRawValue() == 0;
+  return false;
+}
+
+
+/// CheckForIVReuse - Returns the multiple if the stride is the multiple
+/// of a previous stride and it is a legal value for the target addressing
+/// mode scale component. This allows the users of this stride to be rewritten
+/// as prev iv * factor. It returns 0 if no reuse is possible.
+unsigned LoopStrengthReduce::CheckForIVReuse(const SCEVHandle &Stride,
+                                             IVExpr &IV, const Type *Ty) {
+  if (!TLI) return 0;
+
+  if (SCEVConstant *SC = dyn_cast<SCEVConstant>(Stride)) {
+    int64_t SInt = SC->getValue()->getSExtValue();
+    if (SInt == 1) return 0;
+
+    for (TargetLowering::legal_am_scale_iterator
+           I = TLI->legal_am_scale_begin(), E = TLI->legal_am_scale_end();
+         I != E; ++I) {
+      unsigned Scale = *I;
+      if (unsigned(abs(SInt)) < Scale || (SInt % Scale) != 0)
+        continue;
+      std::map<SCEVHandle, IVsOfOneStride>::iterator SI =
+        IVsByStride.find(SCEVUnknown::getIntegerSCEV(SInt/Scale, Type::UIntTy));
+      if (SI == IVsByStride.end())
+        continue;
+      for (std::vector<IVExpr>::iterator II = SI->second.IVs.begin(),
+             IE = SI->second.IVs.end(); II != IE; ++II)
+        // FIXME: Only handle base == 0 for now.
+        // Only reuse previous IV if it would not require a type conversion.
+        if (isZero(II->Base) &&
+            II->Base->getType()->isLosslesslyConvertibleTo(Ty)) {
+          IV = *II;
+          return Scale;
+        }
+    }
+  }
+
+  return 0;
+}
+
 
 /// StrengthReduceStridedIVUsers - Strength reduce all of the users of a single
 /// stride of IV.  All of the users may have different starting values, and this
@@ -801,7 +930,7 @@ void LoopStrengthReduce::StrengthReduceStridedIVUsers(const SCEVHandle &Stride,
     assert(UsersToProcess.back().Base->isLoopInvariant(L) &&
            "Base value is not loop invariant!");
   }
-  
+
   // We now have a whole bunch of uses of like-strided induction variables, but
   // they might all have different bases.  We want to emit one PHI node for this
   // stride which we fold as many common expressions (between the IVs) into as
@@ -809,8 +938,24 @@ void LoopStrengthReduce::StrengthReduceStridedIVUsers(const SCEVHandle &Stride,
   // for the strides (e.g. if we have "A+C+B" and "A+B+D" as our bases, find
   // "A+B"), emit it to the preheader, then remove the expression from the
   // UsersToProcess base values.
-  SCEVHandle CommonExprs = RemoveCommonExpressionsFromUseBases(UsersToProcess);
+  SCEVHandle CommonExprs =
+    RemoveCommonExpressionsFromUseBases(UsersToProcess);
   
+  // Check if it is possible to reuse a IV with stride that is factor of this
+  // stride. And the multiple is a number that can be encoded in the scale
+  // field of the target addressing mode.
+  PHINode *NewPHI = NULL;
+  Value   *IncV   = NULL;
+  IVExpr   ReuseIV;
+  unsigned RewriteFactor = CheckForIVReuse(Stride, ReuseIV,
+                                           CommonExprs->getType());
+  if (RewriteFactor != 0) {
+    DEBUG(std::cerr << "BASED ON IV of STRIDE " << *ReuseIV.Stride
+          << " and BASE " << *ReuseIV.Base << " :\n");
+    NewPHI = ReuseIV.PHI;
+    IncV   = ReuseIV.IncV;
+  }
+
   // Next, figure out what we can represent in the immediate fields of
   // instructions.  If we can represent anything there, move it to the imm
   // fields of the BasedUsers.  We do this so that it increases the commonality
@@ -833,16 +978,16 @@ void LoopStrengthReduce::StrengthReduceStridedIVUsers(const SCEVHandle &Stride,
         if (SI->getOperand(1) == UsersToProcess[i].OperandValToReplace)
           isAddress = true;
       
-      MoveImmediateValues(UsersToProcess[i].Base, UsersToProcess[i].Imm,
+      MoveImmediateValues(TLI, UsersToProcess[i].Base, UsersToProcess[i].Imm,
                           isAddress, L);
     }
   }
- 
+
   // Now that we know what we need to do, insert the PHI node itself.
   //
   DEBUG(std::cerr << "INSERTING IV of STRIDE " << *Stride << " and BASE "
         << *CommonExprs << " :\n");
-    
+
   SCEVExpander Rewriter(*SE, *LI);
   SCEVExpander PreheaderRewriter(*SE, *LI);
   
@@ -851,33 +996,48 @@ void LoopStrengthReduce::StrengthReduceStridedIVUsers(const SCEVHandle &Stride,
   Instruction *PhiInsertBefore = L->getHeader()->begin();
   
   BasicBlock *LatchBlock = L->getLoopLatch();
-  
-  // Create a new Phi for this base, and stick it in the loop header.
+
   const Type *ReplacedTy = CommonExprs->getType();
-  PHINode *NewPHI = new PHINode(ReplacedTy, "iv.", PhiInsertBefore);
-  ++NumInserted;
-  
-  // Insert the stride into the preheader.
-  Value *StrideV = PreheaderRewriter.expandCodeFor(Stride, PreInsertPt,
-                                                   ReplacedTy);
-  if (!isa<ConstantInt>(StrideV)) ++NumVariable;
 
+  // Emit the initial base value into the loop preheader.
+  Value *CommonBaseV
+    = PreheaderRewriter.expandCodeFor(CommonExprs, PreInsertPt,
+                                      ReplacedTy);
 
-  // Emit the initial base value into the loop preheader, and add it to the
-  // Phi node.
-  Value *PHIBaseV = PreheaderRewriter.expandCodeFor(CommonExprs, PreInsertPt,
-                                                    ReplacedTy);
-  NewPHI->addIncoming(PHIBaseV, Preheader);
+  if (RewriteFactor == 0) {
+    // Create a new Phi for this base, and stick it in the loop header.
+    NewPHI = new PHINode(ReplacedTy, "iv.", PhiInsertBefore);
+    ++NumInserted;
   
-  // Emit the increment of the base value before the terminator of the loop
-  // latch block, and add it to the Phi node.
-  SCEVHandle IncExp = SCEVAddExpr::get(SCEVUnknown::get(NewPHI),
-                                       SCEVUnknown::get(StrideV));
+    // Add common base to the new Phi node.
+    NewPHI->addIncoming(CommonBaseV, Preheader);
+
+    // Insert the stride into the preheader.
+    Value *StrideV = PreheaderRewriter.expandCodeFor(Stride, PreInsertPt,
+                                                     ReplacedTy);
+    if (!isa<ConstantInt>(StrideV)) ++NumVariable;
+
+    // Emit the increment of the base value before the terminator of the loop
+    // latch block, and add it to the Phi node.
+    SCEVHandle IncExp = SCEVAddExpr::get(SCEVUnknown::get(NewPHI),
+                                         SCEVUnknown::get(StrideV));
   
-  Value *IncV = Rewriter.expandCodeFor(IncExp, LatchBlock->getTerminator(),
-                                       ReplacedTy);
-  IncV->setName(NewPHI->getName()+".inc");
-  NewPHI->addIncoming(IncV, LatchBlock);
+    IncV = Rewriter.expandCodeFor(IncExp, LatchBlock->getTerminator(),
+                                  ReplacedTy);
+    IncV->setName(NewPHI->getName()+".inc");
+    NewPHI->addIncoming(IncV, LatchBlock);
+
+    // Remember this in case a later stride is multiple of this.
+    IVsByStride[Stride].addIV(Stride, CommonExprs, NewPHI, IncV);
+  } else {
+    Constant *C = dyn_cast<Constant>(CommonBaseV);
+    if (!C ||
+        (!C->isNullValue() &&
+         !isTargetConstant(SCEVUnknown::get(CommonBaseV), TLI)))
+      // We want the common base emitted into the preheader!
+      CommonBaseV = new CastInst(CommonBaseV, CommonBaseV->getType(),
+                                 "commonbase", PreInsertPt);
+  }
 
   // Sort by the base value, so that all IVs with identical bases are next to
   // each other.
@@ -895,7 +1055,7 @@ void LoopStrengthReduce::StrengthReduceStridedIVUsers(const SCEVHandle &Stride,
     // this by forcing a noop cast to be inserted into the preheader in this
     // case.
     if (Constant *C = dyn_cast<Constant>(BaseV))
-      if (!C->isNullValue() && !isTargetConstant(Base)) {
+      if (!C->isNullValue() && !isTargetConstant(Base, TLI)) {
         // We want this constant emitted into the preheader!
         BaseV = new CastInst(BaseV, BaseV->getType(), "preheaderinsert",
                              PreInsertPt);       
@@ -918,18 +1078,38 @@ void LoopStrengthReduce::StrengthReduceStridedIVUsers(const SCEVHandle &Stride,
         if (L->contains(User.Inst->getParent()))
           User.Inst->moveBefore(LatchBlock->getTerminator());
       }
+      if (RewriteOp->getType() != ReplacedTy)
+        RewriteOp = SCEVExpander::InsertCastOfTo(RewriteOp, ReplacedTy);
+
       SCEVHandle RewriteExpr = SCEVUnknown::get(RewriteOp);
 
       // Clear the SCEVExpander's expression map so that we are guaranteed
       // to have the code emitted where we expect it.
       Rewriter.clear();
-     
+
+      // If we are reusing the iv, then it must be multiplied by a constant
+      // factor take advantage of addressing mode scale component.
+      if (RewriteFactor != 0) {
+        RewriteExpr =
+          SCEVMulExpr::get(SCEVUnknown::getIntegerSCEV(RewriteFactor,
+                                                       RewriteExpr->getType()),
+                           RewriteExpr);
+
+        // The common base is emitted in the loop preheader. But since we
+        // are reusing an IV, it has not been used to initialize the PHI node.
+        // Add it to the expression used to rewrite the uses.
+        if (!isa<ConstantInt>(CommonBaseV) ||
+            !cast<ConstantInt>(CommonBaseV)->isNullValue())
+          RewriteExpr = SCEVAddExpr::get(RewriteExpr,
+                                         SCEVUnknown::get(CommonBaseV));
+      }
+
       // Now that we know what we need to do, insert code before User for the
       // immediate and any loop-variant expressions.
       if (!isa<ConstantInt>(BaseV) || !cast<ConstantInt>(BaseV)->isNullValue())
         // Add BaseV to the PHI value if needed.
         RewriteExpr = SCEVAddExpr::get(RewriteExpr, SCEVUnknown::get(BaseV));
-      
+
       User.RewriteInstructionToUseNewBase(RewriteExpr, Rewriter, L, this);
 
       // Mark old value we replaced as possibly dead, so that it is elminated
@@ -967,7 +1147,7 @@ void LoopStrengthReduce::OptimizeIndvars(Loop *L) {
 
   // Finally, get the terminating condition for the loop if possible.  If we
   // can, we want to change it to use a post-incremented version of its
-  // induction variable, to allow coallescing the live ranges for the IV into
+  // induction variable, to allow coalescing the live ranges for the IV into
   // one register value.
   PHINode *SomePHI = cast<PHINode>(L->getHeader()->begin());
   BasicBlock  *Preheader = L->getLoopPreheader();
@@ -1026,12 +1206,36 @@ void LoopStrengthReduce::OptimizeIndvars(Loop *L) {
   }
 
   // If we get to here, we know that we can transform the setcc instruction to
-  // use the post-incremented version of the IV, allowing us to coallesce the
+  // use the post-incremented version of the IV, allowing us to coalesce the
   // live ranges for the IV correctly.
   CondUse->Offset = SCEV::getMinusSCEV(CondUse->Offset, *CondStride);
   CondUse->isUseOfPostIncrementedValue = true;
 }
 
+namespace {
+  // Constant strides come first which in turns are sorted by their absolute
+  // values. If absolute values are the same, then positive strides comes first.
+  // e.g.
+  // 4, -1, X, 1, 2 ==> 1, -1, 2, 4, X
+  struct StrideCompare {
+    bool operator()(const SCEVHandle &LHS, const SCEVHandle &RHS) {
+      SCEVConstant *LHSC = dyn_cast<SCEVConstant>(LHS);
+      SCEVConstant *RHSC = dyn_cast<SCEVConstant>(RHS);
+      if (LHSC && RHSC) {
+        int64_t  LV = LHSC->getValue()->getSExtValue();
+        int64_t  RV = RHSC->getValue()->getSExtValue();
+        uint64_t ALV = (LV < 0) ? -LV : LV;
+        uint64_t ARV = (RV < 0) ? -RV : RV;
+        if (ALV == ARV)
+          return LV > RV;
+        else
+          return ALV < ARV;
+      }
+      return (LHSC && !RHSC);
+    }
+  };
+}
+
 void LoopStrengthReduce::runOnLoop(Loop *L) {
   // First step, transform all loops nesting inside of this loop.
   for (LoopInfo::iterator I = L->begin(), E = L->end(); I != E; ++I)
@@ -1065,7 +1269,18 @@ void LoopStrengthReduce::runOnLoop(Loop *L) {
 
   // If we only have one stride, we can more aggressively eliminate some things.
   bool HasOneStride = IVUsesByStride.size() == 1;
-  
+
+#ifndef NDEBUG
+  DEBUG(std::cerr << "\nLSR on ");
+  DEBUG(L->dump());
+#endif
+
+  // IVsByStride keeps IVs for one particular loop.
+  IVsByStride.clear();
+
+  // Sort the StrideOrder so we process larger strides first.
+  std::stable_sort(StrideOrder.begin(), StrideOrder.end(), StrideCompare());
+
   // Note: this processes each stride/type pair individually.  All users passed
   // into StrengthReduceStridedIVUsers have the same type AND stride.  Also,
   // node that we iterate over IVUsesByStride indirectly by using StrideOrder.
