@@ -23,6 +23,7 @@
 #include <functional>
 #include <set>
 #include <map>
+#include <iostream>
 using namespace llvm;
 
 /// SafeToMergeTerminators - Return true if it is safe to merge these two
@@ -82,7 +83,7 @@ static bool CanPropagatePredecessorsForPHIs(BasicBlock *BB, BasicBlock *Succ) {
   //
   if (isa<PHINode>(Succ->front())) {
     std::set<BasicBlock*> BBPreds(pred_begin(BB), pred_end(BB));
-    for (pred_iterator PI = pred_begin(Succ), PE = pred_end(Succ);\
+    for (pred_iterator PI = pred_begin(Succ), PE = pred_end(Succ);
          PI != PE; ++PI)
       if (std::find(BBPreds.begin(), BBPreds.end(), *PI) != BBPreds.end()) {
         // Loop over all of the PHI nodes checking to see if there are
@@ -115,19 +116,25 @@ static bool CanPropagatePredecessorsForPHIs(BasicBlock *BB, BasicBlock *Succ) {
     }
   if (IsSafe) return true;
   
-  // If the PHI nodes in BB are only used by instructions in Succ, we are ok.
-  IsSafe = true;
-  for (BasicBlock::iterator I = BB->begin(); isa<PHINode>(I) && IsSafe; ++I) {
+  // If the PHI nodes in BB are only used by instructions in Succ, we are ok if
+  // BB and Succ have no common predecessors.
+  for (BasicBlock::iterator I = BB->begin(); isa<PHINode>(I); ++I) {
     PHINode *PN = cast<PHINode>(I);
     for (Value::use_iterator UI = PN->use_begin(), E = PN->use_end(); UI != E;
          ++UI)
-      if (cast<Instruction>(*UI)->getParent() != Succ) {
-        IsSafe = false;
-        break;
-      }
+      if (cast<Instruction>(*UI)->getParent() != Succ)
+        return false;
   }
   
-  return IsSafe;
+  // Scan the predecessor sets of BB and Succ, making sure there are no common
+  // predecessors.  Common predecessors would cause us to build a phi node with
+  // differing incoming values, which is not legal.
+  std::set<BasicBlock*> BBPreds(pred_begin(BB), pred_end(BB));
+  for (pred_iterator PI = pred_begin(Succ), E = pred_end(Succ); PI != E; ++PI)
+    if (BBPreds.count(*PI))
+      return false;
+    
+  return true;
 }
 
 /// TryToSimplifyUncondBranchFromEmptyBlock - BB contains an unconditional
@@ -894,8 +901,6 @@ HoistTerminator:
 /// across this block.
 static bool BlockIsSimpleEnoughToThreadThrough(BasicBlock *BB) {
   BranchInst *BI = cast<BranchInst>(BB->getTerminator());
-  Value *Cond = BI->getCondition();
-  
   unsigned Size = 0;
   
   // If this basic block contains anything other than a PHI (which controls the
@@ -1419,8 +1424,8 @@ bool llvm::SimplifyCFG(BasicBlock *BB) {
                   PBI->setSuccessor(1, OldTrue);
                 }
 
-                if (PBI->getSuccessor(0) == TrueDest ||
-                    PBI->getSuccessor(1) == FalseDest) {
+                if ((PBI->getSuccessor(0) == TrueDest && FalseDest != BB) ||
+                    (PBI->getSuccessor(1) == FalseDest && TrueDest != BB)) {
                   // Clone Cond into the predecessor basic block, and or/and the
                   // two conditions together.
                   Instruction *New = Cond->clone();
@@ -1505,11 +1510,26 @@ bool llvm::SimplifyCFG(BasicBlock *BB) {
                 PBIOp = BIOp = -1;
               }
               
+              // Check to make sure that the other destination of this branch
+              // isn't BB itself.  If so, this is an infinite loop that will
+              // keep getting unwound.
+              if (PBIOp != -1 && PBI->getSuccessor(PBIOp) == BB)
+                PBIOp = BIOp = -1;
+
               // Finally, if everything is ok, fold the branches to logical ops.
               if (PBIOp != -1) {
                 BasicBlock *CommonDest = PBI->getSuccessor(PBIOp);
                 BasicBlock *OtherDest  = BI->getSuccessor(BIOp ^ 1);
 
+                // If OtherDest *is* BB, then this is a basic block with just
+                // a conditional branch in it, where one edge (OtherDesg) goes
+                // back to the block.  We know that the program doesn't get
+                // stuck in the infinite loop, so the condition must be such
+                // that OtherDest isn't branched through. Forward to CommonDest,
+                // and avoid an infinite loop at optimizer time.
+                if (OtherDest == BB)
+                  OtherDest = CommonDest;
+                
                 DEBUG(std::cerr << "FOLDING BRs:" << *PBI->getParent()
                                 << "AND: " << *BI->getParent());
                                 
