@@ -25,6 +25,8 @@ using namespace llvm;
 static RegisterOpt<UnifyFunctionExitNodes>
 X("mergereturn", "Unify function exit nodes");
 
+int UnifyFunctionExitNodes::stub;
+
 Pass *llvm::createUnifyFunctionExitNodesPass() {
   return new UnifyFunctionExitNodes();
 }
@@ -32,6 +34,10 @@ Pass *llvm::createUnifyFunctionExitNodesPass() {
 void UnifyFunctionExitNodes::getAnalysisUsage(AnalysisUsage &AU) const{
   // We preserve the non-critical-edgeness property
   AU.addPreservedID(BreakCriticalEdgesID);
+  // This is a cluster of orthogonal Transforms
+  AU.addPreservedID(PromoteMemoryToRegisterID);
+  AU.addPreservedID(LowerSelectID);
+  AU.addPreservedID(LowerSwitchID);
 }
 
 // UnifyAllExitNodes - Unify all exit nodes of the CFG by creating a new
