@@ -20,6 +20,7 @@
 #include "llvm/Module.h"
 #include "llvm/ADT/StringExtras.h"
 #include "llvm/Support/MathExtras.h"
+#include "llvm/Support/Visibility.h"
 #include <algorithm>
 #include <iostream>
 using namespace llvm;
@@ -308,12 +309,14 @@ ConstantPacked::~ConstantPacked() {
 
 /// UnaryConstantExpr - This class is private to Constants.cpp, and is used
 /// behind the scenes to implement unary constant exprs.
-class UnaryConstantExpr : public ConstantExpr {
+namespace {
+class VISIBILITY_HIDDEN UnaryConstantExpr : public ConstantExpr {
   Use Op;
 public:
   UnaryConstantExpr(unsigned Opcode, Constant *C, const Type *Ty)
     : ConstantExpr(Ty, Opcode, &Op, 1), Op(C, this) {}
 };
+}
 
 static bool isSetCC(unsigned Opcode) {
   return Opcode == Instruction::SetEQ || Opcode == Instruction::SetNE ||
@@ -323,7 +326,8 @@ static bool isSetCC(unsigned Opcode) {
 
 /// BinaryConstantExpr - This class is private to Constants.cpp, and is used
 /// behind the scenes to implement binary constant exprs.
-class BinaryConstantExpr : public ConstantExpr {
+namespace {
+class VISIBILITY_HIDDEN BinaryConstantExpr : public ConstantExpr {
   Use Ops[2];
 public:
   BinaryConstantExpr(unsigned Opcode, Constant *C1, Constant *C2)
@@ -333,10 +337,12 @@ public:
     Ops[1].init(C2, this);
   }
 };
+}
 
 /// SelectConstantExpr - This class is private to Constants.cpp, and is used
 /// behind the scenes to implement select constant exprs.
-class SelectConstantExpr : public ConstantExpr {
+namespace {
+class VISIBILITY_HIDDEN SelectConstantExpr : public ConstantExpr {
   Use Ops[3];
 public:
   SelectConstantExpr(Constant *C1, Constant *C2, Constant *C3)
@@ -346,10 +352,62 @@ public:
     Ops[2].init(C3, this);
   }
 };
+}
+
+/// ExtractElementConstantExpr - This class is private to
+/// Constants.cpp, and is used behind the scenes to implement
+/// extractelement constant exprs.
+namespace {
+class VISIBILITY_HIDDEN ExtractElementConstantExpr : public ConstantExpr {
+  Use Ops[2];
+public:
+  ExtractElementConstantExpr(Constant *C1, Constant *C2)
+    : ConstantExpr(cast<PackedType>(C1->getType())->getElementType(), 
+                   Instruction::ExtractElement, Ops, 2) {
+    Ops[0].init(C1, this);
+    Ops[1].init(C2, this);
+  }
+};
+}
+
+/// InsertElementConstantExpr - This class is private to
+/// Constants.cpp, and is used behind the scenes to implement
+/// insertelement constant exprs.
+namespace {
+class VISIBILITY_HIDDEN InsertElementConstantExpr : public ConstantExpr {
+  Use Ops[3];
+public:
+  InsertElementConstantExpr(Constant *C1, Constant *C2, Constant *C3)
+    : ConstantExpr(C1->getType(), Instruction::InsertElement, 
+                   Ops, 3) {
+    Ops[0].init(C1, this);
+    Ops[1].init(C2, this);
+    Ops[2].init(C3, this);
+  }
+};
+}
+
+/// ShuffleVectorConstantExpr - This class is private to
+/// Constants.cpp, and is used behind the scenes to implement
+/// shufflevector constant exprs.
+namespace {
+class VISIBILITY_HIDDEN ShuffleVectorConstantExpr : public ConstantExpr {
+  Use Ops[3];
+public:
+  ShuffleVectorConstantExpr(Constant *C1, Constant *C2, Constant *C3)
+  : ConstantExpr(C1->getType(), Instruction::ShuffleVector, 
+                 Ops, 3) {
+    Ops[0].init(C1, this);
+    Ops[1].init(C2, this);
+    Ops[2].init(C3, this);
+  }
+};
+}
 
 /// GetElementPtrConstantExpr - This class is private to Constants.cpp, and is
 /// used behind the scenes to implement getelementpr constant exprs.
-struct GetElementPtrConstantExpr : public ConstantExpr {
+namespace {
+struct VISIBILITY_HIDDEN GetElementPtrConstantExpr : public ConstantExpr {
   GetElementPtrConstantExpr(Constant *C, const std::vector<Constant*> &IdxList,
                             const Type *DestTy)
     : ConstantExpr(DestTy, Instruction::GetElementPtr,
@@ -362,6 +420,7 @@ struct GetElementPtrConstantExpr : public ConstantExpr {
     delete [] OperandList;
   }
 };
+}
 
 /// ConstantExpr::get* - Return some common constants without having to
 /// specify the full Instruction::OPCODE identifier.
@@ -438,6 +497,92 @@ Constant *ConstantExpr::getSShr(Constant *C1, Constant *C2) {
                         C1->getType()->getSignedVersion()), C2), C1->getType());
 }
 
+/// getWithOperandReplaced - Return a constant expression identical to this
+/// one, but with the specified operand set to the specified value.
+Constant *ConstantExpr::getWithOperandReplaced(unsigned OpNo,
+                                               Constant *Op) const {
+  assert(OpNo < getNumOperands() && "Operand num is out of range!");
+  assert(Op->getType() == getOperand(OpNo)->getType() &&
+         "Replacing operand with value of different type!");
+  if (getOperand(OpNo) == Op)
+    return const_cast<ConstantExpr*>(this);
+  
+  Constant *Op0, *Op1, *Op2;
+  switch (getOpcode()) {
+  case Instruction::Cast:
+    return ConstantExpr::getCast(Op, getType());
+  case Instruction::Select:
+    Op0 = (OpNo == 0) ? Op : getOperand(0);
+    Op1 = (OpNo == 1) ? Op : getOperand(1);
+    Op2 = (OpNo == 2) ? Op : getOperand(2);
+    return ConstantExpr::getSelect(Op0, Op1, Op2);
+  case Instruction::InsertElement:
+    Op0 = (OpNo == 0) ? Op : getOperand(0);
+    Op1 = (OpNo == 1) ? Op : getOperand(1);
+    Op2 = (OpNo == 2) ? Op : getOperand(2);
+    return ConstantExpr::getInsertElement(Op0, Op1, Op2);
+  case Instruction::ExtractElement:
+    Op0 = (OpNo == 0) ? Op : getOperand(0);
+    Op1 = (OpNo == 1) ? Op : getOperand(1);
+    return ConstantExpr::getExtractElement(Op0, Op1);
+  case Instruction::ShuffleVector:
+    Op0 = (OpNo == 0) ? Op : getOperand(0);
+    Op1 = (OpNo == 1) ? Op : getOperand(1);
+    Op2 = (OpNo == 2) ? Op : getOperand(2);
+    return ConstantExpr::getShuffleVector(Op0, Op1, Op2);
+  case Instruction::GetElementPtr: {
+    std::vector<Constant*> Ops;
+    for (unsigned i = 1, e = getNumOperands(); i != e; ++i)
+      Ops.push_back(getOperand(i));
+    if (OpNo == 0)
+      return ConstantExpr::getGetElementPtr(Op, Ops);
+    Ops[OpNo-1] = Op;
+    return ConstantExpr::getGetElementPtr(getOperand(0), Ops);
+  }
+  default:
+    assert(getNumOperands() == 2 && "Must be binary operator?");
+    Op0 = (OpNo == 0) ? Op : getOperand(0);
+    Op1 = (OpNo == 1) ? Op : getOperand(1);
+    return ConstantExpr::get(getOpcode(), Op0, Op1);
+  }
+}
+
+/// getWithOperands - This returns the current constant expression with the
+/// operands replaced with the specified values.  The specified operands must
+/// match count and type with the existing ones.
+Constant *ConstantExpr::
+getWithOperands(const std::vector<Constant*> &Ops) const {
+  assert(Ops.size() == getNumOperands() && "Operand count mismatch!");
+  bool AnyChange = false;
+  for (unsigned i = 0, e = Ops.size(); i != e; ++i) {
+    assert(Ops[i]->getType() == getOperand(i)->getType() &&
+           "Operand type mismatch!");
+    AnyChange |= Ops[i] != getOperand(i);
+  }
+  if (!AnyChange)  // No operands changed, return self.
+    return const_cast<ConstantExpr*>(this);
+
+  switch (getOpcode()) {
+  case Instruction::Cast:
+    return ConstantExpr::getCast(Ops[0], getType());
+  case Instruction::Select:
+    return ConstantExpr::getSelect(Ops[0], Ops[1], Ops[2]);
+  case Instruction::InsertElement:
+    return ConstantExpr::getInsertElement(Ops[0], Ops[1], Ops[2]);
+  case Instruction::ExtractElement:
+    return ConstantExpr::getExtractElement(Ops[0], Ops[1]);
+  case Instruction::ShuffleVector:
+    return ConstantExpr::getShuffleVector(Ops[0], Ops[1], Ops[2]);
+  case Instruction::GetElementPtr: {
+    std::vector<Constant*> ActualOps(Ops.begin()+1, Ops.end());
+    return ConstantExpr::getGetElementPtr(Ops[0], ActualOps);
+  }
+  default:
+    assert(getNumOperands() == 2 && "Must be binary operator?");
+    return ConstantExpr::get(getOpcode(), Ops[0], Ops[1]);
+  }
+}
+
 
 //===----------------------------------------------------------------------===//
 //                      isValueValidForType implementations
@@ -485,7 +630,7 @@ bool ConstantFP::isValueValidForType(const Type *Ty, double Val) {
   case Type::DoubleTyID:
     return true;          // This is the largest type...
   }
-};
+}
 
 //===----------------------------------------------------------------------===//
 //                      Factory Function Implementation
@@ -497,29 +642,28 @@ bool ConstantFP::isValueValidForType(const Type *Ty, double Val) {
 //
 namespace llvm {
   template<class ConstantClass, class TypeClass, class ValType>
-  struct ConstantCreator {
+  struct VISIBILITY_HIDDEN ConstantCreator {
     static ConstantClass *create(const TypeClass *Ty, const ValType &V) {
       return new ConstantClass(Ty, V);
     }
   };
 
   template<class ConstantClass, class TypeClass>
-  struct ConvertConstantType {
+  struct VISIBILITY_HIDDEN ConvertConstantType {
     static void convert(ConstantClass *OldC, const TypeClass *NewTy) {
       assert(0 && "This type cannot be converted!\n");
       abort();
     }
   };
-}
 
-namespace {
   template<class ValType, class TypeClass, class ConstantClass,
            bool HasLargeKey = false  /*true for arrays and structs*/ >
-  class ValueMap : public AbstractTypeUser {
+  class VISIBILITY_HIDDEN ValueMap : public AbstractTypeUser {
   public:
-    typedef std::pair<const TypeClass*, ValType> MapKey;
-    typedef std::map<MapKey, ConstantClass *> MapTy;
-    typedef typename MapTy::iterator MapIterator;
+    typedef std::pair<const Type*, ValType> MapKey;
+    typedef std::map<MapKey, Constant *> MapTy;
+    typedef std::map<Constant*, typename MapTy::iterator> InverseMapTy;
+    typedef std::map<const Type*, typename MapTy::iterator> AbstractTypeMapTy;
   private:
     /// Map - This is the main map from the element descriptor to the Constants.
     /// This is the primary way we avoid creating two of the same shape
@@ -530,15 +674,16 @@ namespace {
     /// from the constants to their element in Map.  This is important for
     /// removal of constants from the array, which would otherwise have to scan
     /// through the map with very large keys.
-    std::map<ConstantClass*, MapIterator> InverseMap;
+    InverseMapTy InverseMap;
 
-    typedef std::map<const TypeClass*, MapIterator> AbstractTypeMapTy;
+    /// AbstractTypeMap - Map for abstract type constants.
+    ///
     AbstractTypeMapTy AbstractTypeMap;
 
     friend void Constant::clearAllValueMaps();
   private:
     void clear(std::vector<Constant *> &Constants) {
-      for(MapIterator I = Map.begin(); I != Map.end(); ++I)
+      for(typename MapTy::iterator I = Map.begin(); I != Map.end(); ++I)
         Constants.push_back(I->second);
       Map.clear();
       AbstractTypeMap.clear();
@@ -546,32 +691,32 @@ namespace {
     }
 
   public:
-    MapIterator map_end() { return Map.end(); }
+    typename MapTy::iterator map_end() { return Map.end(); }
     
     /// InsertOrGetItem - Return an iterator for the specified element.
     /// If the element exists in the map, the returned iterator points to the
     /// entry and Exists=true.  If not, the iterator points to the newly
     /// inserted entry and returns Exists=false.  Newly inserted entries have
     /// I->second == 0, and should be filled in.
-    MapIterator InsertOrGetItem(std::pair<MapKey, ConstantClass *> &InsertVal,
+    typename MapTy::iterator InsertOrGetItem(std::pair<MapKey, Constant *>
+                                   &InsertVal,
                                    bool &Exists) {
-      std::pair<MapIterator, bool> IP = Map.insert(InsertVal);
+      std::pair<typename MapTy::iterator, bool> IP = Map.insert(InsertVal);
       Exists = !IP.second;
       return IP.first;
     }
     
 private:
-    MapIterator FindExistingElement(ConstantClass *CP) {
+    typename MapTy::iterator FindExistingElement(ConstantClass *CP) {
       if (HasLargeKey) {
-        typename std::map<ConstantClass*, MapIterator>::iterator
-            IMI = InverseMap.find(CP);
+        typename InverseMapTy::iterator IMI = InverseMap.find(CP);
         assert(IMI != InverseMap.end() && IMI->second != Map.end() &&
                IMI->second->second == CP &&
                "InverseMap corrupt!");
         return IMI->second;
       }
       
-      MapIterator I =
+      typename MapTy::iterator I =
         Map.find(MapKey((TypeClass*)CP->getRawType(), getValType(CP)));
       if (I == Map.end() || I->second != CP) {
         // FIXME: This should not use a linear scan.  If this gets to be a
@@ -587,9 +732,9 @@ public:
     /// necessary.
     ConstantClass *getOrCreate(const TypeClass *Ty, const ValType &V) {
       MapKey Lookup(Ty, V);
-      MapIterator I = Map.lower_bound(Lookup);
+      typename MapTy::iterator I = Map.lower_bound(Lookup);
       if (I != Map.end() && I->first == Lookup)
-        return I->second;  // Is it in the map?
+        return static_cast<ConstantClass *>(I->second);  // Is it in the map?
 
       // If no preexisting value, create one now...
       ConstantClass *Result =
@@ -619,7 +764,7 @@ public:
     }
 
     void remove(ConstantClass *CP) {
-      MapIterator I = FindExistingElement(CP);
+      typename MapTy::iterator I = FindExistingElement(CP);
       assert(I != Map.end() && "Constant not found in constant table!");
       assert(I->second == CP && "Didn't find correct element?");
 
@@ -628,15 +773,15 @@ public:
       
       // Now that we found the entry, make sure this isn't the entry that
       // the AbstractTypeMap points to.
-      const TypeClass *Ty = I->first.first;
+      const TypeClass *Ty = static_cast<const TypeClass *>(I->first.first);
       if (Ty->isAbstract()) {
         assert(AbstractTypeMap.count(Ty) &&
                "Abstract type not in AbstractTypeMap?");
-        MapIterator &ATMEntryIt = AbstractTypeMap[Ty];
+        typename MapTy::iterator &ATMEntryIt = AbstractTypeMap[Ty];
         if (ATMEntryIt == I) {
           // Yes, we are removing the representative entry for this type.
           // See if there are any other entries of the same type.
-          MapIterator TmpIt = ATMEntryIt;
+          typename MapTy::iterator TmpIt = ATMEntryIt;
 
           // First check the entry before this one...
           if (TmpIt != Map.begin()) {
@@ -672,9 +817,9 @@ public:
     /// MoveConstantToNewSlot - If we are about to change C to be the element
     /// specified by I, update our internal data structures to reflect this
     /// fact.
-    void MoveConstantToNewSlot(ConstantClass *C, MapIterator I) {
+    void MoveConstantToNewSlot(ConstantClass *C, typename MapTy::iterator I) {
       // First, remove the old location of the specified constant in the map.
-      MapIterator OldI = FindExistingElement(C);
+      typename MapTy::iterator OldI = FindExistingElement(C);
       assert(OldI != Map.end() && "Constant not found in constant table!");
       assert(OldI->second == C && "Didn't find correct element?");
       
@@ -702,7 +847,7 @@ public:
     
     void refineAbstractType(const DerivedType *OldTy, const Type *NewTy) {
       typename AbstractTypeMapTy::iterator I =
-        AbstractTypeMap.find(cast<TypeClass>(OldTy));
+        AbstractTypeMap.find(cast<Type>(OldTy));
 
       assert(I != AbstractTypeMap.end() &&
              "Abstract type not in AbstractTypeMap?");
@@ -712,10 +857,11 @@ public:
       // eliminated eventually.
       do {
         ConvertConstantType<ConstantClass,
-                            TypeClass>::convert(I->second->second,
+                            TypeClass>::convert(
+                                static_cast<ConstantClass *>(I->second->second),
                                                 cast<TypeClass>(NewTy));
 
-        I = AbstractTypeMap.find(cast<TypeClass>(OldTy));
+        I = AbstractTypeMap.find(cast<Type>(OldTy));
       } while (I != AbstractTypeMap.end());
     }
 
@@ -819,6 +965,8 @@ static ValueMap<char, Type, ConstantAggregateZero> AggZeroConstants;
 static char getValType(ConstantAggregateZero *CPZ) { return 0; }
 
 Constant *ConstantAggregateZero::get(const Type *Ty) {
+  assert((isa<StructType>(Ty) || isa<ArrayType>(Ty) || isa<PackedType>(Ty)) &&
+         "Cannot create an aggregate zero of non-aggregate type!");
   return AggZeroConstants.getOrCreate(Ty, 0);
 }
 
@@ -880,20 +1028,23 @@ void ConstantArray::destroyConstant() {
   destroyConstantImpl();
 }
 
-// ConstantArray::get(const string&) - Return an array that is initialized to
-// contain the specified string.  A null terminator is added to the specified
-// string so that it may be used in a natural way...
-//
-Constant *ConstantArray::get(const std::string &Str) {
+/// ConstantArray::get(const string&) - Return an array that is initialized to
+/// contain the specified string.  If length is zero then a null terminator is 
+/// added to the specified string so that it may be used in a natural way. 
+/// Otherwise, the length parameter specifies how much of the string to use 
+/// and it won't be null terminated.
+///
+Constant *ConstantArray::get(const std::string &Str, bool AddNull) {
   std::vector<Constant*> ElementVals;
-
   for (unsigned i = 0; i < Str.length(); ++i)
     ElementVals.push_back(ConstantSInt::get(Type::SByteTy, Str[i]));
 
   // Add a null terminator to the string...
-  ElementVals.push_back(ConstantSInt::get(Type::SByteTy, 0));
+  if (AddNull) {
+    ElementVals.push_back(ConstantSInt::get(Type::SByteTy, 0));
+  }
 
-  ArrayType *ATy = ArrayType::get(Type::SByteTy, Str.length()+1);
+  ArrayType *ATy = ArrayType::get(Type::SByteTy, ElementVals.size());
   return ConstantArray::get(ATy, ElementVals);
 }
 
@@ -1141,7 +1292,15 @@ namespace llvm {
         return new BinaryConstantExpr(V.first, V.second[0], V.second[1]);
       if (V.first == Instruction::Select)
         return new SelectConstantExpr(V.second[0], V.second[1], V.second[2]);
-
+      if (V.first == Instruction::ExtractElement)
+        return new ExtractElementConstantExpr(V.second[0], V.second[1]);
+      if (V.first == Instruction::InsertElement)
+        return new InsertElementConstantExpr(V.second[0], V.second[1],
+                                             V.second[2]);
+      if (V.first == Instruction::ShuffleVector)
+        return new ShuffleVectorConstantExpr(V.second[0], V.second[1],
+                                             V.second[2]);
+      
       assert(V.first == Instruction::GetElementPtr && "Invalid ConstantExpr!");
 
       std::vector<Constant*> IdxList(V.second.begin()+1, V.second.end());
@@ -1277,14 +1436,15 @@ Constant *ConstantExpr::get(unsigned Opcode, Constant *C1, Constant *C2) {
   case Instruction::Mul: case Instruction::Div:
   case Instruction::Rem:
     assert(C1->getType() == C2->getType() && "Op types should be identical!");
-    assert((C1->getType()->isInteger() || C1->getType()->isFloatingPoint()) &&
+    assert((C1->getType()->isInteger() || C1->getType()->isFloatingPoint() ||
+            isa<PackedType>(C1->getType())) &&
            "Tried to create an arithmetic operation on a non-arithmetic type!");
     break;
   case Instruction::And:
   case Instruction::Or:
   case Instruction::Xor:
     assert(C1->getType() == C2->getType() && "Op types should be identical!");
-    assert(C1->getType()->isIntegral() &&
+    assert((C1->getType()->isIntegral() || isa<PackedType>(C1->getType())) &&
            "Tried to create a logical operation on a non-integral type!");
     break;
   case Instruction::SetLT: case Instruction::SetGT: case Instruction::SetLE:
@@ -1294,7 +1454,7 @@ Constant *ConstantExpr::get(unsigned Opcode, Constant *C1, Constant *C2) {
   case Instruction::Shl:
   case Instruction::Shr:
     assert(C2->getType() == Type::UByteTy && "Shift should be by ubyte!");
-    assert(C1->getType()->isInteger() &&
+    assert((C1->getType()->isInteger() || isa<PackedType>(C1->getType())) &&
            "Tried to create a shift operation on a non-integer type!");
     break;
   default:
@@ -1385,6 +1545,69 @@ Constant *ConstantExpr::getGetElementPtr(Constant *C,
   return getGetElementPtrTy(PointerType::get(Ty), C, IdxList);
 }
 
+Constant *ConstantExpr::getExtractElementTy(const Type *ReqTy, Constant *Val,
+                                            Constant *Idx) {
+  if (Constant *FC = ConstantFoldExtractElementInstruction(Val, Idx))
+    return FC;          // Fold a few common cases...
+  // Look up the constant in the table first to ensure uniqueness
+  std::vector<Constant*> ArgVec(1, Val);
+  ArgVec.push_back(Idx);
+  const ExprMapKeyType &Key = std::make_pair(Instruction::ExtractElement,ArgVec);
+  return ExprConstants.getOrCreate(ReqTy, Key);
+}
+
+Constant *ConstantExpr::getExtractElement(Constant *Val, Constant *Idx) {
+  assert(isa<PackedType>(Val->getType()) &&
+         "Tried to create extractelement operation on non-packed type!");
+  assert(Idx->getType() == Type::UIntTy &&
+         "Extractelement index must be uint type!");
+  return getExtractElementTy(cast<PackedType>(Val->getType())->getElementType(),
+                             Val, Idx);
+}
+
+Constant *ConstantExpr::getInsertElementTy(const Type *ReqTy, Constant *Val,
+                                           Constant *Elt, Constant *Idx) {
+  if (Constant *FC = ConstantFoldInsertElementInstruction(Val, Elt, Idx))
+    return FC;          // Fold a few common cases...
+  // Look up the constant in the table first to ensure uniqueness
+  std::vector<Constant*> ArgVec(1, Val);
+  ArgVec.push_back(Elt);
+  ArgVec.push_back(Idx);
+  const ExprMapKeyType &Key = std::make_pair(Instruction::InsertElement,ArgVec);
+  return ExprConstants.getOrCreate(ReqTy, Key);
+}
+
+Constant *ConstantExpr::getInsertElement(Constant *Val, Constant *Elt, 
+                                         Constant *Idx) {
+  assert(isa<PackedType>(Val->getType()) &&
+         "Tried to create insertelement operation on non-packed type!");
+  assert(Elt->getType() == cast<PackedType>(Val->getType())->getElementType()
+         && "Insertelement types must match!");
+  assert(Idx->getType() == Type::UIntTy &&
+         "Insertelement index must be uint type!");
+  return getInsertElementTy(cast<PackedType>(Val->getType())->getElementType(),
+                            Val, Elt, Idx);
+}
+
+Constant *ConstantExpr::getShuffleVectorTy(const Type *ReqTy, Constant *V1,
+                                           Constant *V2, Constant *Mask) {
+  if (Constant *FC = ConstantFoldShuffleVectorInstruction(V1, V2, Mask))
+    return FC;          // Fold a few common cases...
+  // Look up the constant in the table first to ensure uniqueness
+  std::vector<Constant*> ArgVec(1, V1);
+  ArgVec.push_back(V2);
+  ArgVec.push_back(Mask);
+  const ExprMapKeyType &Key = std::make_pair(Instruction::ShuffleVector,ArgVec);
+  return ExprConstants.getOrCreate(ReqTy, Key);
+}
+
+Constant *ConstantExpr::getShuffleVector(Constant *V1, Constant *V2, 
+                                         Constant *Mask) {
+  assert(ShuffleVectorInst::isValidOperands(V1, V2, Mask) &&
+         "Invalid shuffle vector constant expr operands!");
+  return getShuffleVectorTy(V1->getType(), V1, V2, Mask);
+}
+
 
 // destroyConstant - Remove the constant from the constant table...
 //
@@ -1408,7 +1631,7 @@ void ConstantArray::replaceUsesOfWithOnConstant(Value *From, Value *To,
   unsigned OperandToUpdate = U-OperandList;
   assert(getOperand(OperandToUpdate) == From && "ReplaceAllUsesWith broken!");
 
-  std::pair<ArrayConstantsTy::MapKey, ConstantArray*> Lookup;
+  std::pair<ArrayConstantsTy::MapKey, Constant*> Lookup;
   Lookup.first.first = getType();
   Lookup.second = this;
 
@@ -1437,7 +1660,7 @@ void ConstantArray::replaceUsesOfWithOnConstant(Value *From, Value *To,
   } else {
     // Check to see if we have this array type already.
     bool Exists;
-    ArrayConstantsTy::MapIterator I =
+    ArrayConstantsTy::MapTy::iterator I =
       ArrayConstants.InsertOrGetItem(Lookup, Exists);
     
     if (Exists) {
@@ -1473,7 +1696,7 @@ void ConstantStruct::replaceUsesOfWithOnConstant(Value *From, Value *To,
   unsigned OperandToUpdate = U-OperandList;
   assert(getOperand(OperandToUpdate) == From && "ReplaceAllUsesWith broken!");
 
-  std::pair<StructConstantsTy::MapKey, ConstantStruct*> Lookup;
+  std::pair<StructConstantsTy::MapKey, Constant*> Lookup;
   Lookup.first.first = getType();
   Lookup.second = this;
   std::vector<Constant*> &Values = Lookup.first.second;
@@ -1502,7 +1725,7 @@ void ConstantStruct::replaceUsesOfWithOnConstant(Value *From, Value *To,
   } else {
     // Check to see if we have this array type already.
     bool Exists;
-    StructConstantsTy::MapIterator I =
+    StructConstantsTy::MapTy::iterator I =
       StructConstants.InsertOrGetItem(Lookup, Exists);
     
     if (Exists) {
@@ -1580,6 +1803,28 @@ void ConstantExpr::replaceUsesOfWithOnConstant(Value *From, Value *ToV,
     if (C2 == From) C2 = To;
     if (C3 == From) C3 = To;
     Replacement = ConstantExpr::getSelect(C1, C2, C3);
+  } else if (getOpcode() == Instruction::ExtractElement) {
+    Constant *C1 = getOperand(0);
+    Constant *C2 = getOperand(1);
+    if (C1 == From) C1 = To;
+    if (C2 == From) C2 = To;
+    Replacement = ConstantExpr::getExtractElement(C1, C2);
+  } else if (getOpcode() == Instruction::InsertElement) {
+    Constant *C1 = getOperand(0);
+    Constant *C2 = getOperand(1);
+    Constant *C3 = getOperand(1);
+    if (C1 == From) C1 = To;
+    if (C2 == From) C2 = To;
+    if (C3 == From) C3 = To;
+    Replacement = ConstantExpr::getInsertElement(C1, C2, C3);
+  } else if (getOpcode() == Instruction::ShuffleVector) {
+    Constant *C1 = getOperand(0);
+    Constant *C2 = getOperand(1);
+    Constant *C3 = getOperand(2);
+    if (C1 == From) C1 = To;
+    if (C2 == From) C2 = To;
+    if (C3 == From) C3 = To;
+    Replacement = ConstantExpr::getShuffleVector(C1, C2, C3);
   } else if (getNumOperands() == 2) {
     Constant *C1 = getOperand(0);
     Constant *C2 = getOperand(1);
@@ -1628,3 +1873,47 @@ void Constant::clearAllValueMaps() {
     (*I)->destroyConstantImpl();
   Constants.clear();
 }
+
+/// getStringValue - Turn an LLVM constant pointer that eventually points to a
+/// global into a string value.  Return an empty string if we can't do it.
+/// Parameter Chop determines if the result is chopped at the first null
+/// terminator.
+///
+std::string Constant::getStringValue(bool Chop, unsigned Offset) {
+  if (GlobalVariable *GV = dyn_cast<GlobalVariable>(this)) {
+    if (GV->hasInitializer() && isa<ConstantArray>(GV->getInitializer())) {
+      ConstantArray *Init = cast<ConstantArray>(GV->getInitializer());
+      if (Init->isString()) {
+        std::string Result = Init->getAsString();
+        if (Offset < Result.size()) {
+          // If we are pointing INTO The string, erase the beginning...
+          Result.erase(Result.begin(), Result.begin()+Offset);
+
+          // Take off the null terminator, and any string fragments after it.
+          if (Chop) {
+            std::string::size_type NullPos = Result.find_first_of((char)0);
+            if (NullPos != std::string::npos)
+              Result.erase(Result.begin()+NullPos, Result.end());
+          }
+          return Result;
+        }
+      }
+    }
+  } else if (Constant *C = dyn_cast<Constant>(this)) {
+    if (GlobalValue *GV = dyn_cast<GlobalValue>(C))
+      return GV->getStringValue(Chop, Offset);
+    else if (ConstantExpr *CE = dyn_cast<ConstantExpr>(C)) {
+      if (CE->getOpcode() == Instruction::GetElementPtr) {
+        // Turn a gep into the specified offset.
+        if (CE->getNumOperands() == 3 &&
+            cast<Constant>(CE->getOperand(1))->isNullValue() &&
+            isa<ConstantInt>(CE->getOperand(2))) {
+          Offset += cast<ConstantInt>(CE->getOperand(2))->getRawValue();
+          return CE->getOperand(0)->getStringValue(Chop, Offset);
+        }
+      }
+    }
+  }
+  return "";
+}
+
