@@ -200,73 +200,30 @@ void Function::dropAllReferences() {
 /// llvm/Intrinsics.h.
 ///
 unsigned Function::getIntrinsicID() const {
-  if (getName().size() < 5 || getName()[4] != '.' || getName()[0] != 'l' ||
-      getName()[1] != 'l' || getName()[2] != 'v' || getName()[3] != 'm')
+  const std::string& Name = this->getName();
+  if (Name.size() < 5 || Name[4] != '.' || Name[0] != 'l' || Name[1] != 'l'
+      || Name[2] != 'v' || Name[3] != 'm')
     return 0;  // All intrinsics start with 'llvm.'
 
-  assert(getName().size() != 5 && "'llvm.' is an invalid intrinsic name!");
+  assert(Name.size() != 5 && "'llvm.' is an invalid intrinsic name!");
 
-  switch (getName()[5]) {
-  case 'c':
-    if (getName() == "llvm.ctpop") return Intrinsic::ctpop;
-    if (getName() == "llvm.cttz") return Intrinsic::cttz;
-    if (getName() == "llvm.ctlz") return Intrinsic::ctlz;
-    break;
-  case 'd':
-    if (getName() == "llvm.dbg.stoppoint")   return Intrinsic::dbg_stoppoint;
-    if (getName() == "llvm.dbg.region.start")return Intrinsic::dbg_region_start;
-    if (getName() == "llvm.dbg.region.end")  return Intrinsic::dbg_region_end;
-    if (getName() == "llvm.dbg.func.start")  return Intrinsic::dbg_func_start;
-    if (getName() == "llvm.dbg.declare")     return Intrinsic::dbg_declare;
-    break;
-  case 'f':
-    if (getName() == "llvm.frameaddress")  return Intrinsic::frameaddress;
-    break;
-  case 'g':
-    if (getName() == "llvm.gcwrite") return Intrinsic::gcwrite;
-    if (getName() == "llvm.gcread")  return Intrinsic::gcread;
-    if (getName() == "llvm.gcroot")  return Intrinsic::gcroot;
-    break;
-  case 'i':
-    if (getName() == "llvm.isunordered") return Intrinsic::isunordered;
-    break;
-  case 'l':
-    if (getName() == "llvm.longjmp")  return Intrinsic::longjmp;
-    break;
-  case 'm':
-    if (getName() == "llvm.memcpy")  return Intrinsic::memcpy;
-    if (getName() == "llvm.memmove")  return Intrinsic::memmove;
-    if (getName() == "llvm.memset")  return Intrinsic::memset;
-    break;
-  case 'p':
-    if (getName() == "llvm.prefetch")  return Intrinsic::prefetch;
-    if (getName() == "llvm.pcmarker")  return Intrinsic::pcmarker;
-    break;
-  case 'r':
-    if (getName() == "llvm.returnaddress")  return Intrinsic::returnaddress;
-    if (getName() == "llvm.readport")       return Intrinsic::readport;
-    if (getName() == "llvm.readio")         return Intrinsic::readio;
-    break;
-  case 's':
-    if (getName() == "llvm.setjmp")     return Intrinsic::setjmp;
-    if (getName() == "llvm.sigsetjmp")  return Intrinsic::sigsetjmp;
-    if (getName() == "llvm.siglongjmp") return Intrinsic::siglongjmp;
-    if (getName() == "llvm.sqrt")       return Intrinsic::sqrt;
-    break;
-  case 'v':
-    if (getName() == "llvm.va_copy")  return Intrinsic::vacopy;
-    if (getName() == "llvm.va_end")   return Intrinsic::vaend;
-    if (getName() == "llvm.va_start") return Intrinsic::vastart;
-  case 'w':
-    if (getName() == "llvm.writeport") return Intrinsic::writeport;
-    if (getName() == "llvm.writeio")   return Intrinsic::writeio;
-    break;
-  }
-  // The "llvm." namespace is reserved!
-  assert(0 && "Unknown LLVM intrinsic function!");
+#define GET_FUNCTION_RECOGNIZER
+#include "llvm/Intrinsics.gen"
+#undef GET_FUNCTION_RECOGNIZER
   return 0;
 }
 
+const char *Intrinsic::getName(ID id) {
+  assert(id < num_intrinsics && "Invalid intrinsic ID!");
+  const char * const Table[] = {
+    "not_intrinsic",
+#define GET_INTRINSIC_NAME_TABLE
+#include "llvm/Intrinsics.gen"
+#undef GET_INTRINSIC_NAME_TABLE
+  };
+  return Table[id];
+}
+
 Value *IntrinsicInst::StripPointerCasts(Value *Ptr) {
   if (ConstantExpr *CE = dyn_cast<ConstantExpr>(Ptr)) {
     if (CE->getOpcode() == Instruction::Cast) {
