@@ -16,10 +16,12 @@
 #include "llvm/Constants.h"
 #include "llvm/DerivedTypes.h"
 #include "llvm/ADT/STLExtras.h"
+#include "llvm/ADT/StringExtras.h"
 #include "llvm/Support/LeakDetector.h"
 #include "SymbolTableListTraitsImpl.h"
 #include <algorithm>
 #include <cstdarg>
+#include <cstdlib>
 #include <iostream>
 #include <map>
 using namespace llvm;
@@ -52,7 +54,7 @@ iplist<GlobalVariable> &ilist_traits<GlobalVariable>::getList(Module *M) {
 }
 
 // Explicit instantiations of SymbolTableListTraits since some of the methods
-// are not in the public header file...
+// are not in the public header file.
 template class SymbolTableListTraits<GlobalVariable, Module, Module>;
 template class SymbolTableListTraits<Function, Module, Module>;
 
@@ -61,7 +63,7 @@ template class SymbolTableListTraits<Function, Module, Module>;
 //
 
 Module::Module(const std::string &MID)
-  : ModuleID(MID), Endian(AnyEndianness), PtrSize(AnyPointerSize) {
+  : ModuleID(MID), DataLayout("") {
   FunctionList.setItemParent(this);
   FunctionList.setParent(this);
   GlobalList.setItemParent(this);
@@ -84,6 +86,65 @@ void Module::dump() const {
   print(std::cerr);
 }
 
+/// Target endian information...
+Module::Endianness Module::getEndianness() const {
+  std::string temp = DataLayout;
+  Module::Endianness ret = AnyEndianness;
+  
+  while (!temp.empty()) {
+    std::string token = getToken(temp, "-");
+    
+    if (token[0] == 'e') {
+      ret = LittleEndian;
+    } else if (token[0] == 'E') {
+      ret = BigEndian;
+    }
+  }
+  
+  return ret;
+}
+
+void Module::setEndianness(Endianness E) {
+  if (!DataLayout.empty() && E != AnyEndianness)
+    DataLayout += "-";
+  
+  if (E == LittleEndian)
+    DataLayout += "e";
+  else if (E == BigEndian)
+    DataLayout += "E";
+}
+
+/// Target Pointer Size information...
+Module::PointerSize Module::getPointerSize() const {
+  std::string temp = DataLayout;
+  Module::PointerSize ret = AnyPointerSize;
+  
+  while (!temp.empty()) {
+    std::string token = getToken(temp, "-");
+    char signal = getToken(token, ":")[0];
+    
+    if (signal == 'p') {
+      int size = atoi(getToken(token, ":").c_str());
+      if (size == 32)
+        ret = Pointer32;
+      else if (size == 64)
+        ret = Pointer64;
+    }
+  }
+  
+  return ret;
+}
+
+void Module::setPointerSize(PointerSize PS) {
+  if (!DataLayout.empty() && PS != AnyPointerSize)
+    DataLayout += "-";
+  
+  if (PS == Pointer32)
+    DataLayout += "p:32:32";
+  else if (PS == Pointer64)
+    DataLayout += "p:64:64";
+}
+
 //===----------------------------------------------------------------------===//
 // Methods for easy access to the functions in the module.
 //
@@ -189,16 +250,16 @@ Function *Module::getMainFunction() {
 /// specified name, of arbitrary type.  This method returns null if a function
 /// with the specified name is not found.
 ///
-Function *Module::getNamedFunction(const std::string &Name) {
+Function *Module::getNamedFunction(const std::string &Name) const {
   // Loop over all of the functions, looking for the function desired
-  Function *Found = 0;
-  for (iterator I = begin(), E = end(); I != E; ++I)
+  const Function *Found = 0;
+  for (const_iterator I = begin(), E = end(); I != E; ++I)
     if (I->getName() == Name)
       if (I->isExternal())
         Found = I;
       else
-        return I;
-  return Found; // Non-external function not found...
+        return const_cast<Function*>(&(*I));
+  return const_cast<Function*>(Found); // Non-external function not found...
 }
 
 //===----------------------------------------------------------------------===//
@@ -206,22 +267,36 @@ Function *Module::getNamedFunction(const std::string &Name) {
 //
 
 /// getGlobalVariable - Look up the specified global variable in the module
-/// symbol table.  If it does not exist, return null.  Note that this only
-/// returns a global variable if it does not have internal linkage.  The type
-/// argument should be the underlying type of the global, ie, it should not
-/// have the top-level PointerType, which represents the address of the
-/// global.
+/// symbol table.  If it does not exist, return null.  The type argument
+/// should be the underlying type of the global, i.e., it should not have
+/// the top-level PointerType, which represents the address of the global.
+/// If AllowInternal is set to true, this function will return types that
+/// have InternalLinkage. By default, these types are not returned.
 ///
 GlobalVariable *Module::getGlobalVariable(const std::string &Name,
-                                          const Type *Ty) {
+                                          const Type *Ty, bool AllowInternal) {
   if (Value *V = getSymbolTable().lookup(PointerType::get(Ty), Name)) {
     GlobalVariable *Result = cast<GlobalVariable>(V);
-    if (!Result->hasInternalLinkage())
+    if (AllowInternal || !Result->hasInternalLinkage())
       return Result;
   }
   return 0;
 }
 
+/// getNamedGlobal - Return the first global variable in the module with the
+/// specified name, of arbitrary type.  This method returns null if a global
+/// with the specified name is not found.
+///
+GlobalVariable *Module::getNamedGlobal(const std::string &Name) const {
+  // FIXME: This would be much faster with a symbol table that doesn't
+  // discriminate based on type!
+  for (const_global_iterator I = global_begin(), E = global_end();
+       I != E; ++I)
+    if (I->getName() == Name) 
+      return const_cast<GlobalVariable*>(&(*I));
+  return 0;
+}
+
 
 
 //===----------------------------------------------------------------------===//
