@@ -23,9 +23,6 @@
 #include <set>
 using namespace llvm;
 
-// IncludeFile - Stub function used to help linking out.
-IncludeFile::IncludeFile(void*) {}
-
 //===----------------------------------------------------------------------===//
 //   AnalysisID Class Implementation
 //
@@ -38,7 +35,7 @@ static std::vector<const PassInfo*> &getCFGOnlyAnalyses() {
 }
 
 void RegisterPassBase::setOnlyUsesCFG() {
-  getCFGOnlyAnalyses().push_back(PIObj);
+  getCFGOnlyAnalyses().push_back(&PIObj);
 }
 
 //===----------------------------------------------------------------------===//
@@ -78,7 +75,7 @@ void AnalysisUsage::setPreservesCFG() {
 // PassManager implementation - The PassManager class is a simple Pimpl class
 // that wraps the PassManagerT template.
 //
-PassManager::PassManager() : PM(new PassManagerT<Module>()) {}
+PassManager::PassManager() : PM(new ModulePassManager()) {}
 PassManager::~PassManager() { delete PM; }
 void PassManager::add(Pass *P) {
   ModulePass *MP = dynamic_cast<ModulePass*>(P);
@@ -93,19 +90,15 @@ bool PassManager::run(Module &M) { return PM->runOnModule(M); }
 // is like PassManager, but only deals in FunctionPasses.
 //
 FunctionPassManager::FunctionPassManager(ModuleProvider *P) :
-  PM(new PassManagerT<Function>()), MP(P) {}
+  PM(new FunctionPassManagerT()), MP(P) {}
 FunctionPassManager::~FunctionPassManager() { delete PM; }
 void FunctionPassManager::add(FunctionPass *P) { PM->add(P); }
 void FunctionPassManager::add(ImmutablePass *IP) { PM->add(IP); }
 bool FunctionPassManager::run(Function &F) {
-  try {
-    MP->materializeFunction(&F);
-  } catch (std::string& errstr) {
+  std::string errstr;
+  if (MP->materializeFunction(&F, &errstr)) {
     std::cerr << "Error reading bytecode file: " << errstr << "\n";
     abort();
-  } catch (...) {
-    std::cerr << "Error reading bytecode file!\n";
-    abort();
   }
   return PM->run(F);
 }
@@ -194,7 +187,7 @@ void PMDebug::PrintAnalysisSetInfo(unsigned Depth, const char *Msg,
 // Pass Implementation
 //
 
-void ModulePass::addToPassManager(PassManagerT<Module> *PM, AnalysisUsage &AU) {
+void ModulePass::addToPassManager(ModulePassManager *PM, AnalysisUsage &AU) {
   PM->addPass(this, AU);
 }
 
@@ -231,7 +224,7 @@ void Pass::dump() const {
 //===----------------------------------------------------------------------===//
 // ImmutablePass Implementation
 //
-void ImmutablePass::addToPassManager(PassManagerT<Module> *PM,
+void ImmutablePass::addToPassManager(ModulePassManager *PM, 
                                      AnalysisUsage &AU) {
   PM->addPass(this, AU);
 }
@@ -264,12 +257,12 @@ bool FunctionPass::run(Function &F) {
   return Changed | doFinalization(*F.getParent());
 }
 
-void FunctionPass::addToPassManager(PassManagerT<Module> *PM,
+void FunctionPass::addToPassManager(ModulePassManager *PM,
                                     AnalysisUsage &AU) {
   PM->addPass(this, AU);
 }
 
-void FunctionPass::addToPassManager(PassManagerT<Function> *PM,
+void FunctionPass::addToPassManager(FunctionPassManagerT *PM,
                                     AnalysisUsage &AU) {
   PM->addPass(this, AU);
 }
@@ -302,12 +295,12 @@ bool BasicBlockPass::runPass(BasicBlock &BB) {
   return Changed;
 }
 
-void BasicBlockPass::addToPassManager(PassManagerT<Function> *PM,
+void BasicBlockPass::addToPassManager(FunctionPassManagerT *PM,
                                       AnalysisUsage &AU) {
   PM->addPass(this, AU);
 }
 
-void BasicBlockPass::addToPassManager(PassManagerT<BasicBlock> *PM,
+void BasicBlockPass::addToPassManager(BasicBlockPassManager *PM,
                                       AnalysisUsage &AU) {
   PM->addPass(this, AU);
 }
@@ -332,26 +325,25 @@ const PassInfo *Pass::lookupPassInfo(const std::type_info &TI) {
   return (I != PassInfoMap->end()) ? I->second : 0;
 }
 
-void RegisterPassBase::registerPass(PassInfo *PI) {
+void RegisterPassBase::registerPass() {
   if (PassInfoMap == 0)
     PassInfoMap = new std::map<TypeInfo, PassInfo*>();
 
-  assert(PassInfoMap->find(PI->getTypeInfo()) == PassInfoMap->end() &&
+  assert(PassInfoMap->find(PIObj.getTypeInfo()) == PassInfoMap->end() &&
          "Pass already registered!");
-  PIObj = PI;
-  PassInfoMap->insert(std::make_pair(TypeInfo(PI->getTypeInfo()), PI));
+  PassInfoMap->insert(std::make_pair(TypeInfo(PIObj.getTypeInfo()), &PIObj));
 
   // Notify any listeners...
   if (Listeners)
     for (std::vector<PassRegistrationListener*>::iterator
            I = Listeners->begin(), E = Listeners->end(); I != E; ++I)
-      (*I)->passRegistered(PI);
+      (*I)->passRegistered(&PIObj);
 }
 
-void RegisterPassBase::unregisterPass(PassInfo *PI) {
+void RegisterPassBase::unregisterPass() {
   assert(PassInfoMap && "Pass registered but not in map!");
   std::map<TypeInfo, PassInfo*>::iterator I =
-    PassInfoMap->find(PI->getTypeInfo());
+    PassInfoMap->find(PIObj.getTypeInfo());
   assert(I != PassInfoMap->end() && "Pass registered but not in map!");
 
   // Remove pass from the map...
@@ -365,10 +357,7 @@ void RegisterPassBase::unregisterPass(PassInfo *PI) {
   if (Listeners)
     for (std::vector<PassRegistrationListener*>::iterator
            I = Listeners->begin(), E = Listeners->end(); I != E; ++I)
-      (*I)->passUnregistered(PI);
-
-  // Delete the PassInfo object itself...
-  delete PI;
+      (*I)->passUnregistered(&PIObj);
 }
 
 //===----------------------------------------------------------------------===//
@@ -387,14 +376,14 @@ static std::map<const PassInfo *, AnalysisGroupInfo> *AnalysisGroupInfoMap = 0;
 //
 RegisterAGBase::RegisterAGBase(const std::type_info &Interface,
                                const std::type_info *Pass, bool isDefault)
-  : ImplementationInfo(0), isDefaultImplementation(isDefault) {
+  : RegisterPassBase(Interface, PassInfo::AnalysisGroup),
+    ImplementationInfo(0), isDefaultImplementation(isDefault) {
 
   InterfaceInfo = const_cast<PassInfo*>(Pass::lookupPassInfo(Interface));
-  if (InterfaceInfo == 0) {   // First reference to Interface, add it now.
-    InterfaceInfo =   // Create the new PassInfo for the interface...
-      new PassInfo("", "", Interface, PassInfo::AnalysisGroup, 0, 0);
-    registerPass(InterfaceInfo);
-    PIObj = 0;
+  if (InterfaceInfo == 0) {
+    // First reference to Interface, register it now.
+    registerPass();
+    InterfaceInfo = &PIObj;
   }
   assert(InterfaceInfo->getPassType() == PassInfo::AnalysisGroup &&
          "Trying to join an analysis group that is a normal pass!");
@@ -455,10 +444,11 @@ RegisterAGBase::~RegisterAGBase() {
         delete AnalysisGroupInfoMap;
         AnalysisGroupInfoMap = 0;
       }
-
-      unregisterPass(InterfaceInfo);
     }
   }
+  
+  if (InterfaceInfo == &PIObj)
+    unregisterPass();
 }
 
 
