@@ -167,7 +167,7 @@ Type* SymbolTable::remove(type_iterator Entry) {
 
 #if DEBUG_SYMBOL_TABLE
   dump();
-  std::cerr << " Removing Value: " << Result->getName() << "\n";
+  std::cerr << " Removing type: " << Entry->first << "\n";
 #endif
 
   tmap.erase(Entry);
