@@ -20,6 +20,7 @@
 #include "llvm/ADT/SCCIterator.h"
 #include "llvm/ADT/STLExtras.h"
 #include "llvm/Support/MathExtras.h"
+#include "llvm/Support/Visibility.h"
 #include <algorithm>
 #include <iostream>
 using namespace llvm;
@@ -32,6 +33,22 @@ using namespace llvm;
 
 AbstractTypeUser::~AbstractTypeUser() {}
 
+
+//===----------------------------------------------------------------------===//
+//                         Type PATypeHolder Implementation
+//===----------------------------------------------------------------------===//
+
+/// get - This implements the forwarding part of the union-find algorithm for
+/// abstract types.  Before every access to the Type*, we check to see if the
+/// type we are pointing to is forwarding to a new type.  If so, we drop our
+/// reference to the type.
+///
+Type* PATypeHolder::get() const {
+  const Type *NewTy = Ty->getForwardedType();
+  if (!NewTy) return const_cast<Type*>(Ty);
+  return *const_cast<PATypeHolder*>(this) = NewTy;
+}
+
 //===----------------------------------------------------------------------===//
 //                         Type Class Implementation
 //===----------------------------------------------------------------------===//
@@ -366,7 +383,7 @@ const Type *StructType::getTypeAtIndex(const Value *V) const {
 //===----------------------------------------------------------------------===//
 
 namespace {
-  struct PrimType : public Type {
+  struct VISIBILITY_HIDDEN PrimType : public Type {
     PrimType(const char *S, TypeID ID) : Type(S, ID) {}
   };
 }
@@ -1192,10 +1209,8 @@ static TypeMap<PointerValType, PointerType> PointerTypes;
 
 PointerType *PointerType::get(const Type *ValueType) {
   assert(ValueType && "Can't get a pointer to <null> type!");
-  // FIXME: The sparc backend makes void pointers, which is horribly broken.
-  // "Fix" it, then reenable this assertion.
-  //assert(ValueType != Type::VoidTy &&
-  //       "Pointer to void is not valid, use sbyte* instead!");
+  assert(ValueType != Type::VoidTy &&
+         "Pointer to void is not valid, use sbyte* instead!");
   PointerValType PVT(ValueType);
 
   PointerType *PT = PointerTypes.get(PVT);
@@ -1248,7 +1263,7 @@ void Type::removeAbstractTypeUser(AbstractTypeUser *U) const {
 }
 
 
-// refineAbstractTypeTo - This function is used to when it is discovered that
+// refineAbstractTypeTo - This function is used when it is discovered that
 // the 'this' abstract type is actually equivalent to the NewType specified.
 // This causes all users of 'this' to switch to reference the more concrete type
 // NewType and for 'this' to be deleted.
