@@ -1,5 +1,5 @@
 /* Encoding of types for Objective C.
-   Copyright (C) 1993, 1995, 1996, 1997, 1998, 2000, 2002, 2004
+   Copyright (C) 1993, 1995, 1996, 1997, 1998, 2000, 2002
    Free Software Foundation, Inc.
    Contributed by Kresten Krab Thorup
    Bitfield support by Ovidiu Predescu
@@ -79,34 +79,14 @@ Boston, MA 02111-1307, USA.  */
 /* Some ports (eg ARM) allow the structure size boundary to be
    selected at compile-time.  We override the normal definition with
    one that has a constant value for this compilation.  */
-#ifndef BITS_PER_UNIT
-#define BITS_PER_UNIT 8
-#endif
-#undef  STRUCTURE_SIZE_BOUNDARY
+#undef STRUCTURE_SIZE_BOUNDARY
 #define STRUCTURE_SIZE_BOUNDARY (BITS_PER_UNIT * sizeof (struct{char a;}))
 
 /* Some ROUND_TYPE_ALIGN macros use TARGET_foo, and consequently
-   target_flags.  Define a dummy entry here to so we don't die.
-   We have to rename it because target_flags may already have been
-   declared extern.  */
-#define target_flags not_target_flags
-static int __attribute__ ((__unused__)) not_target_flags = 0;
-
-/* Some ROUND_TYPE_ALIGN use ALTIVEC_VECTOR_MODE (rs6000 darwin).
-   Define a dummy ALTIVEC_VECTOR_MODE so it will not die.  */
-#undef ALTIVEC_VECTOR_MODE
-#define ALTIVEC_VECTOR_MODE(MODE) (0)
-
-
-/*  FIXME: while this file has no business including tm.h, this
-    definitely has no business defining this macro but it
-    is only way around without really rewritting this file,
-    should look after the branch of 3.4 to fix this.  */
-#define rs6000_special_round_type_align(STRUCT, COMPUTED, SPECIFIED)	\
-  ((TYPE_FIELDS (STRUCT) != 0						\
-    && DECL_MODE (TYPE_FIELDS (STRUCT)) == DFmode)			\
-   ? MAX (MAX (COMPUTED, SPECIFIED), 64)				\
-   : MAX (COMPUTED, SPECIFIED))
+   target_flags.  Define a dummy entry here to so we don't die.  */
+/* ??? FIXME: As of 2002-06-21, the attribute `unused' doesn't seem to
+   eliminate the warning.  */
+static int __attribute__ ((__unused__)) target_flags = 0;
 
 /*
   return the size of an object specified by type
@@ -780,23 +760,6 @@ objc_layout_structure (const char *type,
   layout->record_align = MAX (layout->record_align, STRUCTURE_SIZE_BOUNDARY);
 }
 
-/* APPLE LOCAL begin Macintosh alignment 2002-2-26 --ff */
-#ifdef RS6000_PIC_OFFSET_TABLE_REGNUM
-/* Ick, darwin.h doesn't work anymore...  Fix this please. */
-#undef ROUND_TYPE_ALIGN
-#define ROUND_TYPE_ALIGN(STRUCT, COMPUTED, SPECIFIED)			\
-  ((TREE_CODE (STRUCT) == RECORD_TYPE					\
-    || TREE_CODE (STRUCT) == UNION_TYPE					\
-    || TREE_CODE (STRUCT) == QUAL_UNION_TYPE)				\
-   && TARGET_ALIGN_NATURAL == 0                         		\
-   ? rs6000_special_round_type_align (STRUCT, COMPUTED, SPECIFIED)	\
-   : (TREE_CODE (STRUCT) == VECTOR_TYPE					\
-      && ALTIVEC_VECTOR_MODE (TYPE_MODE (STRUCT))) 			\
-   ? MAX (MAX ((COMPUTED), (SPECIFIED)), 128)          			 \
-   : MAX ((COMPUTED), (SPECIFIED)))
-#define TARGET_ALIGN_MAC68K 0
-#endif
-/* APPLE LOCAL end Macintosh alignment 2002-2-26 --ff */
 
 BOOL
 objc_layout_structure_next_member (struct objc_struct_layout *layout)
@@ -862,11 +825,9 @@ objc_layout_structure_next_member (struct objc_struct_layout *layout)
 #ifdef BIGGEST_FIELD_ALIGNMENT
   desired_align = MIN (desired_align, BIGGEST_FIELD_ALIGNMENT);
 #endif
-/* APPLE LOCAL begin Macintosh alignment 2002-2-26 --ff */
 #ifdef ADJUST_FIELD_ALIGN
-  desired_align = ADJUST_FIELD_ALIGN (type, desired_align, layout->prev_type == 0);
+  desired_align = ADJUST_FIELD_ALIGN (type, desired_align);
 #endif
-/* APPLE LOCAL end Macintosh alignment 2002-2-26 --ff */
 
   /* Record must have at least as much alignment as any field.
      Otherwise, the alignment of the field within the record
