@@ -54,7 +54,7 @@ struct llvm_exception {
 enum {
   ErrorException = 0,
   SJLJException  = 1,
-  CXXException   = 2,
+  CXXException   = 2
 };
 
 // Language independent exception handling API...
