@@ -12,6 +12,7 @@
 //===----------------------------------------------------------------------===//
 
 #include "BugDriver.h"
+#include "ToolRunner.h"
 #include "ListReducer.h"
 #include "llvm/Constant.h"
 #include "llvm/Instructions.h"
@@ -23,14 +24,21 @@
 #include "llvm/Analysis/Verifier.h"
 #include "llvm/Bytecode/Writer.h"
 #include "llvm/Support/CFG.h"
-#include "llvm/Support/ToolRunner.h"
 #include "llvm/Transforms/Scalar.h"
 #include "llvm/Transforms/Utils/Cloning.h"
 #include "llvm/Support/FileUtilities.h"
+#include "llvm/Support/CommandLine.h"
 #include <fstream>
 #include <set>
 using namespace llvm;
 
+namespace {
+  cl::opt<bool>
+  KeepMain("keep-main",
+           cl::desc("Force function reduction to keep main"),
+           cl::init(false));
+}
+
 namespace llvm {
   class ReducePassList : public ListReducer<const PassInfo*> {
     BugDriver &BD;
@@ -109,12 +117,20 @@ namespace llvm {
 }
 
 bool ReduceCrashingFunctions::TestFuncs(std::vector<Function*> &Funcs) {
+
+  //if main isn't present, claim there is no problem
+  if (KeepMain && find(Funcs.begin(), Funcs.end(), BD.getProgram()->getMainFunction()) == Funcs.end())
+    return false;
+
   // Clone the program to try hacking it apart...
   Module *M = CloneModule(BD.getProgram());
 
   // Convert list to set for fast lookup...
   std::set<Function*> Functions;
   for (unsigned i = 0, e = Funcs.size(); i != e; ++i) {
+    // FIXME: bugpoint should add names to all stripped symbols.
+    assert(!Funcs[i]->getName().empty() &&
+           "Bugpoint doesn't work on stripped modules yet PR718!");
     Function *CMF = M->getFunction(Funcs[i]->getName(),
                                    Funcs[i]->getFunctionType());
     assert(CMF && "Function not in module?!");
