@@ -232,7 +232,10 @@ static void DisambiguateGlobalSymbols(Module *M) {
   // mangler is used by the two code generators), but having symbols with the
   // same name causes warnings to be emitted by the code generator.
   Mangler Mang(*M);
-  for (Module::global_iterator I = M->global_begin(), E = M->global_end(); I != E; ++I)
+  // Agree with the CBE on symbol naming
+  Mang.markCharUnacceptable('.');
+  for (Module::global_iterator I = M->global_begin(), E = M->global_end();
+       I != E; ++I)
     I->setName(Mang.getValueName(I));
   for (Module::iterator  I = M->begin(),  E = M->end();  I != E; ++I)
     I->setName(Mang.getValueName(I));
@@ -860,7 +863,7 @@ bool BugDriver::debugCodeGenerator() {
     std::cout << "  gcc " << SharedObject << " " << TestModuleBC
               << ".s -o " << TestModuleBC << ".exe";
 #if defined (HAVE_LINK_R)
-    std::cout << "-Wl,-R.";
+    std::cout << " -Wl,-R.";
 #endif
     std::cout << "\n";
     std::cout << "  " << TestModuleBC << ".exe";
