@@ -20,6 +20,7 @@
 #include <iostream>
 #include <algorithm>
 #include <iomanip>
+#include <memory>
 
 using namespace llvm;
 
@@ -79,7 +80,7 @@ enum ArchiveOperation {
   QuickAppend,      ///< Quickly append to end of archive
   ReplaceOrInsert,  ///< Replace or Insert members
   DisplayTable,     ///< Display the table of contents
-  Extract,          ///< Extract files back to file system
+  Extract           ///< Extract files back to file system
 };
 
 // Modifiers to follow operation to vary behavior
@@ -483,7 +484,9 @@ void doDelete() {
   }
 
   // We're done editting, reconstruct the archive.
-  TheArchive->writeToDisk(SymTable,TruncateNames,Compression);
+  std::string errmsg;
+  if (!TheArchive->writeToDisk(SymTable,TruncateNames,Compression,&errmsg))
+    throw errmsg;
   if (ReallyVerbose)
     printSymbolTable();
 }
@@ -535,7 +538,9 @@ void doMove() {
   }
 
   // We're done editting, reconstruct the archive.
-  TheArchive->writeToDisk(SymTable,TruncateNames,Compression);
+  std::string errmsg;
+  if (!TheArchive->writeToDisk(SymTable,TruncateNames,Compression,&errmsg))
+    throw errmsg;
   if (ReallyVerbose)
     printSymbolTable();
 }
@@ -554,7 +559,9 @@ void doQuickAppend() {
   }
 
   // We're done editting, reconstruct the archive.
-  TheArchive->writeToDisk(SymTable,TruncateNames,Compression);
+  std::string errmsg;
+  if (!TheArchive->writeToDisk(SymTable,TruncateNames,Compression,&errmsg))
+    throw errmsg;
   if (ReallyVerbose)
     printSymbolTable();
 }
@@ -641,7 +648,9 @@ void doReplaceOrInsert() {
   }
 
   // We're done editting, reconstruct the archive.
-  TheArchive->writeToDisk(SymTable,TruncateNames,Compression);
+  std::string errmsg;
+  if (!TheArchive->writeToDisk(SymTable,TruncateNames,Compression,&errmsg))
+    throw errmsg;
   if (ReallyVerbose)
     printSymbolTable();
 }
