@@ -14,7 +14,6 @@
 
 #include "CLIDebugger.h"
 #include "llvm/Support/CommandLine.h"
-#include "llvm/Support/PluginLoader.h"
 #include "llvm/System/Signals.h"
 #include <iostream>
 
@@ -50,6 +49,7 @@ namespace {
 // main Driver function
 //
 int main(int argc, char **argv, char * const *envp) {
+  std::cout << "NOTE: llvm-db is known useless right now.\n";
   try {
     cl::ParseCommandLineOptions(argc, argv,
                                 " llvm source-level debugger\n");
