@@ -73,7 +73,7 @@ static cl::alias ExportDynamic("export-dynamic",
   cl::desc("Alias for -disable-internalize"));
 
 static cl::list<std::string> LoadableModules("load",
-  cl::value_desc("path to loadable optimization module"),
+  cl::value_desc("path"),
   cl::desc("Load an optimization module and run it"));
 
 // A utility function that adds a pass to the pass manager but will also add
@@ -102,7 +102,7 @@ void Optimize(Module* M) {
     Passes.add(createVerifierPass());
 
   // Add an appropriate TargetData instance for this module...
-  addPass(Passes, new TargetData("gccld", M));
+  addPass(Passes, new TargetData(M));
 
   // Often if the programmer does not specify proper prototypes for the
   // functions they are calling, they end up calling a vararg version of the
@@ -174,7 +174,7 @@ void Optimize(Module* M) {
     sys::DynamicLibrary dll(I->c_str());
     typedef void (*OptimizeFunc)(PassManager&,int);
     OptimizeFunc OF = OptimizeFunc(
-        dll.GetAddressOfSymbol("RunOptimizations"));
+        (intptr_t)dll.GetAddressOfSymbol("RunOptimizations"));
     if (OF == 0) {
       throw std::string("Optimization Module '") + *I +
         "' is missing the RunOptimizations symbol";
