@@ -9,7 +9,8 @@
 //
 // This file declares the types and data needed by ConfigLexer.l
 //
-//===------------------------------------------------------------------------===
+//===----------------------------------------------------------------------===//
+
 #ifndef LLVM_TOOLS_LLVMC_CONFIGLEXER_H
 #define LLVM_TOOLS_LLVMC_CONFIGLEXER_H
 
@@ -102,7 +103,7 @@ enum ConfigLexerTokens {
   TRUETOK,           ///< A boolean true value (true/yes/on)
   VERBOSE_SUBST,     ///< The substitution item %verbose%
   VERSION_TOK,       ///< The name "version" (and variants)
-  WOPTS_SUBST,       ///< The %WOpts% substitution
+  WOPTS_SUBST        ///< The %WOpts% substitution
 };
 
 extern ConfigLexerTokens Configlex();
