@@ -17,14 +17,13 @@
 #                   -norunningtests.
 #  -norunningtests  Do not run the Olden benchmark suite with
 #                   LARGE_PROBLEM_SIZE enabled.
-#  -noexternals     Do not run the external tests (for cases where povray
-#                   or SPEC are not installed)
 #  -nodejagnu       Do not run feature or regression tests
 #  -parallel        Run two parallel jobs with GNU Make.
 #  -release         Build an LLVM Release version
 #  -enable-llcbeta  Enable testing of beta features in llc.
 #  -disable-llc     Disable LLC tests in the nightly tester.
 #  -disable-jit     Disable JIT tests in the nightly tester.
+#  -disable-cbe     Disable C backend tests in the nightly tester.
 #  -verbose         Turn on some debug output
 #  -debug           Print information useful only to maintainers of this script.
 #  -nice            Checkout/Configure/Build with "nice" to reduce impact 
@@ -42,12 +41,15 @@
 #                   override the default.
 #  -ldflags         Next argument specifies that linker options that override
 #                   the default.
+#  -compileflags    Next argument specifies extra options passed to make when
+#                   building LLVM.
 #
 #  ---------------- Options to configure llvm-test ----------------------------
-#  -spec2000path    Path to the benchspec directory in the SPEC 2000 distro
-#  -spec95path      Path to the benchspec directory in the SPEC 95 distro.
-#  -povraypath      Path to the povray sources
-#  -namdpath        Path to the namd sources
+#  -extraflags      Next argument specifies extra options that are passed to
+#                   compile the tests.
+#  -noexternals     Do not run the external tests (for cases where povray
+#                   or SPEC are not installed)
+#  -with-externals  Specify a directory where the external tests are located.
 #
 # CVSROOT is the CVS repository from which the tree will be checked out,
 #  specified either in the full :method:user@host:/dir syntax, or
@@ -97,8 +99,6 @@ my $CVSCOOPT = "-APR";
 my $NICE = "";
 my $NODEJAGNU = 0;
 
-my $LLVMTESTCONFIGARGS = "";
-
 sub ReadFile {
   if (open (FILE, $_[0])) {
     undef $/;
@@ -304,12 +304,16 @@ while (scalar(@ARGV) and ($_ = $ARGV[0], /^[-+]/)) {
                              $CONFIGUREARGS .= " --disable-llc_diffs"; next; }
   if (/^-disable-jit$/)    { $PROGTESTOPTS .= " DISABLE_JIT=1";
                              $CONFIGUREARGS .= " --disable-jit"; next; }
+  if (/^-disable-cbe$/)    { $PROGTESTOPTS .= " DISABLE_CBE=1"; next; }
   if (/^-verbose$/)        { $VERBOSE = 1; next; }
   if (/^-debug$/)          { $DEBUG = 1; next; }
   if (/^-nice$/)           { $NICE = "nice "; next; }
   if (/^-f2c$/)            {
     $CONFIGUREARGS .= " --with-f2c=$ARGV[0]"; shift; next;
   }
+  if (/^-with-externals/)  { 
+    $CONFIGUREARGS .= " --with-externals=$ARGV[0]"; shift; next 
+  }
   if (/^-gnuplotscript$/)  { $PlotScriptFilename = $ARGV[0]; shift; next; }
   if (/^-templatefile$/)   { $Template = $ARGV[0]; shift; next; }
   if (/^-gccpath/)         { 
@@ -328,20 +332,14 @@ while (scalar(@ARGV) and ($_ = $ARGV[0], /^[-+]/)) {
   if (/^-ldflags/)         {
     $MAKEOPTS = "$MAKEOPTS LD.Flags=\'$ARGV[0]\'"; shift; next;
   }
+  if (/^-compileflags/)    {
+    $MAKEOPTS = "$MAKEOPTS $ARGV[0]"; shift; next;
+  }
+  if (/^-extraflags/)      {
+    $CONFIGUREARGS .= " --with-extra-options=\'$ARGV[0]\'"; shift; next;
+  }
   if (/^-noexternals$/)    { $NOEXTERNALS = 1; next; }
   if (/^-nodejagnu$/)      { $NODEJAGNU = 1; next; }
-  if (/^-spec2000path$/)   {
-    $LLVMTESTCONFIGARGS .= " --enable-spec2000=$ARGV[0]"; shift; next;
-  }
-  if (/^-spec95path$/)     {
-    $LLVMTESTCONFIGARGS .= " --enable-spec95=$ARGV[0]"; shift; next;
-  }
-  if (/^-povraypath$/)     {
-    $LLVMTESTCONFIGARGS .= " --enable-povray=$ARGV[0]"; shift; next;
-  }
-  if (/^-namdpath$/)       {
-    $LLVMTESTCONFIGARGS .= " --enable-namd=$ARGV[0]"; shift; next;
-  }
   print "Unknown option: $_ : ignoring!\n";
 }
 
@@ -453,7 +451,7 @@ $LOC = `utils/countloc.sh`;
 #
 if (!$NOCHECKOUT) {
   if ( $VERBOSE ) { print "CONFIGURE STAGE\n"; }
-  my $EXTRAFLAGS = "--enable-spec --with-objroot=.$LLVMTESTCONFIGARGS";
+  my $EXTRAFLAGS = "--enable-spec --with-objroot=.";
   system "(time -p $NICE ./configure $CONFIGUREARGS $EXTRAFLAGS) > $BuildLog 2>&1";
 
   if ( $VERBOSE ) { print "BUILD STAGE\n"; }
