@@ -17,10 +17,12 @@
 #include "TableGenBackend.h"
 #include <map>
 #include <vector>
+#include <string>
 
 namespace llvm {
 
 class RecordVal;
+class BitsInit;
 
 class CodeEmitterGen : public TableGenBackend {
   RecordKeeper &Records;
@@ -32,10 +34,8 @@ public:
 private:
   void emitMachineOpEmitter(std::ostream &o, const std::string &Namespace);
   void emitGetValueBit(std::ostream &o, const std::string &Namespace);
-  void emitInstrOpBits(std::ostream &o,
-                       const std::vector<RecordVal> &Vals,
-                       std::map<std::string, unsigned> &OpOrder,
-                       std::map<std::string, bool> &OpContinuous);
+  void reverseBits(std::vector<Record*> &Insts);
+  int getVariableBit(const std::string &VarName, BitsInit *BI, int bit);
 };
 
 } // End llvm namespace
